#!/usr/bin/env perl
###############################################################################
#    Webmin Sysstats - batch_install.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this program is designed to install webminstats modules without the webmin interface
# rem : the code is very similar to batch_graph.pl
###############################################################################

use strict;
use warnings;
use File::Basename;
use Getopt::Long;
use Pod::Usage;

use English qw(-no_match_vars);

##################################################################################
sub init_batch() {
	my ( $scriptname, $sysstats_dir, undef ) = fileparse($PROGRAM_NAME);
	chdir $sysstats_dir or die "can not chdir to $sysstats_dir: $ERRNO\n";

	# sysstats_dir should be something like /usr/libexec/webmin/sysstats
	push @INC, $sysstats_dir;         # webminstats dir
	push @INC, "$sysstats_dir/..";    # webmin dir

	my $webmin_module = basename($sysstats_dir);    # should be 'sysstats'

	use subs qw( &read_file &text );                # from web-lib.pl
	use subs qw( &graph_data );                     # from sysstats-lib.pl

	## no critic (ProhibitPackageVars)
	use vars
	  qw( %gconfig %config %text $module_config_directory $root_directory $module_name);

	use vars qw( $module_dir );                     # from sysstats-lib.pl

	# prepare for init_config
	# no critic (ProhibitNoWarnings)
	no warnings;
	$main::initial_process_id = $PID;
	use warnings;
	## use critic
	$ENV{'WEBMIN_CONFIG'} = '/etc/webmin' if ( !exists $ENV{'WEBMIN_CONFIG'} );
	$ENV{'WEBMIN_VAR'}    = '/var/webmin' if ( !exists $ENV{'WEBMIN_VAR'} );
	$ENV{'SCRIPT_NAME'} = "$webmin_module/$scriptname";
	$ENV{'SERVER_ROOT'} = $sysstats_dir;
	$ENV{'BATCH_MODE'}  = 'y';

	# call init_config
	## no critic (RequireBarewordIncludes)
	require 'sysstats-lib.pl';
	## use critic

	# fix some others vars after init_config
	$module_name             = $webmin_module;
	$module_config_directory = "$ENV{'WEBMIN_CONFIG'}/$module_name";
	$root_directory          = dirname($sysstats_dir);

	# read config files
	batch_readconf();

	return;
}

###############################################################################
# main
###############################################################################

init_batch();

# variables for program option parsing
my $help;
my $man;

## no critic (ProhibitUnlessBlocks)
unless (
	GetOptions(
		'help' => \$help,
		'man'  => \$man,
	)
  )
{
	pod2usage('bad syntax');
}
## use critic

# check options
if ($help) {
	pod2usage(1);
}
elsif ($man) {
	pod2usage( -verbose => 2 );
}
else {
	check_new_modules();
}

__END__

=head1 NAME

batch_install.pl

=head1 DESCRIPTION

this tool will install webminstats modules in a batch mode (no needs of running webmin).

=head1 SYNOPSIS

batch_install.pl

all tar.gz files on webminstat's modules directory will be checked and installed

=head1 USAGE

=over 4

=item copy the new/updated modules (as tar.gz) on webminstat's module directory

(/usr/libexec/webmin/sysstats/modules, or /usr/share/webmin/sysstats/modules)

=item run batch_install.pl

from the webminstat's directory, as root user.

=back

=head1 REQUIRED ARGUMENTS

none

=head1 OPTIONS

=over 4

=item help

show a summary help

=item man

show a full help

=back

=head1 DIAGNOSTICS

the output is in html (as it will be display in webmin interface)

the debug mode depends from webmin configuration

=head1 EXIT STATUS

0 if ok

=head1 CONFIGURATION

it depends upon webmin configuration (/etc/webmin/config) and webminstats configuration (/etc/webmin/sysstats/config)

=head1 DEPENDENCIES

webmin code (and lib) should be installed

=head1 INCOMPATIBILITIES

not known

=head1 BUGS AND LIMITATIONS

not known yet

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2006 by Eric Gerbier
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=head1 AUTHOR

Eric Gerbier

you can report any bug or suggest to gerbier@users.sourceforge.net

=cut

