#!/usr/bin/env perl
###############################################################################
#    Webmin Sysstats - clean_graph.pl
#
#    Webmin Sysstats
#    Copyright (C) 2001 by Eric Gerbier
#    Bug reports to:  gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this script is used to clean the graph directory
# which contain the png graph cache.

use strict;
use warnings;
use File::Basename;
use English qw(-no_match_vars);

use subs qw( &read_file &text );    # from web-lib.pl
use subs qw( &graph_data );         # from sysstats-lib.pl

## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $root_directory $module_name);

use vars qw( $module_dir );         # from sysstats-lib.pl
##################################################################################
sub init_batch() {
	my ( $scriptname, $sysstats_dir, undef ) = fileparse($PROGRAM_NAME);
	chdir $sysstats_dir or die "can not chdir to $sysstats_dir: $ERRNO\n";

	# sysstats_dir should be something like /usr/libexec/webmin/sysstats
	push @INC, $sysstats_dir;         # webminstats dir
	push @INC, "$sysstats_dir/..";    # webmin dir

	my $webmin_module = basename($sysstats_dir);    # should be 'sysstats'

	# prepare for init_config
	# no critic (ProhibitNoWarnings)
	no warnings;
	$main::initial_process_id = $PID;
	use warnings;
	## use critic
	$ENV{'WEBMIN_CONFIG'} = '/etc/webmin' if ( !exists $ENV{'WEBMIN_CONFIG'} );
	$ENV{'WEBMIN_VAR'}    = '/var/webmin' if ( !exists $ENV{'WEBMIN_VAR'} );
	$ENV{'SCRIPT_NAME'} = "$webmin_module/$scriptname";
	$ENV{'SERVER_ROOT'} = $sysstats_dir;
	$ENV{'BATCH_MODE'}  = 'y';

	# call init_config
	## no critic (RequireBarewordIncludes)
	require 'sysstats-lib.pl';
	## use critic

	# fix some others vars after init_config
	$module_name             = $webmin_module;
	$module_config_directory = "$ENV{'WEBMIN_CONFIG'}/$module_name";
	$root_directory          = dirname($sysstats_dir);

	# read config files
	batch_readconf();

	return;
}

###############################################################################
# main
###############################################################################

init_batch();

# second arg : the number of second to sleep before cleaning
my $delai = $ARGV[0] || 60;

# then the name of module to clean (optionnal)
my $module = $ARGV[1];    # for exemple : cpu-0.15.6

# wait
sleep $delai;

common_clear_cache($module);

exit 0;
