#!/usr/bin/perl -w
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: delete_mod.cgi 5153 2013-02-15 10:13:03Z gerbier $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# to clone a module
# called from edit_module.cgi
###############################################################################

use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

# from web-lib
## no critic (ProhibitPackageVars)
use vars qw( %text %in %access );
## use critic
use subs qw( &ReadParse );
use subs qw( &error_setup &error &text_error &ui_print_header &text );

# from systat-lib
use subs qw( &debug_sub_begin &debug_sub_end );

debug_sub_begin();
wrapper('ReadParse');
$access{'adddel'} or text_error('clone_ecannot');
wrapper( 'ui_print_header', undef, $text{'clone_title'}, undef, 'clone' );

my $cgi = basename($PROGRAM_NAME);
if ( ( !$in{'idx'} ) or ( !$in{'clone'} ) ) {

	# step 1 : display select form
	my @mod_list = get_mod_list(0);
	print wrapper( 'ui_form_start', $cgi );

	# source
	print 'src : ';
	if ( $in{'idx'} ) {
		print wrapper( 'ui_hidden', 'idx', $in{'idx'} );
		print "$in{'idx'}<br />";
	}
	else {
		my @tab;

		foreach my $mod (@mod_list) {
			my $module_name = $mod->{'module_name'};

			# forbid cloning a clone (bugfix 77)
			# next if ( is_clone($module_name) );

			push @tab, $module_name;
		}
		print wrapper( 'ui_select', 'idx', undef, \@tab );
	}

	# clone name
	print 'clone name : ' . wrapper( 'ui_textbox', 'clone', q{}, 15 ) . '<br>';

	# clone data ?
	print $text{'clone_want'}
	  . SPACE()
	  . wrapper( 'ui_yesno_radio', 'clone_data', 0, 2, 0 );

	# submit
	print wrapper( 'ui_form_end', [ [ undef, $text{word_ok} ] ] );
}

elsif ( !$in{'confirm'} ) {

	# step 2 : ask the user to confirm
	my %minfo = get_sysstats_module_info( $in{'idx'} );

	my $clone_name = $in{'clone'};

	# clean from unwanted characters : spaces
	$clone_name =~ s/\s//g;

	print ui_hr();
	print wrapper( 'ui_form_start', $cgi );
	print wrapper( 'ui_hidden',     'idx', $in{'idx'} );
	print wrapper( 'ui_hidden',     'clone', $clone_name );
	print wrapper( 'ui_hidden',     'clone_data', $in{'clone_data'} );
	my $desc = html_strong( $minfo{'module_name'} );

	print text( 'clone_rusure', $desc ), " => $clone_name<p>";
	print wrapper( 'ui_form_end', [ [ 'confirm', $text{word_clone} ] ] );
}
else {

	# step 3 : clone the module
	# clone_mod (sysstats-lib.pl) does not send any return code
	clone_mod( $in{'idx'}, $in{'clone'}, $in{'clone_data'} );

	print ui_hr();
	print $text{'clone_msg'}, "<p>\n";
	print "$in{'idx'} to new $in{'clone'}\n";
}

my_footer();
debug_sub_end();
