#!/usr/bin/perl -w
###############################################################################
#    Webmin Syststats - config.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# display a page with all modules status and links to configure them
# called from index.cgi
###############################################################################
use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars qw(%text $cb $tb %in);
use subs qw( &ui_print_header &read_file );

# from sysstats-lib.pl
use subs qw( &debug_sub_begin &debug_sub_end );
use vars qw( $module_dir );
## use critic

debug_sub_begin();

wrapper('ReadParse');
wrapper( 'ui_print_header', undef, $text{'title_configure_modules'},
	undef, 'config' );

# global actions (not on a module)
my ( @links, @titles, @icons );
add_icon( \@links, \@titles, \@icons, 'install_mod.cgi',
	$text{'configure_addnew'},
	'images/icon_plus.gif' );
add_icon( \@links, \@titles, \@icons, 'delete_mod.cgi', $text{'delete_title'},
	'images/icon_moins.gif' );
add_icon( \@links, \@titles, \@icons, 'clear_alarm.cgi', $text{'clear_alarm'},
	'images/sponge.gif' );
add_icon( \@links, \@titles, \@icons, 'clone_mod.cgi', $text{'clone_title'},
	'images/clonedata.gif' );
my_icons_table( \@links, \@titles, \@icons );

if ( exists $in{'idx'} ) {

	# change runstop for $module
	my $module = $in{'idx'};

	if ( is_module_installed($module) ) {
		my %minfo = get_sysstats_module_info($module);
		my %my_config;
		if ( read_module_config( \%my_config, $minfo{'config_file'} ) ) {

			# need to load module to access write_config sub
			unset_warnings();
			eval { require $minfo{'mod_lib'} };
			set_warnings();
			if ($EVAL_ERROR) {

				# error
				warning(
					"can not find perl lib $minfo{'mod_lib'} : $EVAL_ERROR");
			}
			else {
				mod_runstop( \%minfo, \%my_config );
			}
		}
		else {
			warning("can not read config file $minfo{'config_file'}");
		}

		# reload config with new runstop value
		jredirect('config.cgi');
	}
	else {
		text_error( 'bad_module_name', $module );
	}
}
else {

	# just display the list of modules

	print ui_hr();
	my @mod_list = get_mod_list(0);    # all modules
	if (@mod_list) {
		my @headings = (
			$text{'configure_module_name'}, $text{'configure_module_version'},
			$text{'configure_description'}, $text{'sample_rate'},
			$text{'configure_status'},      $text{'clone_of'},
			$text{'alarm'},                 $text{'action'},
		);
		print wrapper( 'ui_columns_start', \@headings );

		foreach my $mod (@mod_list) {

			my %my_text;
			my @columns;

			read_file( $mod->{'mod_lang'}, \%my_text );

			# name with link to configure
			push @columns,
			  html_link(
				'edit_module.cgi?idx=' . $mod->{'idx'},
				ucfirst lc $my_text{'module_title_index'}
			  );

			# version
			my $version = $mod->{'module_ver'};
			push @columns, display_module_version($version);

			# description
			push @columns,
			  html_link(
				'display.cgi?idx=' . $mod->{'idx'},
				$my_text{'module_description'}
			  );

			# sample rate
			my %my_config;
			read_module_config( \%my_config, $mod->{'config_file'} );
			push @columns, $my_config{'sample_rate'};

			# status
			my $status     = runstop_status( $mod, \%my_config );
			my $config_dir = $mod->{'config_dir'};
			my $is_alarm   = test_perm_alarm($config_dir);
			my $color      = alarm_color( $status, $is_alarm );
			push @columns,
			  html_link( basename($PROGRAM_NAME) . '?idx=' . $mod->{'idx'},
				html_color( $color, $status ) );

			# clones
			my $cloneof = ( is_clone( $mod->{'idx'} ) ) || '&nbsp;';
			push @columns, $cloneof;

			# alarms
			my $msg;
			if ($is_alarm) {
				$msg = html_link( 'clear_alarm.cgi?idx=' . $mod->{'idx'},
					get_alarm_msg($config_dir) );
			}
			else {
				$msg = $text{'ok'};
			}
			push @columns, html_color( $color, $msg );

			# actions
			push @columns, display_module_action( $mod, \%my_config );

			print wrapper( 'ui_columns_row', \@columns );
		}
		print ui_columns_end();
	}
}

my_footer();
debug_sub_end();
