#!/usr/bin/perl
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# to configure action for a module
###############################################################################
# see also sysstats-action-lib.pl

use strict;
use warnings;
use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

# from web-lib
## no critic (ProhibitPackageVars)
use vars qw( %text %in %access );
## use critic
use subs qw( &ReadParse );
use subs qw( &error_setup &error &text_error &ui_print_header &text );

# from sysstat-lib
use subs qw( &debug_sub_begin &debug_sub_end );

# operators in perl syntaxe
## no critic (ProhibitNoisyQuotes)
my @ops = ( '<', '==', '>', '!=', '<=', '>=', );
## use critic

# list of pre-defined actions
my @actions = get_list_actions();

my $cgi = basename($PROGRAM_NAME);
##########################################################################
sub display_actions($) {
	my $rh_codes = shift @_;

	my $nb = scalar keys %{$rh_codes};
	if ($nb) {

		# column title
		my @headings =
		  ( 'key', 'param', 'operator', 'value', 'action', 'arg', 'active' );
		print wrapper( 'ui_columns_start', \@headings );

		# datas
		foreach my $key ( sort keys %{$rh_codes} ) {
			my @columns;
			my @tab = action_decode( $rh_codes->{$key} );
			foreach my $val ( $key, @tab ) {
				push @columns, $val;
			}

			# link for modify
			push @columns,
			  html_link( "$cgi?idx=$in{'idx'}&mode=modify&key=$key", 'modify' );

			# link for delete
			push @columns,
			  html_link( "$cgi?idx=$in{'idx'}&mode=delete&key=$key", 'delete' );

			print wrapper( 'ui_columns_row', \@columns );
		}
		print wrapper('ui_columns_end');
	}
	else {
		print "no action<br>\n";
	}

	print ui_hr();

	# link for new
	print html_link( "$cgi?idx=$in{'idx'}&mode=new", 'add new action' );

	print ui_hr();
	return;
}
##########################################################################
sub new_action_form($) {
	my $rh_config = shift @_;    # config

	print ui_hr();
	print wrapper( 'ui_form_start',  $cgi );
	print wrapper( 'ui_hidden',      'idx', $in{'idx'} );
	print wrapper( 'ui_hidden',      'mode', 'newsave' );
	print wrapper( 'ui_table_start', 'new action', 'summary=new_action', 2 );

	# parameter
	my %params = list_params($rh_config);
	my @params = sort keys %params;
	print wrapper( 'ui_table_row', 'parameter',
		wrapper( 'ui_select', 'parameter_def', undef, \@params ) );

	# operator
	print wrapper( 'ui_table_row', 'operator',
		wrapper( 'ui_select', 'operator_def', undef, \@ops ) );

	# value
	print wrapper( 'ui_table_row', 'value',
		wrapper( 'ui_textbox', 'value_def', q{}, 15 ) );

	# action
	print wrapper( 'ui_table_row', 'action',
		wrapper( 'ui_select', 'action_def', undef, \@actions ) );

	# action parameter
	print wrapper( 'ui_table_row', 'argument',
		wrapper( 'ui_textbox', 'arg_def', q{}, 30 ) );

	# active
	print wrapper( 'ui_table_row', 'active',
		wrapper( 'ui_checkbox', 'active_def', 1, 'active', 1 ) );

	print wrapper('ui_table_end');
	print wrapper( 'ui_form_end',
		[ [ 'save', $text{'word_save'} ], [ 'cancel', $text{'cancel'} ] ] );

	return;
}
##########################################################################
sub new_action_save($$$) {
	my $rh_codes  = shift @_;
	my $minfo     = shift @_;
	my $rh_config = shift @_;    # config

	# test for cancel
	return unless $in{'save'};

	# build code line
	my $param  = $in{'parameter_def'};
	my $op     = $in{'operator_def'};
	my $value  = $in{'value_def'};
	my $action = $in{'action_def'};
	my $arg    = $in{'arg_def'} || SPACE();
	my $active = $in{'active_def'} || 0;

	my $line = action_encode( $param, $op, $value, $action, $arg, $active );

	# search for first available key
	my $index = action_first_free_key($rh_codes);

	$rh_config->{ 'action' . $index } = $line;

	# save config
	write_config( $minfo, $rh_config );

	return;
}
##########################################################################
sub modify_action_form($$) {
	my $rh_codes  = shift @_;
	my $rh_config = shift @_;    # config

	# called with
	my $key = $in{'key'};
	my @tab = action_decode( $rh_codes->{$key} );

	print ui_hr();
	print wrapper( 'ui_form_start', $cgi );
	print wrapper( 'ui_hidden',     'idx', $in{'idx'} );
	print wrapper( 'ui_hidden',     'mode', 'modsave' );
	print wrapper( 'ui_hidden',     'key', $key );
	print wrapper( 'ui_table_start', 'modify action', 'summary=modify_action',
		2 );

	# parameter
	my %params = list_params($rh_config);
	my @params = sort keys %params;
	print wrapper( 'ui_table_row', 'parameter',
		wrapper( 'ui_select', 'parameter_def', $tab[0], \@params ) );

	# operator
	print wrapper( 'ui_table_row', 'operator',
		wrapper( 'ui_select', 'operator_def', $tab[1], \@ops ) );

	# value
	print wrapper( 'ui_table_row', 'value',
		wrapper( 'ui_textbox', 'value_def', $tab[2], 15 ) );

	# action
	print wrapper( 'ui_table_row', 'action',
		wrapper( 'ui_select', 'action_def', $tab[3], \@actions ) );

	# action parameter
	print wrapper( 'ui_table_row', 'argument',
		wrapper( 'ui_textbox', 'arg_def', $tab[4], 30 ) );

	# active
	my $check = ( $tab[5] ) ? 1 : 0;
	print wrapper( 'ui_table_row', 'active',
		wrapper( 'ui_checkbox', 'active_def', 1, 'active', $check ) );

	print wrapper('ui_table_end');
	print wrapper( 'ui_form_end',
		[ [ 'save', $text{'word_save'} ], [ 'cancel', $text{'cancel'} ] ] );

	return;
}
##########################################################################
sub modify_action_save($$$) {
	my $rh_codes  = shift @_;
	my $minfo     = shift @_;
	my $rh_config = shift @_;    # config

	# test for cancel
	return unless $in{'save'};

	# build code line
	my $param  = $in{'parameter_def'};
	my $op     = $in{'operator_def'};
	my $value  = $in{'value_def'};
	my $action = $in{'action_def'};
	my $arg    = $in{'arg_def'} || SPACE();
	my $active = $in{'active_def'} || 0;

	my $line = action_encode( $param, $op, $value, $action, $arg, $active );

	# replace code
	my $key = $in{'key'};
	$rh_config->{$key} = $line;

	# save config
	write_config( $minfo, $rh_config );
	return;
}
##########################################################################
sub delete_action($$) {
	my $minfo     = shift @_;
	my $rh_config = shift @_;    # config

	my $key = $in{'key'};
	delete $rh_config->{$key};

	# save config
	write_config( $minfo, $rh_config );
	return;

}
##########################################################################
sub check_actions() {
	return;
}
##########################################################################

debug_sub_begin();
wrapper( 'ui_print_header', undef, $text{'action'}, undef, 'action' );
ReadParse();

error_setup( $text{'action_err'} );

$access{'configure'} or text_error('action_ecannot');

if ( !exists $in{'idx'} ) {
	text_error('action_ecannot');
}

my %minfo = get_sysstats_module_info( $in{'idx'} );

# load config file
my %my_config;
if ( !read_module_config( \%my_config, $minfo{'config_file'} ) ) {
	error( text( 'ecannot_readconfig', $minfo{'config_file'} ) );
}

# debug
#foreach my $key ( keys %my_config) {
#	print "$key : $my_config{$key}<br>\n";
#}
#print "<hr>";

# load module code
eval { require $minfo{'mod_lib'} };
if ($EVAL_ERROR) {
	warning("can not find perl lib $minfo{'mod_lib'} : $EVAL_ERROR");
	my_footer( $in{'idx'} );
}
my %codes  = action_load( \%my_config );
my %params = list_params( \%my_config );

if ( !%params ) {

	# if empty list, this module does not allow actions
	error( $text{'action_no'} );
	my_footer( $in{'idx'} );
	exit;
}

# quick navigation to others modules
## no critic (ProhibitEscapedMetacharacters)
my $cgi2 = $cgi;
$cgi2 =~ s/\.cgi$//;
navigation( $in{idx}, $cgi2, q{}, 1 );

# todo
# change order
# check ?

## no critic (ProhibitCascadingIfElse)
if ( exists $in{'mode'} ) {
	if ( $in{'mode'} eq 'new' ) {

		# new form
		new_action_form( \%my_config );
		my_footer( $in{'idx'} );
		exit;
	}
	elsif ( $in{'mode'} eq 'newsave' ) {

		# save new action
		new_action_save( \%codes, \%minfo, \%my_config );
	}
	elsif ( $in{'mode'} eq 'modify' ) {

		# modify form
		modify_action_form( \%codes, \%my_config );
		my_footer( $in{'idx'} );
		exit;
	}
	elsif ( $in{'mode'} eq 'modsave' ) {

		modify_action_save( \%codes, \%minfo, \%my_config );

	}
	elsif ( $in{'mode'} eq 'delete' ) {
		delete_action( \%minfo, \%my_config );
	}
	else {
		warning("unknown command $in{'mode'}");
	}
	## use critic

	# reload codes
	%codes = action_load( \%my_config );
}

# display
display_actions( \%codes );

my_footer( $in{'idx'} );
debug_sub_end();
