#!/usr/bin/perl -w
###############################################################################
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# allow to configure the list of module which can be display by display_all (panorama)
###############################################################################

use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in $in %tconfig );

use subs qw( &ui_print_header &read_file &error &text_error &text &tempname );
use subs qw( &ReadParse );

# from sysstats-lib.pl
use subs qw(&debug_sub_begin &debug_sub_end &debug);
use vars qw( $module_dir $graph_dir );
## use critic

debug_sub_begin();
wrapper('ReadParse');

my $COMMA = COMMA();
my $cgi   = basename($PROGRAM_NAME);

# just display running modules
my @mod_list = get_mod_list(1);

if ( exists $in{'cancel'} ) {
	myredirect($cgi);
}
elsif ( exists $in{'modules'} ) {

	# this is the result of the module selection form
	my @list;
	foreach my $minfo (@mod_list) {

		# module name with version
		my $module = $minfo->{'module_name'};

		if ( exists $in{$module} ) {

			# ok selected
			push @list, $module;
		}
	}
	my $arg = join $COMMA, @list;

	# sauvegarde ?
	if ( exists $in{'save'} ) {

		# set global config
		$config{'panorama_modules'} = $arg;

		# save config
		write_file_hash( $module_config_directory . '/config', \%config );
	}

	# call display_all with the list of checked modules
	myredirect("display_all.cgi?modules=$arg");
}
else {

	# display selection form
	wrapper( 'ui_print_header', undef, $text{'panorama_modules'},
		undef, 'config_all' );
	print ui_hr();
	print wrapper( 'ui_form_start', $cgi );
	print wrapper( 'ui_table_start', 'panorama configuration', undef, 8 );

	# check for global config
	my $flag_config = 0;
	my %list_mod;
	if ( exists $config{'panorama_modules'} ) {

		# read saved config
		$flag_config = 1;
		my @list = split $COMMA, $config{'panorama_modules'};
		foreach my $l (@list) {
			$list_mod{$l} = 1;
		}
	}

	foreach my $minfo (@mod_list) {

		my %my_config;
		if ( !read_module_config( \%my_config, $minfo->{'config_file'} ) ) {
			text_error( 'ecannot_readconfig', $minfo->{'config_file'} );
		}

		# module name with version
		my $module = $minfo->{'module_name'};

		my $checked;
		my $EMPTY   = EMPTY();
		my $CHECKED = 'checked';

		# control module display
		if ($flag_config) {

			# from saved config
			$checked = ( exists $list_mod{$module} ) ? $CHECKED : $EMPTY;
		}
		else {

			# no display for stopped modules
			$checked =
			  ( mod_check_runstop( $minfo, \%my_config ) ) ? $EMPTY : $CHECKED;
		}

		print wrapper( 'ui_table_row',
			wrapper( 'ui_checkbox', $module, 1, undef, $checked ), $module );
	}
	print wrapper('ui_table_end');

# todo : add buttons to select all, unselect all, toggle
# but the cgi which receive the form should display it to allow others change
# and not redirect to display_all ...
#print "<td><input type=submit name=\"select_all\" value=\"$text{'select_all'}\"></td>";
#print "<td><input type=submit name=\"unselect_all\" value=\"$text{'unselect_all'}\"></td>";

	print wrapper( 'ui_hidden', 'modules', 1 );
	print wrapper(
		'ui_form_end',
		[
			[ undef,    $text{'display'} ],
			[ 'save',   $text{'word_save'} ],
			[ 'cancel', $text{'cancel'} ]
		]
	);

	my_footer();
}

debug_sub_end();
