#!/usr/bin/perl -w
###############################################################################
#    Webmin Sysstats - debug.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# call the aquisition script
# and display the result
###############################################################################

use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars qw( %gconfig %config %text $module_config_directory $cb $tb %in $in );

use subs qw( &ui_print_header &read_file &error &text &tempname );
use subs qw( &ReadParse );

# from sysstats-lib.pl
use subs qw( &debug_sub_begin &debug_sub_end );
## use critic

###############################################################################

debug_sub_begin();
wrapper('ReadParse');

wrapper( 'ui_print_header', undef, 'debug', undef, 'debug' );

# default mode is the debug mode
my $def_mode = 'y';

# test if called from it-self
if ( exists $in{'idx'} ) {

	# a button was pressed
	if ( $in{'idx'} eq 'debug' ) {

		# set debug mode
		$ENV{DEBUG_MODE} = 'y';
	}
	elsif ( $in{'idx'} eq 'normal' ) {
		delete $ENV{DEBUG_MODE} if exists $ENV{DEBUG_MODE};
	}
	else {

		# default
		$ENV{DEBUG_MODE} = $def_mode;
	}
}
else {

	# default mode
	$ENV{DEBUG_MODE} = $def_mode;
}

print ui_hr();

my $script =
  aquisition_script();    # get path and name to general aquisition script

# execute and display the script
my $pipe = read_pipe_stream($script);
while ( my $ligne = <$pipe> ) {
	display_html($ligne);
}
## no critic (RequireCheckedSyscalls,RequireCheckedClose)
close $pipe;

print ui_hr();

# display the buttons
# to allow run in debug or normal mode
my $cgi = basename($PROGRAM_NAME);

# debug mode
print wrapper( 'ui_form_start', $cgi );
print wrapper( 'ui_form_end', [ [ 'idx', 'debug' ] ] );

# normal mode
print wrapper( 'ui_form_start', $cgi );
print wrapper( 'ui_form_end', [ [ 'idx', 'normal' ] ] );

my_footer();
debug_sub_end();
