#!/usr/bin/perl -w
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# to remove a module
# called from edit_module.cgi
###############################################################################

use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

# from web-lib
## no critic (ProhibitPackageVars)
use vars qw( %text %in %access );
## use critic
use subs qw( &ReadParse );
use subs qw( &error_setup &error &text_error &ui_print_header &text );

# from systat-lib
use subs qw( &debug_sub_begin &debug_sub_end );

debug_sub_begin();
wrapper('ReadParse');
$access{'adddel'} or text_error('delete_ecannot');
wrapper( 'ui_print_header', undef, $text{'delete_title'}, undef, 'delete' );

my $cgi = basename($PROGRAM_NAME);
if ( !$in{'idx'} ) {

	# step one : display select form
	##########

	my @mod_list = get_mod_list(0);
	print wrapper( 'ui_form_start', $cgi );
	print $text{'module_choice'} . SPACE();
	my @mods;
	foreach my $mod (@mod_list) {
		my $module_name = $mod->{'module_name'};
		push @mods, $module_name;
	}
	print wrapper( 'ui_select', 'idx', undef, \@mods );
	print wrapper( 'ui_form_end',
		[ [ 'save', $text{'word_ok'} ], [ 'cancel', $text{'cancel'} ] ] );
}
elsif ( $in{'cancel'} ) {
	jredirect('config.cgi');
}
elsif ( $in{'confirm'} ) {

	# step 3 : really delete module
	##########

	# remove_module (sysstats-lib.pl) does not send any return code
	my %minfo = get_sysstats_module_info( $in{'idx'} );
	remove_module( \%minfo, 0 );

	print "<hr>\n";
	print $text{'delete_msg'}, "<p>\n";
	print "<ul>\n";

	my $pwd = getcwd();
	$pwd .= $minfo{'mod_dir'};

	print text( 'delete_desc', html_strong( $minfo{'module_name'} ),
		"<tt>$pwd</tt>" );
	print "</ul><p>\n";
}
else {

	# step 2 : confirmation form
	##########

	my $module = $in{'idx'};
	if ( is_module_installed($module) ) {

		# first ask the user to confirm
		my %minfo = get_sysstats_module_info($module);

		my $is_cloned_by = is_cloned($module);
		if ($is_cloned_by) {

			# do not allow to delete a cloned module
			print html_strong(
				text( 'delete_cloned', $module, $is_cloned_by ) );
		}
		else {

			# display confirm form
			print ui_hr();
			print wrapper( 'ui_form_start', $cgi );
			print wrapper( 'ui_hidden', 'idx', $module );
			my $desc = html_strong( $minfo{'module_name'} );

			print text( 'delete_rusure', $desc );
			print wrapper(
				'ui_form_end',
				[
					[ 'confirm', $text{'word_delete'} ],
					[ 'cancel',  $text{'cancel'} ]
				]
			);
		}
	}
	else {
		text_error( 'bad_module_name', $module );
	}
}

my_footer();
debug_sub_end();
