#!/usr/bin/perl
###############################################################################
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# common code for display.cgi and display_custom.cgi
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars qw( %gconfig %config %text $module_config_directory $cb $tb %in $in );

use subs qw( &ui_print_header &read_file &error &text_error &text &tempname );
use subs qw( &ReadParse );

# from sysstats-lib.pl
use subs qw(&debug_sub_begin &debug_sub_end);
use vars qw( $graph_dir );
## use critic

###############################################################################
sub display_page_norefresh($$) {
	my $cgi    = shift @_;
	my $allopt = shift @_;

	# link to change refresh/norefresh
	if ( exists $in{'norefresh'} ) {
		print li(
			html_link( "$cgi?idx=$in{idx}${allopt}", $text{'display_refresh'} )
		);
	}
	else {
		print li(
			html_link(
				"$cgi?idx=$in{idx}&norefresh=1${allopt}",
				$text{'display_norefresh'}
			)
		);
	}
	return;
}
###############################################################################
sub display_page_internal($$$) {
	my $cgi     = shift @_;
	my $opt     = shift @_;
	my $display = shift @_;

	if ( exists $in{internal} ) {
		print li(
			html_link(
				"$cgi?idx=$in{idx}${opt}${display}",
				$text{'display_graph'}
			)
		);
	}
	else {
		# internals
		print li(
			html_link(
				"$cgi?idx=$in{idx}${opt}${display}&internal=1",
				$text{'display_internal'}
			)
		);
	}
	return;
}
###############################################################################
sub display_page_next($$$) {
	my $my_config = shift @_;
	my $cgi       = shift @_;
	my $allopt    = shift @_;

	if ( exists $my_config->{'graph_type'} ) {
		my $gtype =
		  ( $my_config->{'graph_type'} + 1 ) % $my_config->{'nb_graph'};

		# overload with cgi arg
		if ( exists $in{'gtype'} ) {
			$gtype = ( $in{'gtype'} + 1 ) % $my_config->{'nb_graph'};
		}
		my $title = $text{'display_next_graph'};
		my $name;
		if ( exists $text{ 'module_index_title' . $gtype } ) {
			$name = $text{ 'module_index_title' . $gtype };
		}
		else {
			$name = 'next';
		}
		print li(
			html_link(
				"$cgi?idx=$in{idx}${allopt}&nocache=1&gtype=$gtype", $name
			)
		);
	}
	return;
}
###############################################################################
sub display_page_other($$$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $cgi       = shift @_;

	if ( $minfo->{'module_multi_display'} ) {
		print '<li>others : ';

		# current display
		my $current = $in{'opt'};
		$current =~ s/display//;

		# like network module
		my $i     = 1;
		my $COMMA = COMMA();
		while ( exists $my_config->{ 'display' . $i } ) {
			my ( $net_num, $sub_display, $title_idx ) =
			  split $COMMA, $my_config->{ 'display' . $i };

			# test for runstop
			if (    ( opt_runstop( $net_num, $my_config ) == 0 )
				and ( $i ne $current ) )
			{
				my $title = $my_config->{ 'module_index_title' . $title_idx };
				print html_link( "$cgi?idx=$in{idx}&opt=display$i", $title )
				  . '&nbsp;&nbsp;';
			}
			$i++;
		}
		print '</li>';
	}
	return;
}
###############################################################################
1;
