#!/usr/bin/perl -w
###############################################################################
#    Webmin Sysstats - display_all.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# display all graph modules on one page
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in $in %tconfig );

use subs qw( &header &read_file &error &text_error &text &tempname );
use subs qw( &ReadParse &icons_table );

# from sysstats-lib.pl
use subs qw(&debug_sub_begin &debug_sub_end &debug);
use vars qw( $module_dir $graph_dir );
## use critic

my $cgi = basename($PROGRAM_NAME);

# cgi parameters
my @opts = ( 'internal', 'legend', 'norefresh' );
my $COMMA = COMMA();
######################################################################################
sub analyse_arg() {

	# analyse result of select_period_form form
	if ( exists $in{'reset'} ) {

		# redirect without any parameters
		clear_cache( EMPTY() );
		myredirect($cgi);
	}
	elsif ( !exists $in{'custom'} ) {

		# no arg from url

		if ( exists $config{'panorama_period'} ) {

			# get default from general config file

			my ( $delai, $scale ) = period2comp( $config{'panorama_period'} );
			$in{'custom'}  = $delai;
			$in{'periode'} = $scale;
		}
		else {

			# default is one hour
			$in{'custom'}  = 1;
			$in{'periode'} = 'hour';
		}
	}
	else {

		# we have a result of the change period form

		# test save button
		if ( exists $in{'save'} ) {

			# the config file format is like : 3d
			# (only the first letter)
			$config{'panorama_period'} = $in{'custom'} . substr $in{'periode'},
			  0,
			  1;
			write_file_hash( $module_config_directory . '/config', \%config );
		}

		# customized display
		clear_cache( EMPTY() );
	}
	return;
}
######################################################################################
# display legend selector form
sub select_legend() {

	my $key = 'legend';

	my ( $legend_opt, $legend_text );
	if ( ( defined $in{$key} ) and ( $in{$key} ) ) {
		$legend_opt  = q{};
		$legend_text = $text{'display_nolegend'};
	}
	else {

		# default
		$legend_opt  = "?$key=1";
		$legend_text = $text{'display_legend'};

		# for graph display
		$ENV{'WEBMINSTAT_NO_LEGEND'} = 'y';
	}

	my $opt = get_opts( $key, \@opts );
	print li( html_link( "${cgi}${legend_opt}$opt", $legend_text ) );
	return;
}
######################################################################################
# display data or internal selector
sub select_datatype() {

	my $key = 'internal';

	my ( $internal_opt, $internal_text );
	if ( ( defined $in{$key} ) and ( $in{$key} ) ) {
		$internal_opt  = q{};
		$internal_text = $text{'display_graph'};

		# for sysstats-rrd-lib, gprint_param sub
		$ENV{'WEBMINSTAT_INTERNALS'} = 'y';
	}
	else {

		# default
		$internal_opt  = "?$key=1";
		$internal_text = $text{'display_internal'};
	}
	my $opt = get_opts( $key, \@opts );
	print li( html_link( "${cgi}${internal_opt}$opt", $internal_text ) );

	return;
}
######################################################################################
sub select_refresh() {

	my $key = 'norefresh';
	my $opt = get_opts( $key, \@opts );

	# refresh/norefresh
	if ( ( exists $in{$key} ) and ( $in{$key} ) ) {
		print li( html_link( "$cgi?$opt", $text{'display_refresh'} ) );
	}
	else {
		print li( html_link( "$cgi?$key=1$opt", $text{'display_norefresh'} ) );
	}
	return;
}
######################################################################################
sub graph_multidisplay_module($$$$$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $r_olinks  = shift @_;
	my $r_otitles = shift @_;
	my $r_oicons  = shift @_;

	my $module = $minfo->{'module_name'};

	# like network module
	my $i = 1;
	while ( exists $my_config->{ 'display' . $i } ) {
		my ( $net_num, $sub_display, $title_idx ) =
		  split $COMMA, $my_config->{ 'display' . $i };

		# test for runstop
		if ( opt_runstop( $net_num, $my_config ) == 0 ) {
			push @{$r_otitles},
			  $my_config->{ 'module_index_title' . $title_idx };
			push @{$r_olinks},
			  'display.cgi?idx=' . $minfo->{'idx'} . '&opt=display' . $i;

			if ( exists $ENV{'WEBMINSTAT_INTERNALS'} ) {
				$minfo->{'tmpgname'} =
				  build_tmpgname( $module . $i . '_internal' );
			}
			else {
				$minfo->{'tmpgname'} = build_tmpgname( $module . $i );
			}
			my $graph = build_graph_name( $minfo, 'custom' );
			my $graph_stamped = $graph . png_stamp();

			if ( !-r $graph_stamped ) {

				# build only if cache is empty
				if ( exists $ENV{'WEBMINSTAT_INTERNALS'} ) {
					graph_internal_data( $minfo, $my_config, "display$i" );
				}
				else {
					graph_data( $minfo, $my_config, "display$i" );
				}
				link $graph, $graph_stamped;
			}
			push @{$r_oicons}, $graph_stamped;
		}
		else {
			debug("stopped $net_num : no display");
		}
		$i++;
	}
	return;
}
######################################################################################
sub graph_classic_module($$$$$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $r_olinks  = shift @_;
	my $r_otitles = shift @_;
	my $r_oicons  = shift @_;

	my $module = $minfo->{'module_name'};

	# "normal" modules : cpu and all others
	if ( $my_config->{'module_title_index'} ) {
		push @{$r_otitles}, $my_config->{'module_title_index'};
	}
	else {
		push @{$r_otitles}, $text{'module_title_index'};
	}
	push @{$r_olinks}, 'display.cgi?idx=' . $minfo->{'idx'};

	if ( exists $ENV{'WEBMINSTAT_INTERNALS'} ) {
		$minfo->{'tmpgname'} = build_tmpgname( $module . '_internal' );
	}
	else {
		$minfo->{'tmpgname'} = build_tmpgname($module);
	}
	my $graph = build_graph_name( $minfo, 'custom' );
	my $graph_stamped = $graph . png_stamp();

	if ( !-r $graph_stamped ) {

		# build only if cache is empty
		if ( exists $ENV{'WEBMINSTAT_INTERNALS'} ) {
			graph_internal_data( $minfo, $my_config );
		}
		else {
			graph_data( $minfo, $my_config );
		}
		link $graph, $graph_stamped;
	}
	push @{$r_oicons}, $graph_stamped;

	return;
}
######################################################################################
sub graph_module($$$$$$$) {
	my $flag_arg    = shift @_;
	my $flag_config = shift @_;
	my $inter       = shift @_;
	my $list_mod    = shift @_;
	my $r_olinks    = shift @_;
	my $r_otitles   = shift @_;
	my $r_oicons    = shift @_;

	my $wanted_sizex = 200;
	my $wanted_sizey = 100;
	my ( $real_sizex, $real_sizey );
	my @mod_list = get_mod_list(1);    # running modules
	foreach my $minfo (@mod_list) {
		my %my_config;
		if ( !read_module_config( \%my_config, $minfo->{'config_file'} ) ) {
			text_error( 'ecannot_readconfig', $minfo->{'config_file'} );
		}

		# module name with version
		my $module = $minfo->{'module_name'};

		# control on modules to display
		# 1 : cgi arg
		# 2 : configuration file
		# 3 : by default all running

		if ($flag_arg) {
			next unless ( exists $list_mod->{$module} );
		}
		elsif ($flag_config) {
			next unless ( exists $list_mod->{$module} );
		}
		else {

			# no display for stopped modules
			next if ( mod_check_runstop( $minfo, \%my_config ) );
		}

		if ( !read_file( $minfo->{'mod_lang'}, \%text ) ) {
			text_error( 'ecannot_readlang', $minfo->{'mod_lang'} );
		}

		# to avoid warnings on redefined subs
		clear_redefine();

		unset_warnings();
		require $minfo->{'mod_lib'};
		set_warnings();

		#my %defaults;

		$minfo->{'graph_sizex'} = $wanted_sizex;
		$minfo->{'graph_sizey'} = $wanted_sizey;

		# to be used in sysstats-lib::graph_common
		$minfo->{'custom_period'} = $inter;

		# check for module controled display
		if ( $minfo->{'module_multi_display'} ) {

			graph_multidisplay_module( $minfo, \%my_config, $r_olinks,
				$r_otitles, $r_oicons );
		}
		else {

			graph_classic_module( $minfo, \%my_config, $r_olinks, $r_otitles,
				$r_oicons );
		}

		# real sized returned by sysstats-lib::graph_common
		$real_sizex = $minfo->{'graph_sizex'};
		$real_sizey = $minfo->{'graph_sizey'};
	}
	return ( $real_sizex, $real_sizey );
}
######################################################################################
# main
######################################################################################

debug_sub_begin();
wrapper('ReadParse');

analyse_arg();

my $inter = get_period_scale( $in{'periode'} ) * $in{'custom'};

wrapper(
	'ui_print_header', undef, $text{'all_modules'}, undef,
	'display_all',     undef, undef,                undef,
	undef,             get_meta()
);

print ui_hr();

# link to module configuration
print "<ul>\n";
print li( html_link( 'config_display_all.cgi', $text{'panorama_modules'} ) );

# form to change period
print '<li>';
select_period_form($cgi);
print '</li>';

select_legend();      # legend
select_datatype();    # data or internals
select_refresh();     # refresh or not
print "</ul>\n";
print ui_hr();

# clear cache
clear_cache();

my @mod_list = get_mod_list(1);    # running modules

my $flag_arg    = 0;               # is set from config form ?
my $flag_config = 0;               # is set from config file
my %list_mod;                      # list of modules to display

# build the list of modules if asked necessary
if ( exists $in{'modules'} ) {

	# call from config_display_all.cgi with a list of module to display
	$flag_arg = 1;
	my @list = split $COMMA, $in{'modules'};
	foreach my $l (@list) {
		$list_mod{$l} = 1;
	}
}
elsif ( exists $config{'panorama_modules'} ) {

	# use the saved configuration list of modules
	$flag_config = 1;
	my @list = split $COMMA, $config{'panorama_modules'};
	foreach my $l (@list) {
		$list_mod{$l} = 1;
	}
}

# build the icons and links
my ( @olinks, @otitles, @oicons );
my ( $real_sizex, $real_sizey ) = graph_module(
	$flag_arg, $flag_config, $inter, \%list_mod,
	\@olinks,  \@otitles,    \@oicons
);

my $noicons_save = 0;
if ( $tconfig{'noicons'} ) {

	# save old value
	$noicons_save = $tconfig{'noicons'};

	# change state for generate_icon to force display icons
	$tconfig{'noicons'} = 0;
}

# Display modules graphs as icons
graph_table( \@olinks, \@otitles, \@oicons, 3, undef, $real_sizex,
	$real_sizey );

# restore value
$tconfig{'noicons'} = $noicons_save if ($noicons_save);

# clean graph cache
delayed_clear_cache();

print "<hr>\n";

my_footer();
debug_sub_end();
