#!/usr/bin/perl
###############################################################################
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# in charge to display custom graphs
#
# ps : this cgi do not use perl -w
# because a strange problem with the animation mode
#
# rem : the code is very close from display.cgi
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
require './display-lib.pl';
## use critic

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars qw( %gconfig %config %text $module_config_directory $cb $tb %in $in );

use subs qw( &ui_print_header &read_file &error &text_error &text &tempname );
use subs qw( &ReadParse );

# from sysstats-lib.pl
use subs qw(&debug_sub_begin &debug_sub_end);
use vars qw( $graph_dir );
## use critic

my %scale = (
	'hourly'  => 'hour',
	'daily'   => 'day',
	'weekly'  => 'week',
	'monthly' => 'month',
	'yearly'  => 'year',
);

my %r_scale = reverse %scale;
my $flag_manip;
###############################################################################
# display a graph (hour, ...)
sub display_image ($$$$) {
	my $minfo = shift @_;
	my $stamp = shift @_;
	my $size  = shift @_;
	my $type  = shift @_;    # custom

	# we use the stamped graph
	my $graph_name = build_graph_name( $minfo, $type ) . $stamp;
	my $img =
	    '<a name='
	  . $type
	  . '><img src="'
	  . $graph_name
	  . '" alt="'
	  . $text{$type} . '" '
	  . $size . '></a>';

	my @tdtags  = ('align=center');
	my @columns = ($img);
	print wrapper( 'ui_columns_row', \@columns, \@tdtags );
	return;
}
###############################################################################
sub display_page_links($$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;

	debug_sub_begin();

	my $EMPTY = EMPTY();

	my $cgi = basename($PROGRAM_NAME);

	# options
	#########

	# should take care of $in{'opt'} !!!
	my $opt = ( exists $in{'opt'} ) ? "&opt=$in{'opt'}" : $EMPTY;

	# same for display ( a number, not the text associated to it)
	my $real_display = ( exists $in{'display'} ) ? $in{'display'} : '0';
	my $display =
	  ( exists $in{'display'} ) ? "&display=$in{'display'}" : $EMPTY;

	# same for internals
	my $internal =
	  ( exists $in{'internal'} ) ? "&internal=$in{'internal'}" : $EMPTY;

	my $allopt = $opt . $display . $internal;

	# print links at top screen
	###########################
	print "<ul>\n";

	# quick navigation to others modules
	## no critic (ProhibitEscapedMetacharacters)
	my $cgi2 = $cgi;
	$cgi2 =~ s/\.cgi$//;
	print '<li>';
	navigation( $in{idx}, $cgi2, $display . $internal, 1 );
	print '</li>';

	# periods : from hour to year and custom : links to graphs
	print '<li>period : ';

	foreach my $n ( list_periods() ) {
		print html_link( "display.cgi?idx=$in{idx}${allopt}#$r_scale{$n}",
			$text{ $r_scale{$n} } )
		  . '&nbsp;&nbsp;';
	}

	print '</li>';

	# link to clear cache
	print li(
		html_link(
			"$cgi?idx=$in{idx}&nocache=1${allopt}",
			$text{'display_nocache'}
		)
	);

	# link to change refresh/norefresh
	display_page_norefresh( $cgi, $allopt );

	# link to internals
	display_page_internal( $cgi, $opt, $display );

	# next graph
	display_page_next( $my_config, $cgi, $allopt );

	# others displays, except the current display and the "stop" (-1) one
	my %list = get_display_list($my_config);
	foreach my $elem ( keys %list ) {
		print li(
			"$text{'display_others'} : "
			  . html_link(
				"$cgi?idx=$in{idx}$opt&nocache=1&display=$elem",
				$list{$elem}
			  )
		) if ( ( $elem != -1 ) and ( $elem != $real_display ) );
	}

	# for module controled display (ex : network)
	display_page_other( $minfo, $my_config, $cgi );

	print "</ul>\n";
	debug_sub_end();
	return;
}
###############################################################################
# display the html code
sub display_page($$$$$) {
	my ( $minfo, $my_config, $stamp, $module, $size ) =
	  ( $_[0], $_[1], $_[2], $_[3], $_[4] );

	debug_sub_begin();

	display_page_links( $minfo, $my_config );

	# period configuration
	print ui_hr();
	if ($flag_manip) {
		select_period_form( $PROGRAM_NAME, $minfo, 1 );
	}
	else {
		select_period_form( $PROGRAM_NAME, $minfo, 0 );
	}
	print ui_hr();

	# cadre general for graphs
	my @headings = ( $text{'module_title_display'} );
	print wrapper( 'ui_columns_start', \@headings );

	# print graphs
	##############
	foreach my $n ('custom') {
		display_image( $minfo, $stamp, $size, $n );
	}
	print wrapper('ui_columns_end');

	# end
	#########
	# todo : translate status message ?
	# my $status = runstop_status( $minfo, $my_config );

	# actions summary
	print "$text{'action'} : " . display_module_action( $minfo, $my_config );

	print ui_hr();
	print display_module_info( $minfo, $my_config ) . "\n";

	my_footer( $in{idx} );
	debug_sub_end();
	return;
}
###############################################################################
# analyse result of select_period_form form
sub parse_options($$$) {
	my $module_name = shift @_;
	my $minfo       = shift @_;
	my $my_config   = shift @_;

	if ( exists $in{'reset'} ) {

		# redirect without any parameters
		clear_cache($module_name);
		myredirect("display_custom.cgi?idx=$in{'idx'}");
	}
	elsif ( !exists $in{'custom'} ) {

		# no arg from url
		if ( exists $my_config->{'period'} ) {

			# use saved parameter from config file
			$in{'periode'} = $my_config->{'period'};
			$in{'custom'}  = $my_config->{'custom'};
		}
		else {

			# default
			$in{'periode'} = 'hour';
			$in{'custom'}  = 10;
		}

		# we do not save yet custom_ymin and custom_ymax (see below)
		delete $in{'custom_ymin'}   if ( exists $in{'custom_ymin'} );
		delete $in{'custom_ymax'}   if ( exists $in{'custom_ymax'} );
		delete $in{'custom_datend'} if ( exists $in{'custom_datend'} );

#if ( ( exists $my_config{'custom_ymin'}) and (defined  $my_config{'custom_ymin'} ) ) {
#	$in{'custom_ymin'} = $my_config{'custom_ymin'};
#}
#if ( ( exists $my_config{'custom_ymax'}) and (defined $my_config{'custom_ymax'}) ) {
#	$in{'custom_ymax'} = $my_config{'custom_ymax'};
#}

	}
	elsif ( exists $in{'save'} ) {

		# save config
		$my_config->{'period'} = $in{'periode'};
		$my_config->{'custom'} = $in{'custom'};

	  # we do no not save yet custom_ymin custom_ymax
	  #if ( ( exists $in{'custom_ymin'} ) and ( defined $in{'custom_ymin'} ) ) {
	  #	$my_config{'custom_ymin'} = $in{'custom_ymin'};
	  #} else {
	  #	delete $my_config{'custom_ymin'};
	  #}
	  #if ( ( exists $in{'custom_ymax'} ) and ( defined $in{'custom_ymax'} ) ) {
	  #	$my_config{'custom_ymax'} = $in{'custom_ymax'};
	  #} else {
	  #	delete $my_config{'custom_ymax'};
		##}

		write_config( $minfo, $my_config );
	}
	return;
}
###############################################################################
sub build_period($) {
	my $minfo = shift @_;

	if ( exists $in{'custom_ydef'} ) {

	}
	else {
		if ( ( exists $in{'custom_ymin'} ) and ( defined $in{'custom_ymin'} ) )
		{
			$minfo->{'custom_ymin'} = $in{'custom_ymin'};

			#$minfo{'custom_rigid'} = 1;
		}
		if ( ( exists $in{'custom_ymax'} ) and ( defined $in{'custom_ymax'} ) )
		{
			$minfo->{'custom_ymax'}  = $in{'custom_ymax'};
			$minfo->{'custom_rigid'} = 1;
		}
	}

	# end date
	if ( exists $in{'custom_now'} ) {

		$minfo->{'custom_end'} = -1;
		info('custom_now');
	}
	elsif ( exists $in{'custom_datend'} ) {

		# we have to convert a date
		if ( $flag_manip and $in{'custom_datend'} ) {
			my $date = ParseDate( $in{'custom_datend'} );
			my $sec = UnixDate( $date, '%s' );

			# date in the past will be negative
			$minfo->{'custom_end'} = $sec - time;
		}
		else {

			# empty or zero means now
			$minfo->{'custom_end'} = -1;
		}
	}

	# to be used in sysstats-lib::graph_common
	$minfo->{'custom_period'} =
	  get_period_scale( $in{'periode'} ) * $in{'custom'};
	return;
}
###############################################################################
# main
###############################################################################

debug_sub_begin();
wrapper('ReadParse');

eval { require Date::Manip };
if ($EVAL_ERROR) {
	$flag_manip = 0;
	info('perl module Date::Manip not found, all will not work');
}
else {
	import Date::Manip;
	$flag_manip = 1;
}

# module name
my $module_name = $in{'idx'};

my ( %my_config, %defaults );
my %minfo = read_module_conf( $module_name, \%text, \%my_config, \%defaults );

my $SPACE = SPACE();
my $COMMA = COMMA();

# build display title
my $title = $text{'title_display_custom'} . $SPACE . $module_name;
if ( $minfo{'module_multi_display'} ) {

	# network for exemple
	my $display = $in{'opt'};

	# we add the title to make the difference
	my ( undef, undef, $title_idx ) = split $COMMA, $my_config{$display};
	$title .= $SPACE . $my_config{ 'module_index_title' . $title_idx };
}

# clear cache ?
if ( exists $in{'nocache'} ) {
	clear_cache($module_name);
}

# necessary for graph_data and write_config sub
require $minfo{'mod_lib'};

# analyse result of select_period_form form
parse_options( $module_name, \%minfo, \%my_config );

wrapper(
	'ui_print_header', undef, $title, undef,
	$module_name,      undef, undef,  undef,
	undef,             get_meta()
);

build_period( \%minfo );

if ( exists $in{internal} ) {
	$minfo{'tmpgname'} = build_tmpgname( $module_name . '_internal' );
}
else {
	$minfo{'tmpgname'} = build_tmpgname($module_name);
}
my $stamp = png_stamp();

my $size = EMPTY();    # optionnal size for html

# test for file in cache
my $test_graph = build_graph_name( \%minfo, 'custom' ) . $stamp;
if ( !-e $test_graph ) {

	# no files in cache : rebuild all

	# call sub in module lib
	if ( exists $in{internal} ) {
		graph_internal_data( \%minfo, \%my_config, $in{'opt'} );
	}
	else {
		graph_data( \%minfo, \%my_config, $in{'opt'} );
	}

	# real sized returned by sysstats-lib::graph_common
	my $real_sizex = $minfo{'graph_sizex'};
	my $real_sizey = $minfo{'graph_sizey'};
	if ( defined $real_sizex ) {
		$size = " width=\"$real_sizex\" height=\"$real_sizey\"";
	}

	# link between the image from rrdtool and a stamped file, used in html
	foreach my $n ('custom') {

		# real graph name produced  by graph_data
		my $gname = build_graph_name( \%minfo, $n );

		# stamped name to allow cache
		my $lname = $gname . $stamp;

		link $gname, $lname;
	}

	# cache cleaning
	delayed_clear_cache($module_name);
}

display_page( \%minfo, \%my_config, $stamp, $module_name, $size );
debug_sub_end();
