#!/usr/bin/perl -w
###############################################################################
#    Webmin Sysstats - edit_module.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this cgi is used to configure any module : it calls the module's mod_config
# function
# is called from config.cgi, or from display.cgi
###############################################################################

use strict;
use warnings;

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars qw( %gconfig %config %text $module_config_directory $cb $tb %in );
use subs qw( &ReadParse );
use subs qw( &error &read_file &error &text &ui_print_header );

# from sysstat-lib.pl
use vars qw( %access );
use subs qw( &debug_sub_begin &debug_sub_end &space &space2);
## use critic

###############################################################################
sub bandeau($$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;

	html_comment('begin bandeau');
	print wrapper('ui_columns_start');

	my @columns;
	push @columns, $text{'module_title_index'};
	my $field;

	# link to the home page
	if ( $minfo->{'module_webpage'} ) {
		$field .= html_link(
			$minfo->{'module_webpage'},
			'<img src="images/gohome16.gif" border=0 alt=gohome>',
			$text{'configure_homepage'}
		);
	}
	else {
		$field .= html_space2();
	}
	$field .= html_space();

	# pause/run button
	if ( $access{'runstop'} ) {
		if ( !mod_check_runstop( $minfo, $my_config ) ) {
			$field .= html_link(
				"edit_module.cgi?idx=$in{'idx'}&runstop=1",
				'<img src="images/pause16.gif" border=0 alt=pause>',
				$text{configure_stop}
			);
		}
		else {
			$field .= html_link(
				"edit_module.cgi?idx=$in{'idx'}&runstop=1",
				'<img src="images/record16.gif" border=0 alt=record>',
				$text{configure_start}
			);
		}
	}
	else {
		$field .= html_space2();
	}
	$field .= html_space();

	if ( $access{'configure'} ) {

		# upgrade button
		$field .= html_link(
			"install_mod.cgi?idx=$in{'idx'}&upgrade=1",
			'<img src="images/icon_plus16.gif" border=0 alt="update">',
			$text{configure_upgrade}
		);
		$field .= html_space();

		# clone button
		if ( is_clone( $in{'idx'} ) ) {
			$field .= html_space2();
		}
		else {
			$field .= html_link(
				"clone_mod.cgi?idx=$in{'idx'}",
				'<img src="images/clonedata16.gif" border=0 alt="clone">',
				$text{clone_title}
			);
		}
		$field .= html_space();

		# empty button
		$field .= html_link(
			"empty_mod.cgi?idx=$in{'idx'}",
			'<img src="images/empty16.gif" border=0 alt="trash">',
			$text{configure_empty}
		);
		$field .= html_space();

		# trash button
		$field .= html_link(
			"delete_mod.cgi?idx=$in{'idx'}",
			'<img src="images/trash16.gif" border=0 alt="trash">',
			$text{configure_delete}
		);

	}
	push @columns, $field;
	my @tdtags = ( undef, 'align=right' );
	print wrapper( 'ui_columns_header', \@columns, \@tdtags );

	print wrapper('ui_columns_end');
	html_comment('end bandeau');

	return;
}
###############################################################################
# main
###############################################################################
debug_sub_begin();
ReadParse();
$access{'configure'} or text_error('configure_ecannot');

# get module info
#################

my $module_name = $in{'idx'};
my ( %my_config, %defaults );
my %minfo = read_module_conf( $module_name, \%text, \%my_config, \%defaults );

# is needed by mod_runstop
require $minfo{'mod_lib'};

# execute actions
#################
# toggle runstop if action is asked (from this form)
mod_runstop( \%minfo, \%my_config ) if ( $in{'runstop'} );

# display html code
###################
wrapper( 'ui_print_header', undef,
	$text{'title_configure_module'} . " $module_name",
	undef, 'config-' . $module_name );
print ui_hr();

navigation( $in{'idx'}, 'edit_module', q{}, 0 );

# display config file
print html_link( "show_module_config.cgi?idx=$in{'idx'}",
	$text{'show_module_config'} );
print ui_hr();

# bandeau
bandeau( \%minfo, \%my_config );

# call module sub
print '<table border summary="top" ' . w100() . ">\n";
begin_row($cb);
print "<td>\n";
html_comment('begin mod_config');
mod_config( \%minfo, \%my_config, \%defaults );
html_comment('end mod_config');
print "</td>\n";
end_row();

print "</table>\n\n";
print ui_hr();

# actions summary
print "$text{'action'} : " . display_module_action( \%minfo, \%my_config );

print ui_hr();

#print "module $module $minfo{'module_ver'}\n";
print display_module_info( \%minfo, \%my_config ) . "\n";

my_footer( $in{'idx'} );
debug_sub_end();
