#!/usr/bin/perl -w
###############################################################################
#    Webmin Syststats - install_mod_p2.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# module installation , part 2 : download module and copy it to webminstats
# module directory : it does not install it !
# the real install is done by index.cgi : detect changes, test, and install modules
# code inspired by webmin code : webmin/install_mod.cgi
#
# the webminstats form is install_mod.cgi
###############################################################################

use strict;
use warnings;

use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars qw( %text $in %in %ENV );

# from sysstats-lib.pl
use vars qw( $module_dir );
## use critic

debug_sub_begin();

error_setup( $text{'inst_mod_err4'} );

my $no_upload;
if ( $ENV{'REQUEST_METHOD'} eq 'POST' ) {
	unset_warnings();
	ReadParseMime();
	set_warnings();
}
else {
	ReadParse();
	$no_upload = 1;
}

my $file;
if ( $in{'source'} == 0 ) {

	# from local file
	error_setup( text( 'ins_mod_err1', $in{'file'} ) );
	$file = $in{'file'};

	if ( !-r $file ) {
		error( $text{'inst_mod_efile'} );
	}
	my_copy( $file, $module_dir, 'install_mod_p2' );
}
elsif ( $in{'source'} == 1 ) {

	# from uploaded file
	error_setup( $text{'inst_mod_err2'} );

	# a generic name only if we do not have the real module name
	$file = $module_dir . q{/};
	$file .=
	  ( exists $in{'upload_filename'} )
	  ? $in{'upload_filename'}
	  : 'upload.tar.gz';

	# if ($no_upload) {
	if ( !$in{'upload'} ) {
		error( $text{'inst_mod_ebrowser'} );
	}

	# write from cgi variables to file
	my $fh_mod;
	## no critic (RequireBriefOpen)
	if ( open $fh_mod, '>', $file ) {
		print {$fh_mod} $in{'upload'};
		my_close( $fh_mod, $file );
	}
	else {
		warning("can not write to $file : $ERRNO");
	}
}
elsif ( $in{'source'} == 2 ) {

	# from ftp or http url
	error_setup( text( 'inst_mod_err3', $in{'url'} ) );

	# a generic name as we do not have the real module name
	$file = $module_dir . '/url.tar.gz';

	if ( $in{'url'} =~ /^http:\/\/([^\/]+)(\/.*)$/ ) {
		my $host = $1;
		my $page = $2;
		my $port = 80;
		if ( $host =~ /^(.*):(\d+)$/ ) {
			$host = $1;
			$port = $2;
		}
		http_download( $host, $port, $page, $file )
		  ;    # from web-lib-funcs.pl (webmin)
	}
	elsif ( $in{'url'} =~ /^ftp:\/\/([^\/]+)\/(.*)$/ ) {
		my $host    = $1;
		my $ftpfile = $2;
		ftp_download( $host, $ftpfile, $file ); # from web-lib-funcs.pl (webmin)
	}
	else {
		error( $text{'inst_mod_eurl'} );
	}
}

my $option = ( $in{'upgrade'} ) ? '?upgrade=1' : EMPTY();
myredirect( 'index.cgi' . $option );
debug_sub_end();
