#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - log.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# display log file and allow to empty it
# called from index.cgi or from it-self (buttons)
###############################################################################

use strict;
use warnings;

use English '-no_match_vars';

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

## no critic (ProhibitPackageVars)
# from web-lib.pl
use vars qw( %gconfig %config %text $module_config_directory $cb $tb %in $in );

use subs
  qw( &ui_print_header &ui_columns_start &ui_columns_row &ui_columns_end &ui_form_start &ui_form_end &ui_subheading &ReadParse );

# from sysstats-lib.pl
use subs qw( &debug_sub_begin &debug_sub_end );
## use critic

###############################################################################

debug_sub_begin();
wrapper('ReadParse');

my $debug_file     = get_debug_file();
my $aquisition_log = get_aquisition_journal();
my $action_log     = action_journal();

wrapper( 'ui_print_header', undef, 'logs file', undef, 'log' );
my $cgi = basename($PROGRAM_NAME);

print wrapper('ui_columns_start');

# button for debug_file
my @tab;
push @tab,
  wrapper( 'ui_form_start', $cgi )
  . wrapper( 'ui_form_end', [ [ 'log', 'web_log' ] ] );

# button for aquisition
push @tab,
  wrapper( 'ui_form_start', $cgi )
  . wrapper( 'ui_form_end', [ [ 'log', 'aquisition_log' ] ] );

# button for action
push @tab,
  wrapper( 'ui_form_start', $cgi )
  . wrapper( 'ui_form_end', [ [ 'log', 'action_log' ] ] );

print wrapper( 'ui_columns_row', \@tab );
print wrapper('ui_columns_end');

my $log;
if ( exists $in{'log'} ) {
	if ( $in{'log'} eq 'web_log' ) {
		$log = $debug_file;
	}
	elsif ( $in{'log'} eq 'aquisition_log' ) {
		$log = $aquisition_log;
	}
	elsif ( $in{'log'} eq 'action_log' ) {
		$log = $action_log;
	}
	else {

		# default
		$log = $debug_file;
		$in{'log'} = 'web_log';
	}
}
else {

	# default
	$log = $debug_file;
	$in{'log'} = 'web_log';
}

# display log name and type
print ui_subheading("$in{'log'} : $log");

# get the number of lines to display (0 for all)
# from webminstat config file
my $max = $config{'display_log'} || 0;

my $action = $in{'action'};
if ( exists $in{'action'} ) {

	# it is called from a button (form from it-self)
	if ( $action eq 'clear' ) {

		# click on clear button : empty file and quit
		empty_file($log);

		# return to index
		jredirect('log.cgi');
	}
	elsif ( $action eq 'full' ) {
		$max = 0;
	}
	else {

		# bad action
		warning("bad action $action for log.cgi");
	}
}

# display the html form
print ui_hr();

# open log and display lines
my $r_tab = read_full_file($log);
my $empty = 1;

my $limit;
if ($max) {

	# do not display first lines before limit
	my $nb_total = scalar @{$r_tab};
	$limit = $nb_total - $max;
}
else {
	# no limit
	$limit = 0;
}

if ( $limit > 0 ) {
	display_html(" ... skip first lines, keep last $max lines ... ");
}
my $i = 0;
foreach my $ligne ( @{$r_tab} ) {
	$i++;
	display_html($ligne) if ( $i > $limit );
	$empty = 0;
}

if ($empty) {
	print '(empty)';
}
print ui_hr();

# now display buttons
# clear
print wrapper( 'ui_form_start', $cgi );
print wrapper( 'ui_hidden', 'log', $in{'log'} );
print wrapper( 'ui_form_end', [ [ 'action', 'clear' ], [ 'action', 'full' ] ] );

my_footer();
debug_sub_end();
