
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a lib dedicated to rrdtools sub
# the code was duplicated from post-install to be integrated in a lib
# migration is not ended (it will done when post-install will use this lib)
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

use File::Basename;
use Cwd;

# require, not use, to allow error diagnostic (index.cgi)
require RRDs;

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars qw(  %gconfig %config %text $module_config_directory );
use vars qw( $module_dir );    # from sysstats-lib.pl
## use critic

## no critic (RequireBarewordIncludes)
require 'sysstats-constants-lib.pl';
## use critic

my $stamp_ext = '.stamp';      # stamp extension use to mark if a check was done
my $SLASH     = SLASH();
###############################################################################

# read and parse module info and return it as a hash
sub get_sysstats_module_info($) {
	my ( $m, %rv );
	$m = $_[0];                # module name (ex cpu )

	#debug_sub_begin();

	my $mdir = $module_dir . $SLASH . $m . $SLASH;    # ex modules/cpu/

	my $minfo_file = $mdir . 'module.info';
	if ( !read_file( $minfo_file, \%rv ) ) {
		text_error('main_ecannet_read_mod_info');
	}

	$rv{'idx'}         = $m;                                       # module name
	$rv{'config_dir'}  = $module_config_directory . $SLASH . $mdir;
	$rv{'config_file'} = $rv{'config_dir'} . 'config';
	$rv{'mod_dir'}     = $mdir;
	$rv{'defaults_file'} = $rv{'mod_dir'} . 'defaults.cfg';
	$rv{'mod_lib'}       = "$rv{'mod_dir'}$rv{'module_name'}-lib.pl";
	$rv{'module_webpage'} =
	  'http://webminstats.sourceforge.net/modules/' . $m . '.html';

	# database is on config_dir unless if dbtables is set
	$rv{'dbtables'} =
	  $config{'dbtables'} ? $config{'dbtables'} : $rv{'config_dir'};

	# search a translation language
	my $lang = get_lang( $rv{'mod_dir'} );
	$rv{'mod_lang'} = $rv{'mod_dir'} . 'lang/' . $lang;

	#debug_sub_end();
	return %rv;
}
###############################################################################
# change runstop status (toggle)
sub mod_runstop($$) {
	my ( $minfo, $my_config ) = ( $_[0], $_[1] );

	debug_sub_begin();

	$my_config->{'runstop'} = $my_config->{'runstop'} ? '0' : '1';

	# do not call write_config as it will check conf and
	# may change runstop
	common_write_config( $minfo, $my_config );
	debug_sub_end();
	return;
}

###############################################################################
# return module status
# be careful : 1 for stop, 0 for run !
sub mod_check_runstop($$) {
	my ( $minfo, $my_config ) = ( $_[0], $_[1] );

	debug_sub_begin();
	return $my_config->{'runstop'};
}
###############################################################################
# return status in a human format
sub runstop_status($$) {
	my ( $minfo, $my_config ) = ( $_[0], $_[1] );

	my $stat = mod_check_runstop( $minfo, $my_config );
	my $status = $stat ? 'stopped' : 'run';
	return $status;
}
###############################################################################
# runstop param : 0 stand for run
#                 1 stand for stop
# return true if is to be runned
#sub is_param_runstop($) {
#	my $runstop = $_[0];
#	return !$runstop;
#}
###############################################################################
# change (invert) runstop parameter
sub toggle_param_runstop($) {
	my $runstop = $_[0];

	return $runstop ? 0 : 1;
}
###############################################################################
# get module name from module config
sub get_modconfig_name($) {
	my $r_modconfig = shift @_;

	return $r_modconfig->{'name'};
}
###############################################################################
# remove empty values in config files
sub clean_config ($) {
	my $r_modconfig = shift @_;

	my $changes = 0;
	foreach my $key ( keys %{$r_modconfig} ) {
		if ( !defined $r_modconfig->{$key} ) {
			delete $r_modconfig->{$key};
			debug("clean $key");
			$changes++;
		}
	}
	return $changes;
}
###############################################################################
sub list_used_param($;$) {
	my $rh_config = shift @_;    # config
	my $ind = shift @_ || 1;   # allow to choice a specific column as name (irq)

	# first build list of parameters already used in config
	my %known;
	foreach my $key ( keys %{$rh_config} ) {
		my $val = $rh_config->{$key};

		#info("list_used_param : $key $val");
		if ( is_vol( $key, $val ) ) {
			my $id = get_vol_name( $val, $ind );
			if ( exists $known{$id} ) {
				info("duplicate name $id");
			}
			else {
				$known{$id} = $key;

				#info("find vol $id");
			}
		}
	}
	return %known;
}
###############################################################################
# remove parameters already in config file
sub filter_known($@) {
	my $rh_config = shift @_;    # config
	my @vols      = @_;          # full list of available paramaters

	# first build list of parameters already used in config
	my %known = list_used_param($rh_config);

	# next filter the given parameters to return only unused
	my @unknown;
	foreach my $vol (@vols) {
		push @unknown, $vol unless ( exists $known{$vol} );
	}

	return @unknown;
}
###############################################################################
# change directory to the module directory
sub goto_dir($) {
	my $dir = shift @_;

	if ( -d $dir ) {

		# ok $dir is already a directory name
		my $old_dir = getcwd();
		if ( chdir $dir ) {

			# return old dir so that caller can return to it
			return $old_dir;
		}
		else {
			warning("can not chdir to $dir : $ERRNO");
			return;
		}
	}
	else {
		warning("can not chdir to $dir : not a directory");
		return;
	}
}
###############################################################################
# to supress bad config lines (runstop key without value)
sub clean_runstop ($) {
	my $r_modconfig = shift @_;

	my $stamp = 'runstop' . $stamp_ext;
	if ( !-e $stamp ) {

		if (   ( !$r_modconfig->{'runstop'} )
			&& ( !defined $r_modconfig->{'runstop'} ) )
		{
			$r_modconfig->{'runstop'} = 0;
			my $module = get_modconfig_name($r_modconfig);
			warning("$module : clean_runstop fix");
			return 1;
		}
		else {
			debug( 'runstop = ' . $r_modconfig->{runstop} );
		}
		touch($stamp);
	}
	else {
		debug('clean_runstop already stamped');
	}
	return;
}
###############################################################################
# return (undef == false) if does not match a "volume" parameter
# else return the database id (id.rrd database file)
sub is_vol($$) {
	my $key = shift @_;    # vol12
	my $val = shift @_;    # 5,my_param, ....

	return if ( $key =~ m/^display/ );     # in network
	return if ( $key =~ m/^gdisplay/ );    # display system
	return if ( $key =~ m/^action/ );      # action lib
	return if ( $key !~ m/^\S+\d+$/ );     # should be like : vol12

	# the database id is a number
	if ( $val =~ m/^(\d+),/ ) {
		return $1;
	}
	else {
		return;
	}
}
###############################################################################
sub get_vol_prefix($) {
	my $key = shift @_;

	my $prefix = $key;
	$prefix =~ s/\d//g;
	return $prefix;
}
###############################################################################
# return the description name for a "volume" parameter
sub get_vol_name($;$) {
	my $val = shift @_;        # config line
	my $id = shift @_ || 1;    # allow to choice a specific column for name

	# we assume the order
	# parameter 0 : database id (see below is_vol)
	# parameter 1 : description name
	my @tab = split /,/, $val;
	return $tab[$id];
}
###############################################################################
# return the max volume id
# real volume id key (vol5), not database
# see also check_db_uid
sub get_max_vol($) {
	debug_sub_begin();
	my $r_modconfig = shift @_;

	# build volumes hash
	my $max = 0;
	foreach my $key ( keys %{$r_modconfig} ) {
		my $vol = is_vol( $key, $r_modconfig->{$key} );
		if ($vol) {

			# get key id
			if ( $key =~ m/(\d+)$/ ) {
				my $key_id = $1;
				$max = $key_id if ( $key_id > $max );
			}
		}
	}
	debug_sub_end();
	return $max;
}
###############################################################################
# change display value
# old : 0 => display; 1 => no display
# new : 0 => display; -1 => no display
sub mod_change_display ($) {
	my $r_modconfig = shift @_;

	my $changes = 0;
	my $stamp   = 'new_display' . $stamp_ext;
	if ( !-e $stamp ) {

		foreach my $key ( keys %{$r_modconfig} ) {

			next unless ( is_vol( $key, $r_modconfig->{$key} ) );

			if ( $r_modconfig->{$key} =~ s/,([01]),1$/,$1,-1/ ) {
				my $module = get_modconfig_name($r_modconfig);
				warning("$module : change_display $key");
				$changes++;
			}
		}
		touch($stamp);
	}
	else {
		debug('change_display already stamped');
	}
	return $changes;
}

######################################################
# checks to fix bad module config
######################################################
# check db_uid and fix it if necessary
# rem db_uid is the the number of the first available rrd database
# it begins at 1 (not 0)
# return true is some changes are done
sub check_db_uid ($@) {
	my $r_modconfig = shift @_;    # module config
	my @list_rrd    = @_;
	debug_sub_begin();

	my $iret = 0;
	if ( exists $r_modconfig->{'db_uid'} ) {
		my $db_uid = $r_modconfig->{'db_uid'};
		my $module = get_modconfig_name($r_modconfig);

		# search for max rrd name
		my $maxvol = 0;
		my $nb_rrd = 0;
		foreach my $name (@list_rrd) {

			# get number for vol modules
			if ( $name =~ m/(\d+)[.]rrd$/ ) {
				$maxvol = $1 if ( $1 > $maxvol );
			}
			elsif ( $name =~ m/internal[.]rrd$/ ) {

				# internal data are special
				next;
			}
			else {

				# not a volume (ex: samba.rrd)
				$nb_rrd++;
			}
		}
		my $max = ($maxvol) ? $maxvol : $nb_rrd;
		debug("max rrd : $max");

		# the first available is the next one
		$max++;
		if ( $max != $db_uid ) {
			info("$module : fix db_uid=$max (was $db_uid)");
			$r_modconfig->{'db_uid'} = $max;
			$iret = 1;
		}
		else {
			debug('check_db_uid : ok');
		}

		# test for runstop
		if ( $max == 1 ) {

			# no database : module is to be stopped
			if ( $r_modconfig->{'runstop'} == 0 ) {
				$r_modconfig->{'runstop'} = 1;
				$iret = 1;
				info("$module : stop module because db_uid = 1");
			}
		}
	}
	else {
		debug('check_db_uid : no db_uid');
	}
	debug_sub_end();
	return $iret;
}
######################################################
# check if config file use duplicate databases
# it is just a warning, as I do not know how to fix it
sub check_duplicate_rrd($) {
	my $r_modconfig = shift @_;
	debug_sub_begin();

	my $module = get_modconfig_name($r_modconfig);
	my %database;
	foreach my $key ( keys %{$r_modconfig} ) {
		my $vol = is_vol( $key, $r_modconfig->{$key} );
		if ($vol) {
			if ( exists $database{$vol} ) {
				warning(
"$module duplicate database id $vol for $key and $database{$vol}"
				);
			}
			else {
				$database{$vol} = $key;
			}
		}
	}

	debug_sub_end();
	return 0;
}
######################################################
# suppress unused config :
# - volume without database
# - database without volumes
sub check_unused($@) {
	my $r_modconfig = shift @_;
	my @list_rrd    = @_;
	debug_sub_begin();

	my $module      = get_modconfig_name($r_modconfig);
	my $return_code = 0;

	my $ko = 1;    # to force a first loop
	while ($ko) {
		$ko = 0;    # suppose all is ok

		# build database hash
		my %databases;
		foreach my $name (@list_rrd) {

			# suppress path and extension
			my $basename = basename( $name, '.rrd' );

			# names are number for "dynamic" modules
			if ( $basename =~ m/^(\d+)$/ ) {
				$databases{$1} = $name;
			}
		}

		# build volumes hash (database id)
		my %vols;
		foreach my $key ( keys %{$r_modconfig} ) {
			my $vol = is_vol( $key, $r_modconfig->{$key} );
			if ($vol) {
				$vols{$vol} = $key;
			}
		}

		# check databases files
		foreach my $b ( keys %databases ) {
			if ( !exists $vols{$b} ) {
				warning("$module unused database $b : delete $databases{$b}");

				# remove unused
				unlink $databases{$b};
			}
		}

		# check config vols (database id)
	  VOLS: foreach my $v ( keys %vols ) {
			if ( !exists $databases{$v} ) {
				my $key  = $vols{$v};
				my $name = get_vol_name( $r_modconfig->{$key} );
				warning(
"$module volume $key ($name) without database $v : delete $key"
				);

   # we can not remove a volume in the middle ...
   # and call ro remove_vol (sysstats-web-lib.pl) will change all orders
   # so we should re-analyse (loop)
   # rem : check_db_uid will be done after all changes (cf caller :check_config)
				remove_vol( $r_modconfig, DOT(), $key );

				$ko = 1;           # to continue the loop
				$return_code++;    # errors
				last VOLS;         # only one change
			}
		}
	}
	return $return_code;
}
######################################################
# check if config file use duplicate descriptions
# do not fix anything
sub check_duplicate_name($) {
	my $r_modconfig = shift @_;
	debug_sub_begin();

	my $module = get_modconfig_name($r_modconfig);

	my %names;
	foreach my $key ( keys %{$r_modconfig} ) {
		my $vol = is_vol( $key, $r_modconfig->{$key} );
		if ($vol) {
			my $name = get_vol_name( $r_modconfig->{$key} );
			if ( exists $names{$name} ) {
				warning(
"$module duplicate database description $name for $key and $names{$name}"
				);
			}
			else {
				$names{$name} = $key;
			}
		}
	}

	debug_sub_end();
	return 0;
}
######################################################
# common test for runstop
# should be the last one in check_for_runstop
# set run if exists parameter, else stop
sub common_check_for_runstop($) {
	my $my_config = $_[0];

	# todo  : should also test upgrade or keep to avoid  warnings
	if ( get_max_vol($my_config) == 0 ) {
		$my_config->{'runstop'} = 1;    # stop
		info( 'disable ' . $my_config->{'name'} . ' : no parameters found' );

	}
	elsif ( exists $ENV{'WEBMINSTATS_UPDATE'} ) {

		# keep config
	}
	else {
		$my_config->{'runstop'} = 0;    # run
	}
	return;
}
######################################################
# add module name in config file if not exists (old versions)
sub check_name($$) {
	my $module      = shift @_;
	my $r_modconfig = shift @_;

	if ( exists $r_modconfig->{'name'} ) {
		if ( $r_modconfig->{'name'} eq $module ) {
			return 0;
		}
		else {

			# changed name
			$r_modconfig->{'name'} = $module;
			return 1;
		}
	}
	else {

		# no name : add it
		$r_modconfig->{'name'} = $module;
		return 1;
	}
}
###############################################################################
# clean unused display labels
sub clean_display($) {
	my $r_modconfig = shift @_;

	my $module = $r_modconfig->{'name'};
	my %used_display;
	$used_display{-1} = 1;    # stop (reserved display)
	$used_display{0}  = 1;    # default (reserved display)

	# search for used display
	foreach my $key ( keys %{$r_modconfig} ) {
		next unless ( is_vol( $key, $r_modconfig->{$key} ) );
		if ( $r_modconfig->{$key} =~ m/,(\d+)$/ ) {
			$used_display{$1} = 1;
			debug("$module found display $1 ($key)");
		}
		else {
			debug("$module strange vol line $r_modconfig->{$key} for $key");
		}
	}

	# search for display keywords
	my $nb_delete = 0;
	foreach my $key ( keys %{$r_modconfig} ) {
		if ( $key =~ m/^gdisplay(\d+)$/ ) {
			my $display = $1;
			if ( !exists $used_display{$display} ) {
				$nb_delete++;
				delete $r_modconfig->{$key};
				info("$module delete unused display $display");
			}
			else {
				debug("$module keep display $display");
			}
		}
	}
	return $nb_delete;
}
##########################################################
# get prefix from vol line key
sub get_pre_id($) {
	my $key = shift @_;

	# key are in the form : vol5
	if ( $key =~ m/^(\D+)(\d+)$/ ) {
		return ( $1, $2 );
	}
	else {
		return;
	}
}
##########################################################
# remove a volume
sub remove_vol($$$) {
	my $my_config     = shift @_;
	my $dbtables_path = shift @_;
	my $key           = shift @_;

	debug_sub_begin();
	my ( $pre, $vol_num ) = get_pre_id($key);

	# get database id to be deleted
	my ( $db_id, undef ) = unpack_config( $my_config->{$key} );

	# remove database
	my_unlink( $dbtables_path . $db_id . '.rrd' );

	# special hack for disk module : inode rrd
	my $inode_rrd = $dbtables_path . 'i' . $db_id . '.rrd';
	if ( -f $inode_rrd ) {
		my_unlink($inode_rrd);
	}

	# move volumes configuration
	while ( defined $my_config->{ $pre . ( $vol_num + 1 ) } ) {
		$my_config->{ $pre . $vol_num } =
		  $my_config->{ $pre . ( $vol_num + 1 ) };

		#debug("remove_vol : change $vol_num");
		$vol_num++;
	}

	# the last one is a fake
	delete $my_config->{ $pre . $vol_num };

	#debug("remove_vol $vol_num");

	debug_sub_end();
	return;
}
###############################################################################
# guess language from webmin config
# and check if file exists
# the order is user/host/default
sub get_lang($;) {
	my $rep = shift @_ || EMPTY();    # directory (optionnal)

	my $lang;

	my $remote_user = $ENV{'REMOTE_USER'};

# to avoid warning : Odd number of elements in anonymous hash at sysstats-conf-lib.pl line 688
	no warnings 'misc';
	if (    ($remote_user)
		and ( defined $gconfig{ 'lang_' . $remote_user } )
		and ( -r { $rep . 'lang/' . $gconfig{ 'lang_' . $remote_user } } ) )
	{

		# user
		$lang = $gconfig{ 'lang_' . $remote_user };
	}
	elsif ( ( defined $gconfig{'lang'} )
		and ( -r { $rep . 'lang/' . $gconfig{'lang'} } ) )
	{

		# host
		$lang = $gconfig{'lang'};
	}
	else {

		# default
		$lang = 'en';
	}
	use warnings;
	return $lang;
}
###############################################################################
# read translation text
sub read_text(;$) {
	my $rep = shift @_ || EMPTY();

	my $lang = get_lang($rep);
	read_file( $rep . "lang/$lang", \%text );

	return;
}
###############################################################################
# read config files for batch scripts
sub batch_readconf() {

	# read config files
	read_file( $ENV{'WEBMIN_CONFIG'} . '/config', \%gconfig )
	  or die "can not read webmin config : $ERRNO\n";    # webmin config
	read_file( $module_config_directory . '/config', \%config )
	  or die "can not read webminstats config : $ERRNO\n";  # webminstats config

	# translation
	read_text();
	return;
}
###############################################################################
# read module all config files
sub read_module_conf($$$$) {
	my $module_name = shift @_;                             # input
	my $r_text      = shift @_;                             # hash result
	my $r_my_config = shift @_;                             # hash result
	my $r_defaults  = shift @_;                             # hash result

	my %minfo = get_sysstats_module_info($module_name);

	# translation
	if ( !read_file( $minfo{'mod_lang'}, $r_text ) ) {
		text_error( 'ecannot_readlang', $minfo{'mod_lang'} );
	}

	# config file
	if ( !read_module_config( $r_my_config, $minfo{'config_file'} ) ) {
		text_error( 'ecannot_readconfig', $minfo{'config_file'} );
	}

	# defaults file
	if ( -r $minfo{'defaults_file'} ) {
		if ( !read_file( $minfo{'defaults_file'}, $r_defaults ) ) {
			error( text( 'ecannot_readdefaults', $minfo{'defaults_file'} ) );
		}
	}

	return %minfo;
}
###############################################################################
# help to migrate data name prefix (pre)
sub change_pre($$$) {
	my $my_config = shift @_;    # hash result
	my $old_pre   = shift @_;
	my $new_pre   = shift @_;

	my $changes = 0;

	if ( $my_config->{'pre'} eq $old_pre ) {
		$my_config->{'pre'} = $new_pre;
		$changes++;
	}

	my $module_name = $my_config->{'name'};
	foreach my $key ( keys %{$my_config} ) {
		if ( $key =~ m/^$old_pre/ ) {
			my $new_key = $key;
			$new_key =~ s/$old_pre/$new_pre/;
			$my_config->{$new_key} = $my_config->{$key};
			delete $my_config->{$key};
			info("$module_name : change vol name from $key to $new_key");
			$changes++;
		}
	}

	return $changes;
}
###############################################################################
1;
