
###############################################################################
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this is a very general lib, with constants
###############################################################################
use strict;
use warnings;
use File::Path;    # mkpath
use File::Copy;    # mkpath

## no critic(ProhibitPackageVars)
use vars qw( $module_name );

##########################################################
my %constants = (
	COMMA  => q{,},
	DASH   => q{-},
	DIESE  => q{#},
	DOT    => q{.},
	DQUOTE => q{"},
	EMPTY  => q{},
	EQUAL  => q{=},
	PERIOD => q{:},
	PIPE   => q{|},
	QUOTE  => q{'},
	SLASH  => q{/},
	SPACE  => q{ },
	STAR   => q{*},
	PCENT  => q{%},

	COLOR_OK     => '#00ff00',    # green
	COLOR_BETA   => '#ffa500',    # orange
	COLOR_UPDATE => '#ff0000',    # red
	COLOR_ALARM  => '#ff0000',    # red
	COLOR_WARN   => '#ffa500',    # orange
	COLOR_OFF    => '#000000',    # black
);

# we build here sub to each constant
## no critic (ProhibitNoStrict)
foreach my $id ( keys %constants ) {
	no strict 'refs';
	*{$id} = sub () { $constants{$id} };
}

##########################################################
# 2 design choice :
# avoid /tmp (is cleaned by some distrib) : use /var/webmin
# allow webmin cloning : should contain webmin module name
sub get_tempdir() {

	my $oldtmpdir = "/var/webmin/$module_name/";
	my $tmpdir    = "/var/webmin/modules/$module_name/";

	if ( !-d $tmpdir ) {
		mkpath($tmpdir);
	}

	# move logs to official directory
	if ( -d $oldtmpdir ) {
		rmdir $tmpdir;
		move $oldtmpdir, $tmpdir;
	}
	return $tmpdir;
}
##########################################################
sub get_aquisition_journal() {
	return get_tempdir() . 'aquisition.log';
}
##########################################################
# return log name
sub action_journal() {
	return get_tempdir() . 'action.log';
}
##########################################################
sub get_debug_file() {
	return get_tempdir() . 'webminstats.debug';
}
##########################################################
# the module script is a perl script
# it should be the only one in /etc/...
# and for debugging, it is interesting to have a name
# relative to the module name, not a generic name
# ( ex : Argument "U" isn't numeric in subtraction (-) at run.pl )
sub get_module_script($) {
	my $module = shift @_;

	return $module . '.pl';
}
##########################################################
# define a default name (for example if run just a module,
#  not systats.pl ) : may be not good in case of a clone !
if ( !defined $module_name ) {
	$module_name = 'sysstats';
}

1;
