
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
#
# this lib is dedicated to cgi/html global code
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in %access);

use subs qw( &init_config &get_module_acl );
use subs qw( &foreign_call &lock_file &unlock_file &read_file );

# from sysstat-lib
use subs qw( &debug_sub_begin &debug_sub_end );
## use critic

# webmin javascript lib to display errors
# is optionnal to work with old webmin versions
my $java_flag = 1;
## no critic (RequireBarewordIncludes)
$WARNING = 0;
eval { require '../javascript-lib.pl' };    # from webmin
$WARNING = 1;
## use critic
if ($EVAL_ERROR) {

	# no warning to avoid too many messages
	$java_flag = 0;
}
###############################################################################
#              section : debug/printing
###############################################################################
# detect debug mode
# can be set from the general config file
# or by a parameter in the url
sub is_debug_mode () {
	return ( $config{'debug_mode'} || $in{'debug'} || $ENV{'DEBUG_MODE'} );
}
##############################################################################
sub set_debug() {
	$ENV{'DEBUG_MODE'} = 1;
	return;
}
##############################################################################
sub unset_debug() {
	delete $ENV{'DEBUG_MODE'};
	return;
}
##############################################################################
# write to webminstats log file
# low-level : should be called by another sub (debug ...)
sub my_log($;$) {
	my $texte = shift @_;
	my $level = shift @_ || 2;

	journaliser( get_debug_file(), get_caller($level) . $texte );
	return;
}
######################################################################
# used to print debugging messages
sub debug($;$) {
	my $text = shift @_;
	my $level = shift @_ || 2;
	my_log( 'debug ' . $text, $level ) if ( is_debug_mode() );
	return;
}
######################################################################
# top-level debug function to print a hashing var
sub debug_hash($%) {
	my $text = shift @_;
	my %tab  = @_;

	if ( is_debug_mode() ) {

		debug( "debug_hash : $text", 3 );

		foreach my $elem ( sort keys %tab ) {
			if ( defined $tab{$elem} ) {
				debug( "  tab{ $elem } = $tab{$elem}", 3 );
			}
			else {
				debug( "  tab{ $elem } = undef", 3 );
			}
		}
	}
	return;
}
#####################################################################
# to display warning messages
# use java functions from webmin if available
sub warning($) {
	my $text = shift @_;

	# first log to file
	my_log( 'warning : ' . $text );

	# display a warning widget
	if ( $java_flag and is_interactif() ) {
		jalert($text);
	}
	else {
		print 'WARNING : ' . $text . "<br>\n";
	}
	return;
}
#####################################################################
# messages to be logged without warning
sub info($) {
	my $text = shift @_;

	my_log( 'info : ' . $text );
	return;
}
#####################################################################
# to exit program with a message
# use java functions from webmin if available
sub dying($) {
	my $text = shift @_;

	# first log to file
	my_log( 'die : ' . $text );

	# display a warning widget
	if ( $java_flag and is_interactif() ) {
		jalert($text);
		exit;
	}
	else {
		error($text);
	}
	return;
}
######################################################################
# a tracing function
# should be called at begin of each subroutine
sub debug_sub_begin() {
	debug( 'begin sub', 3 );
	return;
}
######################################################################
# a tracing function
# should be called at end of each subroutine
sub debug_sub_end() {
	debug( 'end sub', 3 );
	return;
}
###############################################################################
# call webmin log function
# log will be available from https://localhost:10000/webminlog/
sub my_webmin_log($$$) {
	my $action = shift @_;
	my $type   = shift @_;
	my $object = shift @_;

	my %h = ();
	wrapper( 'webmin_log', $action, $type, $object, \%h, 'webminstats' );
	return;
}
###############################################################################
# text_error(text params)
sub text_error {

	## no critic (ProhibitAmpersandSigils)
	&error( &text(@_) );

	return;
}

###############################################################################
1;
