
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# the main webminstats lib
# it contains several code section :
# - debug/print
# - diverse
# - release status
###############################################################################
use strict;
use warnings;

# we use the "our" keywords to share variables betweens scripts
# so we require perl 5.6
require 5.006;

#use CGI qw(:standard param); # no more used : webmin lib is enought
#use Data::Dumper;    # internal debug use
use File::Copy;
use File::Path;    # rmtree
use File::Basename;
use POSIX qw( strftime );
use Cwd;           # getcwd
use English qw(-no_match_vars);
use CGI::Carp qw(fatalsToBrowser warningsToBrowser carpout);

## no critic (ProhibitNoisyQuotes)
use lib '.';       # for webminstats libs
use lib '..';      # for webmin libs

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory %in $module_name $root_directory $trust_unknown_referers);
$trust_unknown_referers = 1;    # fix problem with display refresh

use subs qw( &init_config &get_module_acl &webmin_log &error );
use subs qw( &foreign_call &lock_file &unlock_file &read_file );

# import webmin library
# is necessary to configure module_name for get_tempdir
## no critic (ProhibitAmpersandSigils)
$WARNING = 0;
eval { require WebminCore };
if ($EVAL_ERROR) {

	# Fallback for older Webmins
	do '../web-lib.pl';
}
else {
	import WebminCore;
}
$WARNING = 1;

## no critic (RequireBarewordIncludes)
# import webminstats libraries
# be carefull : errors from lib go to ?
require
  'sysstats-constants-lib.pl';    # basic commons, temp dir (needs module_name)
require 'sysstats-io-lib.pl';     # io
require 'sysstats-gen-lib.pl';    # common functions
require 'sysstats-web-lib.pl';    # web
require 'sysstats-rrd-lib.pl';    # rrdtool
require 'sysstats-conf-lib.pl';   # config
require 'sysstats-cron-lib.pl';   # cron (start/stop)
require 'sysstats-update-lib.pl'; # software update
require 'sysstats-action-lib.pl'; # actions
require 'sysstats-clone-lib.pl';  # clones

# prepare debug file
my $debug_dir = get_tempdir();
mkdir $debug_dir if ( !-d $debug_dir );
my $debug_file = get_debug_file();

# where to send errors ?
if ( $ENV{'BATCH_MODE'} ) {

	# in batch mode : to the console
	warningsToBrowser(1);
}
else {

	# in web mode : to the debug file
	## no critic (RequireBriefOpen)
	my $fh_log;
	if ( open $fh_log, '>>', $debug_file ) {
		$OUTPUT_AUTOFLUSH = 1;
		carpout($fh_log);
	}
	else {
		die "Unable to open $debug_file : $ERRNO\n";
	}
}

###############################################################################
# directories
# $debug_dir (webminstats) : path to log file
# $module_config_directory : /etc/webmin/sysstats
# $root_directory          : /usr/libexec/webmin
# $module_name    	   : sysstats (directory name)
###############################################################################
# some contants
my $EMPTY  = EMPTY();
my $PERIOD = PERIOD();
my $SLASH  = SLASH();

## no critic (ProhibitAmpersandSigils)
&wrapper('init_config');

# global variables
use vars qw( %access );
our %access;
%access = get_module_acl() if ( &is_interactif() );
our $cron_dir   = '/etc/cron.d/';
our $module_dir = 'modules';
our $graph_dir  = 'graphs';

my $general_script = 'sysstats.pl';
my $general_lib    = 'sysstats-gen-lib.pl';
my $aquisition_lib = 'sysstats-aquisition-lib.pl';
our $defline = 'LINE2';
## use critic

clean_env();

###############################################################################
# used by batch_install.pl batch_graph.pl
# the code before should call it with & to avoid the warning :
# main::is_interactif() called too early to check prototype
sub is_interactif() {
	return !exists $ENV{'BATCH_MODE'};
}
###############################################################################
#	section : diverses
###############################################################################
# return full path to webminstats directory : /usr/libexec/webmin/sysstats/
sub sysstat_dir() {
	return $root_directory . $SLASH . $module_name . $SLASH;
}
###############################################################################
# not used yet
sub mod_check_upgrade($$$) {
	my ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );

	return 1;
}
###############################################################################
# return true if is to be runned
sub is_param_display($) {
	my $display = $_[0];
	return $display >= 0;
}
###############################################################################
# change display status
# from stopped to display 0
# from any display to stop (-1)
sub toggle_param_display($) {
	my $display = $_[0];

	return is_param_display($display) ? -1 : 0;
}
###############################################################################
# return a list of database names for the designed module
sub get_rrd_names($) {
	my $minfo = $_[0];

	# search for a database
	return get_file_names( $minfo->{'dbtables'}, '*.rrd' );
}

###############################################################################
# remove a module
sub remove_module($$) {
	my $minfo       = shift @_;
	my $flag_update = shift @_;

	debug_sub_begin();
	my $module_name = $minfo->{'idx'};

	my_webmin_log( 'delete', 'module', build_full_name($minfo) );

	# delete module config directory
	rmtree( $minfo->{'config_dir'} );

	# database
	if ( exists $minfo->{'dbtables'} ) {
		rmtree( $minfo->{'dbtables'} );
	}

	# delete module directory if exist
	if ( exists $minfo->{'mod_dir'} ) {
		rmtree( $minfo->{'mod_dir'} );
	}

	# delete help files if not an update
	if ( !$flag_update ) {
		my $help_path   = sysstat_dir() . 'help/';
		my @config_help = get_file_names( $help_path, "config-$module_name*" );
		my @help        = get_file_names( $help_path, "$module_name*" );
		unlink @config_help, @help;
	}

	# un-register clones modules
	clone_unreg($module_name);

	debug_sub_end();
	return;
}

######################################################################
# get webminstats version (used to be display on index)
sub get_sysstats_version() {
	debug_sub_begin();

	my $minfo_file = 'module.info';

	my %rv;

	if ( !read_file( $minfo_file, \%rv ) ) {
		text_error('main_ecannet_read_mod_info');
	}

	debug_sub_end();
	return $rv{'module_ver'};
}
###############################################################################
# because functions in webmin lib are not enough clean
# we have to disable warnings before calls
# be careful : do not use this if the sub return an array
sub wrapper ($;@) {
	my $sub = shift @_;

# code to debug calls
# my ( $package, $filename, $line ) = caller();
# debug("call wrapper on $sub from package=$package, file=$filename, line=$line");

	debug_sub_begin();
	unset_warnings();

	## no critic (ProhibitNoStrict)
	no strict 'refs';

	## no critic (ProhibitDoubleSigils)
	my $ret = &$sub(@_);
	## use critic

	use strict;

	set_warnings();
	debug_sub_end();
	return $ret;
}

# this one is for sub which will return an array
sub wrapper_ar ($;@) {
	my $sub = shift @_;

	debug_sub_begin();
	unset_warnings();

	## no critic (ProhibitNoStrict)
	no strict 'refs';

	## no critic (ProhibitDoubleSigils)
	my @ret = &$sub(@_);
	## use critic

	use strict;

	set_warnings();
	debug_sub_end();
	return @ret;
}
###############################################################################
sub set_ymin($) {
	my $minfo = shift @_;

	my $ymin;
	if (    ( exists $minfo->{'custom_ymin'} )
		and ( defined $minfo->{'custom_ymin'} ) )
	{
		$ymin = $minfo->{'custom_ymin'};
	}
	else {
		$ymin = 0;
	}
	return $ymin;
}
###############################################################################
## no critic (ProhibitManyArgs)
sub build_graph($$$$$$$$@) {
	my $minfo      = shift @_;
	my $graph      = shift @_;
	my $start      = shift @_;
	my $end        = shift @_;
	my $sizex      = shift @_;
	my $sizey      = shift @_;
	my $title      = shift @_;
	my $y_scale    = shift @_;
	my @rrd_detals = @_;

	# special x-axis labels
	my %xgrid = (
		hour  => 'MINUTE:1:MINUTE:5:MINUTE:10:0:%H:%M',
		month => 'WEEK:1:MONTH:1:MONTH:1:2592000:%B'
	);

	my $date = strftime '%a %b %e %Hh%Mm%Ss %Y', localtime;
	my $sig = "COMMENT:Created $date";

	my @common;
	push @common, build_graph_name( $minfo, $graph );
	push @common, '--start', $start, '--end', $end;
	push @common, '-h',      $sizey, '-w',    $sizex;
	push @common, '-a', 'PNG', '--title', $title, '--interlaced';
	push @common, @{$y_scale};
	if ( exists $minfo->{'custom_period'} ) {
		push @common, '-x', $xgrid{'month'}
		  if ( $minfo->{'custom_period'} < -2_678_400 );
	}
	else {
		push @common, '-x', $xgrid{$graph}
		  if ( exists $xgrid{$graph} );
	}

	debug("rrdtool graph @common @rrd_detals  $sig");

	my ( undef, $xsize, $ysize ) = RRDs::graph( @common, @rrd_detals, $sig );
	my $ERR = RRDs::error();
	warning("ERROR while creating graph : $ERR") if $ERR;
	return ( $xsize, $ysize );
}
## use critic
###############################################################################
# common code for all modules, to create the same graph
sub graph_common($$@) {
	my $minfo      = shift @_;
	my $my_config  = shift @_;
	my @rrd_detals = @_;

	debug_sub_begin();

	unset_warnings();
	eval { require RRDs };
	set_warnings();
	if ($EVAL_ERROR) {

		# should never occur : it is tested by index.cgi
		# except in batch mode ?
		warning('can not use RRDs perl module');
		return;
	}

	# local or global config
	my $sizex = $minfo->{'graph_sizex'} || $config{'graph_sizex'};
	my $sizey = $minfo->{'graph_sizey'} || $config{'graph_sizey'};

	# y-axis : min
	my @y_scale;
	my $ymin = set_ymin($minfo);

	if (    ( exists $my_config->{'logarithme'} )
		and ( $my_config->{'logarithme'} ) )
	{
		if ( !$ymin ) {
			$ymin = 0.1;
		}

		# log scale
		@y_scale = ( '-l', $ymin, '--logarithmic' );
	}
	else {

		# classic scale
		@y_scale = ( '-l', $ymin );
	}
	$minfo->{'custom_ymin'} = $ymin;

	# y-axis : max
	if (    ( exists $minfo->{'custom_ymax'} )
		and ( defined $minfo->{'custom_ymax'} ) )
	{
		my $ymax = $minfo->{'custom_ymax'};

		push @y_scale, ( '-u', $ymax );
	}

	if ( exists $minfo->{'custom_rigid'} ) {
		push @y_scale, '--rigid';
	}

	# date of end of graph in second : 0 for now
	my $end;
	if (    ( exists $minfo->{'custom_end'} )
		and ( defined $minfo->{'custom_end'} ) )
	{

		# to create graph in the past (batch_graph)
		$end = $minfo->{'custom_end'};
	}
	else {

		# must be a value (not 'now'), because we have some operation,
		# so we asked -1 second
		$end = -1;
	}
	$minfo->{'custom_end'} = $end;

	my ( $xsize, $ysize );
	if ( exists $minfo->{'custom_period'} ) {

		# call from display_custom/batch_graph
		my $title = $minfo->{'module_name'};
		my $start = $end - $minfo->{'custom_period'};

		( $xsize, $ysize ) = build_graph(
			$minfo, 'custom', $start,    $end, $sizex,
			$sizey, $title,   \@y_scale, @rrd_detals
		);
	}
	else {

		# classic display : from display.cgi/batch_graph
		foreach my $graph ( list_periods() ) {
			my $title = $text{$graph};

			my $start = $end - get_periode( get_rra_index($graph) );

			( $xsize, $ysize ) = build_graph(
				$minfo, $graph, $start,    $end, $sizex,
				$sizey, $title, \@y_scale, @rrd_detals
			);
		}
	}

	# return real size, to allow build clean html code
	$minfo->{'graph_sizex'} = $xsize;
	$minfo->{'graph_sizey'} = $ysize;
	debug_sub_end();
	return;
}
###############################################################################
# a special code to display internal data
sub graph_internal_data($$;$) {
	my ( $minfo, $my_config, $opt ) = ( $_[0], $_[1], $_[2] );
	debug_sub_begin();

	my $database = $minfo->{'dbtables'} . 'internal.rrd';
	if ( -f $database ) {
		my $cf          = $my_config->{'cf'};
		my $module_name = $my_config->{'name'};
		my @rrd_detals;
		gprint_title( $module_name, 'internal', \@rrd_detals );
		add_def( \@rrd_detals, 'total', $database, 'total', $cf );
		gprint_param( \@rrd_detals, 'total', 'duration', 'ff0000:LINE2' );
		graph_common( $minfo, $my_config, @rrd_detals );
	}
	else {
		info("internal database $database not found");
	}
	debug_sub_end();
	return;
}

###############################################################################
sub add_meta($$$) {
	my $minfo = shift @_;
	my $tag   = shift @_;
	my $value = shift @_;

	if ( ( exists $minfo->{$tag} ) and ( defined $minfo->{$tag} ) ) {
		debug("add_meta skip already set $tag");
	}
	else {
		$minfo->{$tag} = $value;
	}
	return;
}
###############################################################################
# list of "classic" graphs
sub list_periods() {
	return ( 'hour', 'day', 'week', 'month', 'year' );
}
###############################################################################
# common code for all modules, to create the same rrd files
sub add_rra ($) {
	my $cf = shift @_;    # consolidation fonction

	debug_sub_begin();

	if ( !defined $cf ) {
		$cf = 'AVERAGE';
	}

	# cf https://lists.oetiker.ch/pipermail/rrd-users/2002-May/004583.html
	# for computers which are not running all the day
	# should allow monthly/yearly graphs with less holes
	my $xff = 0.75;

	my $cmd  = $EMPTY;
	my @data = get_rra_param();    # from sysstats-rrd-lib.pl
	foreach my $dat (@data) {
		$cmd .= " RRA:$cf:$xff:" . $dat->{'period'} . $PERIOD . $dat->{'nr'};
	}

	debug_sub_end();
	return $cmd;
}
###############################################################################
# (re)write module configuration file
sub common_write_config($$) {
	debug_sub_begin();
	my ( $minfo, $my_config ) = ( $_[0], $_[1] );

	# save old one
	rename $minfo->{'config_file'}, $minfo->{'config_file'} . '.old';

	# add version
	$my_config->{'version'} = $minfo->{'module_ver'};

	write_file_hash( $minfo->{'config_file'}, $my_config );

	# config file may contain passwords : so restrict perms
	chmod oct 600, $minfo->{'config_file'};

	debug_sub_end();
	return;
}
##########################################################
# heartbeat (cf rrdtool) is the max period available between 2
# data, before marking a 'hole' (NaN : not a number) in aquisition
# in seconds
# if the load is heavy, cron task may be longer than expected
sub compute_heartbeat($) {
	debug_sub_begin();
	my $sample_rate = shift @_;    # sample rate (in minutes)

	# take period + 50 % + 30 s
	# exemple : rate = 1 (minute) -> heart = 102
	#                  2          -> heart = 174
	#		   3          -> heart = 246
	#                  4          -> heart = 318
	my $heartbeat = ( 60 * $sample_rate * 2.5 );
	debug_sub_end();
	return int $heartbeat;
}
###############################################################################
# allow to check/change heartbeat (see rrdtool)
sub check_heartbeat ($$) {
	debug_sub_begin();
	my ( $minfo, $my_config ) = ( $_[0], $_[1] );

	my $new_heartbeat = compute_heartbeat( $my_config->{'sample_rate'} );

	my @rrd_list = get_rrd_names($minfo);

	# could be empty with modules such generic or custom !
	if ( ( scalar @rrd_list ) != 0 ) {
		my $old_heartbeat = get_heartbeat( ( get_rrd_names($minfo) )[0] );

		change_heartbeat( $minfo, $new_heartbeat )
		  if ( $old_heartbeat != $new_heartbeat );
	}

	debug_sub_end();
	return;
}
##########################################################
# get a new sample rate from url arguments
# and modify config hash
sub test_sample_rate($$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;

	my $old_sample_rate = $my_config->{'sample_rate'};

	my $new_sample_rate = $in{'sample_rate'};

	# validity test
	if ( !is_unsigned_int($new_sample_rate) ) {
		warning("$new_sample_rate is not a valid sample rate");
		return;
	}

	# detect change
	if ( $old_sample_rate != $new_sample_rate ) {
		$my_config->{'sample_rate'} = $new_sample_rate;
		change_heartbeat( $minfo, compute_heartbeat($new_sample_rate) );
	}
	debug_sub_end();
	return;
}
##########################################################
# try to generate very differents color : primary, then mixed
sub generate_colors() {
	debug_sub_begin();

	# rgb (red/green/blue) colors
	my @tab = (
		'000000',    # noir
		'ff0000',    # rouge
		'00ff00',    # tilleul (vert clair)
		'0000ff',    # bleu
		'ffff00',    # jaune
		'00ffff',    # cyan
		'ff00ff',    # magenta/fushia
		'008000',    # vert
		'800000',    # marron
		'808080',    # gris
		'808000',    # olive
		'ba5716',    # marronclair
		'ffa500',    # orange
		'008080',    # bleu-moyen
		'7fff00',    # vert-clair
		'800080',    # violet
		'c0c000',    # kaki
		'000080',    # bleu fonce
		'04a000',    # vert-fonce
		'f7f74c',    # jaune clair
	);

	debug_sub_end();
	return @tab;
}
##########################################################
# set a new entry $key in hash table my_config if $fic exists
# used to monitor log files
sub set_config_file($$$) {
	debug_sub_begin();
	my $my_config = shift @_;
	my $key       = shift @_;
	my $fic       = shift @_;

	if ( !test_file($fic) ) {

		# file does not exists
		$my_config->{$key} = $EMPTY;
		$my_config->{'runstop'} = 1;
	}
	else {

		# ok : file exists
		$my_config->{$key} = $fic;
		$my_config->{'runstop'} = 0;
	}

	debug_sub_end();
	return;
}
##########################################################
# for 0.9.5 to be used in modules
sub checked($) {
	my $arg = shift @_;
	return $arg ? 'checked' : $EMPTY;
}
##########################################################
sub common_clear_cache($) {
	my $module = shift @_;

	if ( !chdir $graph_dir ) {
		warning("(clean_graph) can not chdir to $graph_dir : $ERRNO");
		return;
	}

	my $ALL     = STAR();
	my @allfics = glob $ALL;

	# security : remove very old graphs (older than 1 day)
	# may occur if the computer is stopped when this script is sleeping
	foreach my $fic (@allfics) {
		unlink $fic if ( ( -e $fic ) and ( -M _ > 1 ) );
	}

	# cleaning
	if ( defined $module ) {

		# only clean graph from specified module
		unlink glob $module . $ALL;
	}
	else {

		# no module, clean all graphs
		unlink glob $ALL;
	}
	chparent();
	return;
}
##########################################################
# common code for all display
# clean cache immediatly
sub clear_cache ($) {
	my $module_name = shift @_;

	debug_sub_begin();
	common_clear_cache($module_name);
	debug_sub_end();
	return;
}
##########################################################
# common code for all display
# clean cache after a delay (60 s)
# v1 : system (nohup $cmd & )
#sub delayed_clear_cache_v1($) {
#	my $module_name = shift @_;
#	debug_sub_begin();
#
#	# "old" method : depends upon shell syntaxe (bourne/posix)
#	my $nohup_cmd = search_command('nohup');
#
#	my $cmd =
#	    "$nohup_cmd "
#	  . sysstat_dir()
#	  . "/clean_graph.pl 60 $module_name  > /dev/null &";
#
#	$ENV{'SHELL'} = '/bin/sh';
#	system $cmd;
#
#	debug_sub_end();
#	return;
#}
##########################################################
# v2 : system (  at -f $cmd now + 1 minutes)
# we have to create a temp file (for exe with tmpnam from POSIX
# write the command to it, then launch at
# just do not forget to remove the temporary file

#sub delayed_clear_cache_v2(;$) {
#	my $module_name = shift @_ || q{};
#	debug_sub_begin();
#
#	my $cmd;
#
#	# does at exists ?
#	# we do not check if atd is running !
#	my $at_cmd = search_command('at');
#	if ($at_cmd) {
#
#		# create a uniq file name
#		my $name = tmpnam();    # from POSIX
#		my $fh;
#		## no critic (RequireBriefOpen)
#		if ( ( !-f $name ) && ( open $fh, '>', $name ) ) {
#
#			print {$fh} sysstat_dir() . "clean_graph.pl 60 $module_name\n";
#			print {$fh} "rm -f $name\n";    # remove it-self
#			my_close( $fh, $name );
#
#			$cmd = "$at_cmd -f $name now + 1 minutes 2>&1";
#
#			debug($cmd);
#			$ENV{'SHELL'} = '/bin/sh';
#			## no critic (ProhibitBacktickOperators)
#			my $rep = `$cmd`;
#			## use critic
#
#			# check output
#			if ( $rep =~ m/No atd running/ ) {
#				warning('check if atd daemon is running');
#			}
#		}
#		else {
#
#			# no temp file
#			old_delayed_clear_cache($module_name);
#		}
#		## use critic
#	}
#	else {
#
#		# no at command
#		old_delayed_clear_cache($module_name);
#	}
#
#	debug_sub_end();
#	return;
#}
##########################################################
# v3 will work only on unix
sub delayed_clear_cache(;$) {
	my $module_name = shift @_ || $EMPTY;
	debug_sub_begin();

	## no critic (ProhibitUnlessBlocks)
	unless (fork) {

		# child
		$ENV{'PATH'} = "$ENV{'PATH'}:.";
		exec 'clean_graph.pl', 60, $module_name;
	}
	debug_sub_end();
	return;
}

##########################################################
# this stamp is used for all display to build a graph name which
# change :
# - is used as a cache by display*.cgi to avoid building graph
# - is also used to avoid browser cache
sub png_stamp() {

	# time stamp for web refresh (in minutes)
	my @tps = localtime;

	# stamp in minutes
	# see also clean_graph.pl
	my $stamp = DOT() . $tps[2] . $tps[1];

	# change stamp for network and other multi-display modules
	if ( exists $in{'opt'} ) {
		$stamp .= q{o} . $in{'opt'};
	}

	# name depends also upon gtype : so change display will change graphs
	if ( exists $in{'gtype'} ) {
		$stamp .= q{g} . $in{'gtype'};
	}

	# same for display
	if ( exists $in{'display'} ) {
		$stamp .= q{d} . $in{'display'};
	}

	return $stamp;
}
##########################################################
# see also graph_common and display*.cgi
sub build_graph_name($$) {
	my $minfo  = shift @_;
	my $period = shift @_;    # may be : custom, hour, day (see list_periods)

	if ( $period eq 'custom' ) {
		$period .= $minfo->{'custom_period'};
		if ( exists $minfo->{'custom_end'} ) {
			$period .= 'e' . $minfo->{'custom_end'};
		}
		else {
			$period .= 'e-1';
		}
	}
	return $minfo->{'tmpgname'} . '_' . $period . '.png';
}
##########################################################
sub build_tmpgname($) {
	my $module_name = shift @_;

	return $graph_dir . $SLASH . $module_name;
}
##########################################################
# generic pack and unpack sub to store more info
# without changing config line
sub gen_pack($$$) {
	my $sep  = shift @_;
	my $var1 = shift @_;
	my $var2 = shift @_;

	return join $sep, $var1, $var2;
}
##########################################################
sub gen_unpack($$$) {
	my $sep  = shift @_;
	my $val  = shift @_;
	my $def2 = shift @_;    # default value for second parameter

	my ( $var1, $var2 ) = split /$sep/, $val, 2;
	$var2 = $def2 if ( !defined $var2 );
	return ( $var1, $var2 );
}
##########################################################
# extract color and thickness
sub get_line_config($) {
	my $val = shift @_;

	my ( $colour, $thick ) = gen_unpack( $PERIOD, $val, $defline );
	return ( $colour, $thick );
}
##########################################################
sub set_line_config($$) {
	my $colour = shift @_;
	my $thick  = shift @_;
	return gen_pack( $PERIOD, $colour, $thick );
}
##########################################################
# set color and thickness
sub line_col($$) {
	my $name     = shift @_;
	my $defaults = shift @_;
	return set_line_config( get_colourselect( $name, $defaults ),
		get_lineselect($name) );
}
##########################################################
# convert periods in seconds
sub get_period_scale($) {
	my $scale   = shift @_;
	my %hperiod = (
		'hour'  => 3_600,
		'day'   => 86_400,
		'week'  => 604_800,
		'month' => 2_678_400,
		'year'  => 31_536_000,
	);

	if ( exists $hperiod{$scale} ) {
		return $hperiod{$scale};
	}
	else {
		warning("unknown period $scale");
		return;
	}
}
##########################################################
# copy an old base to a new place
# used when upgrading a module and change names
sub update_base($$$) {
	debug_sub_begin();
	my $minfo   = shift @_;
	my $oldname = shift @_;
	my $newname = shift @_;

	my $oldbase = $minfo->{'dbtables'} . $SLASH . $oldname;

	# copy old database
	if ( -f $oldbase ) {
		my $newbase = $minfo->{'dbtables'} . $SLASH . $newname;

		# from File::Copy
		if ( move( $oldbase, $newbase ) ) {
			debug("move $oldbase into $newbase\n");
		}
		else {
			warning("pb to move $oldbase to $newbase : $ERRNO");
		}
	}
	else {
		warning("can not find database $oldbase\n");
	}
	debug_sub_end();
	return;
}
##########################################################
# cf = condolidation function
# see change_cf sub in sysstats-web-lib
sub test_cf($$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;

	my $old_cf = $my_config->{'cf'};
	my $new_cf = $in{'cf'};

	# control entry
	# entry comes from a select so no human error possible
	# an attacker may try to change it in url
	my @cf_list = ( 'AVERAGE', 'MAX', 'MIN', 'LAST' );
	my $ko = 1;
	foreach my $cf (@cf_list) {
		if ( $new_cf eq $cf ) {
			$ko = 0;
			last;
		}
	}
	if ($ko) {
		warning("cf parameter : $new_cf is not a valid value");
		return;
	}

	if ( $old_cf ne $new_cf ) {
		debug("change cf from $old_cf to $new_cf\n");

		$my_config->{'cf'} = $in{'cf'};

		# change cf in databases
		module_mod_cf( $minfo->{'config_dir'}, $old_cf, $new_cf );
	}

	debug_sub_end();
	return;
}
##########################################################
# available value are "U" or numeric
sub check_minmax($) {
	my $value = shift @_;

	return ( ( $value eq 'U' ) or is_signed_real($value) );
}
##########################################################
# test for min/max arg from url and apply
sub test_minmax($$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;

	# control entry
	if ( !check_minmax( $in{'minimum'} ) ) {
		warning("$in{'minimum'} is not a valid minimum value (U or numeric)");
		return;
	}
	if ( !check_minmax( $in{'maximum'} ) ) {
		warning("$in{'maximum'} is not a valid maximum value (U or numeric)");
		return;
	}

	module_mod_minmax( $minfo->{'config_dir'}, $in{'minimum'}, $in{'maximum'} );

	debug_sub_end();
	return;
}
##########################################################
# test changes set by change_database
sub test_database_change($$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;

	test_sample_rate( $minfo, $my_config );
	test_cf( $minfo, $my_config );
	test_minmax( $minfo, $my_config );

	debug_sub_end();
	return;
}
##########################################################
# convert a period coded as '10d' to '(10  , days )'
sub period2comp($) {
	my $period = shift @_;

	my %period_scale = (
		H => 'hour',
		D => 'day',
		W => 'week',
		M => 'month',
		Y => 'year',
	);

	if ( $period =~ m/(\d+)(\w)/ ) {
		my $delai = $1;
		my $scale = uc $2;

		if ( exists $period_scale{$scale} ) {
			return ( $delai, $period_scale{$scale} );
		}
		else {
			warning("can not find scale for $scale");
			return;
		}
	}
	else {
		warning("can not decode $period");
		return;
	}
}
##########################################################
# transform a format like 10H in seconds
sub period2sec($) {
	my $period = shift @_;

	my $s_period;    # in seconds

	if ( defined $period ) {
		my ( $delai, $scale ) = period2comp($period);

		if ( defined $scale ) {
			$s_period = get_period_scale($scale) * $delai;
		}
		else {
			warning(
"bad period (scale=$scale, delai=$delai), use H for hours, D for day, W for week, M for month, Y for years"
			);
		}
	}
	return $s_period;

}
##########################################################
# get cgi parameters for graph_data sub
sub get_cgi_graph($) {
	my $my_config = $_[0];
	debug_sub_begin();

	my $graph_type = $my_config->{'graph_type'};

	# take care of cgi parameters
	if ( exists $in{'gtype'} ) {
		$graph_type = $in{'gtype'};
	}

	# current display
	my $c_display;
	if ( exists $in{'display'} ) {
		$c_display = $in{'display'};
	}
	elsif ( exists $ENV{'display'} ) {
		$c_display = $ENV{'display'};
	}
	else {
		$c_display = 0;
	}
	debug_sub_end();
	return ( $graph_type, $c_display );
}
##########################################################
# get list of display from config
# keys are numbers : -1, 0, 1 ...
# values are text
sub get_display_list($) {
	my $my_config = $_[0];

	my %list;
	foreach my $opt ( keys %{$my_config} ) {
		if ( $opt =~ m/gdisplay(.*)/ ) {
			my $gdisplay = $1;
			$list{$gdisplay} = $my_config->{$opt};
		}
	}
	return %list;
}
##########################################################
# get new display from url arg
sub test_for_new_display($;$) {
	my $my_config = $_[0];
	my $name = $_[1] || 'display_def';

	my $new_display = $in{$name};

	# test for display : force to create an dummy label
	$my_config->{ 'gdisplay' . $new_display } = $new_display
	  if ( !exists $my_config->{ 'gdisplay' . $new_display } );
	return;
}
##########################################################
# convert module name (cpu) to the real directory name (cpu-1.0)
sub build_full_name($) {
	my $minfo = shift @_;

	return $minfo->{'module_name'} . DASH() . $minfo->{'module_ver'};
}
##########################################################
# set common config for a module
sub init_module_config($$) {
	my $name          = shift @_;
	my $flag_gdisplay = shift @_;

	my %config = (
		'cf'          => 'AVERAGE',
		'db_uid'      => 1,           # index for next rrd database
		'graph_type'  => 1,
		'keep'        => 0,           # dynamic/static
		'name'        => $name,
		'nb_graph'    => 2,           # 2 graphs (normal/log)
		'runstop'     => 0,           # run
		'sample_rate' => 1,           # every minute
	);

	if ($flag_gdisplay) {
		$config{'gdisplay0'}  = 'default';
		$config{'gdisplay-1'} = 'stop';
	}
	return %config;
}
##########################################################
# used to pack/unpack a parameter in one line for myconfig
sub pack_config(@) {
	return join COMMA(), @_;
}
##########################################################
sub unpack_config($) {
	my $line = shift @_;
	return split /,/, $line;
}
##########################################################
# to avoid warning about uninitialised var
# will be used to fix the problem with apache server
sub webprefix() {
	return ( defined $gconfig{'webprefix'} )
	  ? $gconfig{'webprefix'} . $SLASH
	  : $EMPTY;
}
##########################################################
# return a sorted (by name) list of installed module hash
# of config file
sub get_mod_list($) {
	my $filter = shift @_;    # if set, only return running modules

	debug_sub_begin();
	my @mod_list;
	my $fh_module;
	if ( !opendir $fh_module, $module_dir ) {
		warning("can not open $module_dir : $ERRNO");
		return;
	}
	else {
	  MOD: foreach my $m ( readdir $fh_module ) {
			## no critic (ProhibitNoisyQuotes)
			next MOD if ( $m eq '.' || $m eq '..' );
			## use critic
			# do not warn about modules to install
			## no critic (ProhibitEscapedMetacharacters)
			next MOD if ( $m =~ m/\.tar\.gz$/ );
			my $m_path = $module_dir . $SLASH . $m;
			if ( !-d $m_path ) {
				warning("$m is not a directory");
				next MOD;
			}

			my $info_path = $module_dir . $SLASH . $m . '/module.info';
			if ( !-r $info_path ) {
				warning("can not read module.info for $m");
				next MOD;
			}

			my %minfo = get_sysstats_module_info($m);

			if ($filter) {

				# we have to check for running modules
				my %my_config;
				if ( !read_module_config( \%my_config, $minfo{'config_file'} ) )
				{
					text_error( 'ecannot_readconfig', $minfo{'config_file'} );
				}
				if ( !mod_check_runstop( \%minfo, \%my_config ) ) {
					push @mod_list, \%minfo;
				}
			}
			else {

				# just return all modules
				push @mod_list, \%minfo;
			}
		}

		closedir $fh_module
		  or warning("can not close directory $module_dir : $ERRNO");
		debug_sub_end();

		# sort by module name
		my @sorted_mod_list =
		  sort { ( lc $a->{'module_name'} ) cmp( lc $b->{'module_name'} ) }
		  @mod_list;
		return @sorted_mod_list;
	}
}
##########################################################
sub is_module_installed($) {
	my $filter = shift @_;    # searched module

	my @mod_list = get_mod_list(0);    # all module
	foreach my $mod (@mod_list) {
		my $mod_name = $mod->{'idx'};
		if ( $mod_name eq $filter ) {
			return $mod;
		}
	}
	return 0;
}
###############################################################################
# 		section : display release status
###############################################################################
{

	# local variable to store web version page
	my %dist_versions;

	#return list of versions from sourceforge web page
	# or undef
	sub get_dist_version() {
		debug_sub_begin();
		if ( !%dist_versions ) {

# version 1 : LWP::Simple , very easy and simple to code, but can not configure timeout
#			my $url = 'http://webminstats.sourceforge.net/version';
#			eval {require  LWP::Simple};
#			if ($EVAL_ERROR) {
#				warning( #'missing library LWP : can not get sourceforge versions list');
#				return;
#			}
#			$content =  LWP::Simple::get($url);

 # version 2 : LWP::UserAgent : a little bit more complex
 #			eval { require LWP::UserAgent };
 #			if ($EVAL_ERROR) {
 #				warning( #'missing library LWP : can not get sourceforge versions list'	);
 #				return;
 #			}
 #			else {
 #				my $ua = LWP::UserAgent->new();
 #
 #				# check if a proxy is set in webmin
 #				if ($gconfig{'http_proxy'}) {
 #					# proxy should be a full url
 #					$ua->proxy( 'http', $gconfig{'http_proxy'} );
 #				}
 #
 #				$ua->timeout(5);
 #				my $response = $ua->get($url);    # HTTP::Response
 #				if ( $response->is_success ) {
 #
 #					# get file content
 #					my $content = $response->content;
 #
 #					# parse each line to search for product
 #					my @dist_versions = split /\n/, $content;
 #
 #					# now build hash
 #					foreach my $elem (@dist_versions) {
 #						my ( $d_prod, $d_version ) = split q{ }, $elem;
 #						$dist_versions{$d_prod} = $d_version;
 #					}
 #				}
 #				else {
 #					warning( 'can not get version from internet site : '
 #						  . $response->status_line() );
 #					return;
 #				}
 #			}
 #
 #			# now build hash
 #			foreach my $elem (@dist_versions) {
 #			 	my ( $d_prod, $d_version ) = split q{ }, $elem;
 #			 	$dist_versions{$d_prod} = $d_version;
 #			}

# version 3 : use webmin api make_http_connection/read_http_connection/close_http_connection
# it uses also the webmin proxy and do not need extra perl module
			my $host = 'webminstats.sourceforge.net';
			my $handle = wrapper( 'make_http_connection', $host, 80, 0, 'GET',
				'/version' );
			if ( !ref $handle ) {
				warning( 'can not get version from internet site :' . $handle );
				return;
			}

			write_http_connection( $handle, "Host: $host\r\n" );
			write_http_connection( $handle, "User-agent: Webmin\r\n" );
			write_http_connection( $handle, "\r\n" );

			while (
				defined(
					my $line = wrapper( 'read_http_connection', $handle )
				)
			  )
			{

				# we got also http header : filter lines
				if ( $line =~ m/^(\w+) ([\d.]+)/ ) {
					my $d_prod    = $1;
					my $d_version = $2;
					$dist_versions{$d_prod} = $d_version;
					debug("find $d_prod version $d_version");
				}
			}
			close_http_connection($handle);

		}
		debug_sub_end();
		return %dist_versions;
	}
}
###############################################################################
sub split_number($) {
	my $ver = shift @_;

	return split /[.]/, $ver;
}
###############################################################################
# use same return as cmp or <=>
# there is a problem with version like
# 1.9 and 1.10 : 1.10 is seem as 1.1
sub compare_version($$) {
	my $v1 = shift @_;
	my $v2 = shift @_;

	my @av1 = split_number($v1);
	my @av2 = split_number($v2);

	my $max1 = scalar @av1;
	my $max2 = scalar @av2;
	if ( $max1 != $max2 ) {
		warning("not same size : $v1 and $v2");
		return;
	}
	my $i = 0;
	while ( $i < $max1 ) {
		if ( $av1[$i] < $av2[$i] ) {

			return -1;
		}
		elsif ( $av1[$i] > $av2[$i] ) {

			# if the difference if too big, these may be the bug
			if ( $av1[$i] - $av2[$i] > 5 ) {

				# if we have 1.9 % 1.1 (1.1 or 1.10 ?)
				return -1;
			}
			else {
				return 1;
			}
		}
		else {
			$i++;
		}
	}
	return 0;
}

###############################################################################
sub is_core($) {
	my $cur_product = shift @_;
	return ( $cur_product eq 'webminstats' );
}
###############################################################################
sub update_value($) {
	my $cur_product = shift @_;

	my $ret;
	if ( is_core($cur_product) ) {
		$ret = 100;    # core release
	}
	else {
		$ret = 1;      # module release
	}
	return $ret;
}
###############################################################################
sub get_download_url($$$$$) {
	my $product      = shift @_;
	my $version      = shift @_;
	my $core_version = shift @_;
	my $beta_flag    = shift @_;
	my $update_flag  = shift @_;

	if ( !$version ) {
		return '&nbsp';
	}
	elsif ( is_core($product) ) {
		if ($beta_flag) {
			return html_link(
"https://sourceforge.net/projects/webminstats/files/Sysstats-devel/$version/sysstats-$version.tgz/download",
				$version
			);
		}
		else {
			return html_link(
"https://sourceforge.net/projects/webminstats/files/Sysstats/$version/sysstats-$version.tgz/download",
				$version
			);
		}
	}
	else {

		# modules
		if ($beta_flag) {
			return html_link(
"https://sourceforge.net/projects/webminstats/files/modules-devel/$core_version/$product-$version.tar.gz/download",
				$version
			);
		}
		elsif ($update_flag) {
			return html_link(
"https://sourceforge.net/projects/webminstats/files/updates/$core_version/$product-$version.tar.gz/download",
				$version
			);
		}
		else {
			return html_link(
"https://sourceforge.net/projects/webminstats/files/modules/$core_version/$product-$version.tar.gz/download",
				$version
			);
		}
	}
}
###############################################################################
# common colors for module version status
my $color_ok     = COLOR_OK();
my $color_beta   = COLOR_BETA();
my $color_update = COLOR_UPDATE();

# check if a new release is available for one product
sub check_product_update($$$$) {
	my $cur_product = shift @_;
	my $cur_version = shift @_;    # installed version
	my $level =
	  shift @_;    # level 1 to only print status; level 2 for full status
	my $rh_dversions =
	  shift @_;    # last available version : reference from web site

	debug_sub_begin();

	my $ret = 0;
	my $color;
	if ( !exists $rh_dversions->{$cur_product} ) {

		# may occur for clones
		# same as a beta
		debug("can not find version for $cur_product");
		$ret   = 1000;
		$color = $color_beta;
	}
	else {

		# get last stable version
		my $last_version = $rh_dversions->{$cur_product};

		#info("module : $cur_product ($cur_version) last : $last_version" );
		# get last beta version
		my $beta_name = $cur_product . '-devel';
		my $last_beta;
		if ( exists $rh_dversions->{$beta_name} ) {
			$last_beta = $rh_dversions->{$beta_name};
		}

		# can get version file
		# then compare with current version
		# we assume version are in the form : x.y
		# and beta release contains aphabetic char

		if ( $cur_version =~ m/[^\d.]/ ) {

			# the current release is a beta
			my $cur2_version = $cur_version;
			$cur2_version =~ s/[^\d.]//g;
			my $res = compare_version( $cur2_version, $last_version );
			if ( $res == 0 ) {

			# same release number, but the locale version is a beta : can update
				$ret   = update_value($cur_product);
				$color = $color_update;
			}
			elsif ( $res == -1 ) {

				# locale version seems less than ref : can update
				$ret   = update_value($cur_product);
				$color = $color_update;
			}
			elsif ( $res == 1 ) {

				# locale version seems greater than ref : real beta
				$ret   = 1000;
				$color = $color_beta;
			}
		}
		else {

			# no beta release
			my $res = compare_version( $cur_version, $last_version );
			if ( $res == 0 ) {

				# ok : same version
				$ret   = 0;
				$color = $color_ok;
			}
			else {

				# the current release can not be > reference release
				$ret   = update_value($cur_product);
				$color = $color_update;
			}
		}

		# beta release ?
		if ( $level == 2 ) {

			# core version
			my $core_version = $rh_dversions->{'webminstats'};

			#begin_row();
			my @columns;
			push @columns, html_color( $color, $cur_product );
			push @columns, $cur_version;
			push @columns,
			  get_download_url( $cur_product, $last_version, $core_version, 0,
				$ret );
			push @columns,
			  get_download_url( $cur_product, $last_beta, $core_version, 1, 0 );

			print wrapper( 'ui_columns_row', \@columns );
		}
	}
	debug_sub_end();
	return $ret;
}
###############################################################################
# level is used to control output :
# 1 display only a summary (ok/ko)
# 2 display a full state of all modules
# return the number of modules not ok with the web site
sub check_update($$) {
	my $rh_prod = shift @_;
	my $level   = shift @_;
	debug_sub_begin();

	my %dversions = get_dist_version();
	return if ( !%dversions );

	if ( $level == 2 ) {
		my @columns = (
			$text{check_name},      $text{check_current},
			$text{check_available}, $text{'check_betav'},
		);
		print wrapper( 'ui_columns_start', \@columns );
	}

	my $ret   = 0;
	my $nb_ko = 0;
	foreach my $prod ( sort keys %{$rh_prod} ) {

		# check for clones
		if ( is_clone($prod) ) {

			# replace clone name by the orig name
			$prod = is_clone($prod);
		}
		$nb_ko +=
		  check_product_update( $prod, $rh_prod->{$prod}, $level, \%dversions );
	}
	if ( $level == 1 ) {
		if ( $nb_ko == 0 ) {

			# ok : use last core and module
			print html_color( $color_ok, $text{'check_ok'} ) . '<br>';
		}
		elsif ( $nb_ko >= 1000 ) {

			# using beta
			print html_color( $color_beta, $text{'check_beta'} ) . '<br>';
			$ret = 1;
		}
		elsif ( $nb_ko >= 100 ) {

			# a new core is available
			print html_color( $color_update, $text{'check_major'} ) . '<br>';
			$ret = 1;
		}
		else {

			# new release(s) available
			print html_color( $color_update, $text{'check_minor'} ) . '<br>';
			$ret = 1;
		}
	}
	elsif ( $level == 2 ) {

		#print "</table>\n";
		print wrapper('ui_columns_end');
	}
	debug_sub_end();
	return $ret;
}
###############################################################################
# a function to force a default empty value,
# avoid warning about undefined variables
sub clean_var($) {
	my $var = shift @_;
	return ( defined $var ) ? $var : q{};
}
###############################################################################
# get a webmin config parameter
# can be global or for a user
sub get_gconfig($) {
	my $key = shift @_;

	my $res;
	my $remote_user = $ENV{'REMOTE_USER'};
	my $user_key    = $key . '_' . $remote_user;
	if ( ($remote_user) and ( defined $gconfig{$user_key} ) ) {
		$res = $gconfig{$user_key};
	}
	elsif ( ( defined $gconfig{$key} ) ) {
		$res = $gconfig{$key};
	}
	else {
		$res = undef;
	}
	return $res;
}
###############################################################################
1;
