
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a lib dedicated to rrdtools sub
# the code was duplicated from post-install to be integrated in a lib
# migration is not ended (it will done when post-install will use this lib)
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

use File::Basename;
use Cwd;

# require, not use, to allow error diagnostic (index.cgi)
require RRDs;

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars qw(  %config %text );
## use critic

## no critic (RequireBarewordIncludes)
require 'sysstats-gen-lib.pl';    # chparent
## use critic

my $stamp_ext = '.stamp';    # stamp extension use to mark if a check was done
###############################################################################
# restore xml to rrd and clean
sub rrdtool_restore($$) {
	my $ficxml = shift @_;    # xml source
	my $ficrrd = shift @_;    # rrd dest

	unlink $ficrrd;
	rrdtool( 'restore', $ficxml, $ficrrd );
	unlink $ficxml;

	return;
}
###############################################################################
# call external rrdtool command
# note : all rrdtool command do not exist in RRDs perl module
# so I have to system call to rrdtool (restore, resize)
sub rrdtool($@) {
	my $cmd = shift @_;    # restore, resize
	my @arg = @_;

	system $config{rrdtool}, $cmd, @arg;
	return;
}
###############################################################################
# call external rrdtool command in a pipe
# note : all rrdtool command do not exist in RRDs perl module
# so I have to system call to rrdtool (dump, xport )
sub piped_rrdtool($$) {
	my $cmd  = shift @_;    # dump, xport
	my $args = shift @_;

	my $fh;
	## no critic (ProhibitTwoArgOpen,RequireBriefOpen)
	if ( !open $fh, "$config{rrdtool} $cmd $args |" ) {
		warning("can not open rrdtool  $cmd $args : $ERRNO");
		return;
	}
	else {
		return $fh;
	}
	## use critic
}
###############################################################################
# return list rrdtool database
sub get_rrd_liste() {
	my @liste = glob '*.rrd';
	return @liste;
}

###############################################################################
# define rra variable : period and number of rows
sub get_rra_param() {

	my @params;

	# first data (hourly) : each minute * 70 records = 1h10
	$params[0]->{'period'} = 1;
	$params[0]->{'nr'}     = 70;

	# second data (daily) : each 2 minutes * 750 rec = 25h
	$params[1]->{'period'} = 2;
	$params[1]->{'nr'}     = 750;

	# third data (weekly) : each 15 minutes * 770 rec = 8j
	$params[2]->{'period'} = 15;
	$params[2]->{'nr'}     = 770;

	# fourth data (monthly) : each 60 minutes * 2200 rec = 91 jours
	$params[3]->{'period'} = 60;
	$params[3]->{'nr'}     = 2200;

	# fith data (yearly) : each 12h = 720 minutes * 730 rec = 365 jours
	$params[4]->{'period'} = 720;
	$params[4]->{'nr'}     = 730;

	return @params;
}
###############################################################################
# return the "size" in seconds of an rra
sub get_periode($) {
	my $index = shift @_;    # 0 to 4

	my @params = get_rra_param();

	return $params[$index]->{'period'} * $params[$index]->{'nr'} * 60;
}
###############################################################################
# convert a period in human form (hour, day ...) to an index (0 to 4)
sub get_rra_index($) {
	my $period = shift @_;    # hour, day, ...

	my %index =
	  ( 'hour' => 0, 'day' => 1, 'week' => 2, 'month' => 3, 'year' => 4 );

	if ( exists $index{$period} ) {
		return $index{$period};
	}
	else {
		warning("unknown period $period");
		return;
	}
}
###############################################################################
# a function to have an rrd grow to keep more records
sub grow_rra_nr($$@) {
	my $period    = shift @_;  # period to work on in human form (hour, day ...)
	my $tail_min  = shift @_;  # number min of records
	my @liste_rrd = @_;

	debug_sub_begin();
	my $index = get_rra_index($period);

	# stamp
	my $stamp = $period . '-size-' . $tail_min . $stamp_ext;
	if ( !-e $stamp ) {

		foreach my $fic (@liste_rrd) {

			# get number of records
			my $info   = RRDs::info($fic);
			my $taille = $info->{"rra[$index].rows"};

			if ( $taille < $tail_min ) {
				my $ajout = $tail_min - $taille;
				warning("grow_rra_nr $period on $fic add $ajout records");

				rrdtool( 'resize', $fic, $index, 'GROW', $ajout );
				my_rename( 'resize.rrd', $fic );
			}
			else {
				debug("grow_rra_nr $period on $fic : size $taille already ok");
			}
		}
		touch($stamp);
	}
	else {
		debug("grow_rra_nr $period already stamped");
	}
	debug_sub_end();
	return;
}
######################################################
# a function to change an rra pdp (interval between 2 records in minutes)
sub change_rra_pdp($$@) {
	my $period    = shift @_;  # period to work on in human form (hour, day ...)
	my $pdp       = shift @_;  # in minutes as returned by get_rra_param
	my @liste_rrd = @_;

	debug_sub_begin();
	my $index = get_rra_index($period);

	# stamp
	my $stamp = $period . '-pdp-' . $pdp . $stamp_ext;
	if ( !-e $stamp ) {

	  RRD: foreach my $ficrrd (@liste_rrd) {

			# get period (pdp)
			my $info        = RRDs::info($ficrrd);
			my $current_pdp = $info->{"rra[$index].pdp_per_row"};

			if ( $current_pdp != $pdp ) {
				warning(
"change_rra_pdp $period on $ficrrd period from $current_pdp to $pdp"
				);

				# dump to xml
				my $fh_in = piped_rrdtool( 'dump', $ficrrd );
				if ( !$fh_in ) {
					next RRD;
				}
				my $fh_out;
				my $ficxml = $ficrrd . '.xml';
				## no critic (RequireBriefOpen)
				if ( open $fh_out, '>', $ficxml ) {

					# analyse and rewrite to ficxml
					while (<$fh_in>) {
						## no critic (ProhibitDeepNests)
						if (m/<pdp_per_row> $current_pdp/) {

							# the pdp is very specific of an rra
							# it is not necessary to test for index
							print {$fh_out}
							  "<pdp_per_row> $pdp </pdp_per_row>\n";
						}
						else {

							# write all other unchanged
							print {$fh_out} $_;
						}
					}
					my_close( $fh_in,  "dump of $ficrrd" );
					my_close( $fh_out, $ficxml );

					# restore xml to rrd and clean
					rrdtool_restore( $ficxml, $ficrrd );
				}
				else {
					warning("can not open xml file : $ERRNO");
					my_close( $fh_in, "dump of $ficrrd" );
					next RRD;
				}
			}
			else {
				debug(
"change_rra_pdp $period on $ficrrd period $current_pdp already ok"
				);
			}
		}
		touch($stamp);
	}
	else {
		debug("change_rra_pdp $period already stamped");
	}

	debug_sub_end();
	return;
}
######################################################
# a function to have to change the consolidation function (cf)
# for all rra
# cf : average, min, max, last
sub mod_cf($$@) {
	debug_sub_begin();
	my $old_cf    = shift @_;    # old cf name
	my $new_cf    = shift @_;    # new cf name
	my @liste_rrd = @_;

	# stamp
	my $stamp = 'cf.' . $new_cf . $stamp_ext;

	if ( !-e $stamp ) {
	  RRD: foreach my $ficrrd (@liste_rrd) {
			my $info = RRDs::info($ficrrd);

			# test on all databases for changes
			my $change = 0;
			my $max    = scalar get_rra_param();
		  RRA: for my $i ( 0 .. $max ) {

				my $cf = $info->{ 'rra[' . $i . '].cf' };
				if ( $cf ne $new_cf ) {

					# found a bad cf
					$change++;
					warning(
						"mod_cf file $ficrrd base $i old rra $cf to $new_cf");
					last RRA;
				}
			}

			if ($change) {

				# dump to xml in a pipe
				my $fh_in = piped_rrdtool( 'dump', $ficrrd );
				if ( !$fh_in ) {
					next RRD;
				}
				my $fh_out;
				my $ficxml = $ficrrd . '.xml';
				## no critic (RequireBriefOpen)
				if ( open $fh_out, '>', $ficxml ) {

					# rewrite to change cf
					while (<$fh_in>) {
						s/$old_cf/$new_cf/;
						print {$fh_out} $_;
					}

					my_close( $fh_in,  "dump of $ficrrd" );
					my_close( $fh_out, $ficxml );

					# restore xml to rrd and clean
					rrdtool_restore( $ficxml, $ficrrd );
				}
				else {
					warning("can not open xml file : $ERRNO");
					my_close( $fh_in, "dump of $ficrrd" );
					next RRD;
				}

			}
			else {
				debug("mod_cf file $ficrrd already ok");
			}
		}
		touch($stamp);
	}
	else {
		debug("mod_cf already stamped\n");
	}
	debug_sub_end();
	return;
}
######################################################
# apply mod_cf on a module
sub module_mod_cf($$$) {
	debug_sub_begin();
	my $module = shift @_;    # module name (ex : cpu )
	my $old_cf = shift @_;    # old cf
	my $new_cf = shift @_;    # new cf

	my $save_dir = goto_dir($module);
	if ( !$save_dir ) {
		return;
	}

	my @liste_rrd = get_rrd_liste();
	mod_cf( $old_cf, $new_cf, @liste_rrd );

	chdir $save_dir;
	debug_sub_end();
	return;
}
######################################################
# change dst (derive, counter, gauge)
# is called by counter_to_derive, gauge_to_derive, derive_to_gauge
sub change_dst($$@) {
	my $new_dst = shift @_;
	my $fic     = shift @_;    # rrd name
	my @param   = @_;          # list of parameters

	my $info = RRDs::info($fic);

	my $changes = 0;
	for my $param (@_) {
		my $type = $info->{ 'ds[' . $param . '].type' };
		if ( $type ne $new_dst ) {
			$changes++;
			info(
				"change_dst fic $fic param $param type $type to $new_dst fixed"
			);

			# set type to new dst
			RRDs::tune( $fic, '-d', "$param:$new_dst" );

			# change minimum to zero to avoid negative value
			RRDs::tune( $fic, '-i', $param . ':0' );
		}
		else {
			debug(
				"change_dst fic $fic param $param (type $new_dst) already ok");
		}
	}    # param loop
	return $changes;
}

######################################################
# used to change from gauge to derive
# arg : module  param1  param2 ...
sub counter_to_derive($@) {
	my $module = shift @_;    # module name, ex: cpu
	my @param  = @_;          # list of parameters

	my $save_dir = goto_dir($module);
	if ( !$save_dir ) {
		return;
	}

	my $stamp = 'c2derive' . $stamp_ext;
	if ( !-e $stamp ) {
		my @liste_rrd = get_rrd_liste();

		foreach my $fic (@liste_rrd) {
			change_dst( 'DERIVE', $fic, @param );
		}
		touch($stamp);
	}
	else {
		debug("counter_to_derive $module already stamped");
	}
	chdir $save_dir;
	return;
}

######################################################
# used to change from gauge to derive
# arg : module database param1 param2 param3 ...
sub gauge_to_derive($$@) {
	my $module = shift @_;    # module name, ex : cpu
	my $ficrrd = shift @_;    # rrd database file name
	my @param  = @_;          # list of database parameters

	my $save_dir = goto_dir($module);
	if ( !$save_dir ) {
		return;
	}

	my $stamp = 'derive.' . $ficrrd . $stamp_ext;
	if ( !-e $stamp ) {
		my $changes = change_dst( 'DERIVE', $ficrrd, @param );
		if ($changes) {

			my $fh_in = piped_rrdtool( 'dump', $ficrrd );
			if ( !$fh_in ) {
				return;
			}
			my $fh_out;
			my $ficxml = $ficrrd . '.xml';
			## no critic (RequireBriefOpen)
			if ( open $fh_out, '>', $ficxml ) {

				# change last value to a big one
				while (<$fh_in>) {
					if (m/<last_ds>/) {
						print {$fh_out} "<last_ds> 999999999\n";
					}
					else {
						print {$fh_out} $_;
					}
				}

				my_close( $fh_in,  "dump of $ficrrd" );
				my_close( $fh_out, $ficxml );

				# restore xml to rrd and clean
				rrdtool_restore( $ficxml, $ficrrd );
			}
			else {
				warning("can not open xml file : $ERRNO");
				my_close( $fh_in, "dump of $ficrrd" );
				return;
			}

		}
		touch($stamp);
	}
	else {
		debug("gauge_to_derive $module already stamped");
	}
	chdir $save_dir;
	return;
}
######################################################
# used to change from derive to gauge
# arg : module database param1 param2 param3 ...
sub derive_to_gauge($$@) {
	my $module = shift @_;    # module name, ex : cpu
	my $ficrrd = shift @_;    # rrd database file name
	my @param  = @_;          # list of database parameters

	my $save_dir = goto_dir($module);
	if ( !$save_dir ) {
		return;
	}

	my $stamp = 'gauge.' . $ficrrd . $stamp_ext;
	if ( !-e $stamp ) {
		my $change = change_dst( 'GAUGE', $ficrrd, @param );
		touch($stamp);
	}
	else {
		debug("derive_to_gauge $module already stamped");
	}

	chdir $save_dir;
	return;
}
######################################################
# remove old bash script if exists
# (now all is in perl)
sub clean_sh() {
	my $old_script = 'sysstats.sh';
	if ( -e $old_script ) {
		unlink $old_script;
	}
	return;
}
######################################################
# remove all stamps
#sub clean_stamp {
#	unlink glob '*' . $stamp_ext;
#	return;
#}
# extract the minimum and maximum value acceptable from an rrd file
sub get_minmax($) {
	debug_sub_begin();
	my $ficrrd = shift @_;    # rrd file

	if ( !defined $ficrrd ) {
		return ( 'U', 'U' );
	}

	my $hash = RRDs::info($ficrrd);
	my $err  = RRDs::error();
	warning("can not get info on database $ficrrd : $err") if $err;

	my $min;
	my $max;

	foreach my $key ( keys %{$hash} ) {

		#search first minimal_heartbeat (suppose all are same value)
		if ( $key =~ m/[.]min$/ ) {
			if ( defined $hash->{$key} ) {
				$min = $hash->{$key};
			}
			else {
				$min = 'U';
			}
		}
		elsif ( $key =~ m/[.]max$/ ) {
			if ( defined $hash->{$key} ) {
				$max = $hash->{$key};
			}
			else {
				$max = 'U';
			}
		}
	}

	debug_sub_end();
	return ( $min, $max );
}
######################################################
# extract from an rrd, the names of parameters
sub get_dsnames($) {
	my $ficrrd = shift @_;    # rrd file name
	debug_sub_begin();

	my $hash = RRDs::info($ficrrd);
	my $err  = RRDs::error();
	warning("can not get info on database $ficrrd : $err") if $err;

	my @list_ds;
	foreach my $key ( keys %{$hash} ) {

		# we choose here "type", but another property
		# (as last_ds, minimal_heartbeat, value ... )
		# will be ok too
		if ( $key =~ m/^ds\[(\w+)\].type/ ) {
			push @list_ds, $1;
		}
	}
	debug_sub_end();
	return @list_ds;
}
######################################################
# set the minimum and maximum value acceptable on rrd files
sub mod_minmax($$$) {
	debug_sub_begin();
	my $min         = shift @_;
	my $max         = shift @_;
	my $ra_list_rrd = shift @_;

	foreach my $ficrrd ( @{$ra_list_rrd} ) {

		my ( $old_min, $old_max ) = get_minmax($ficrrd);
		if ( ( $old_min ne $min ) or ( $old_max ne $max ) ) {
			debug(
"   mod_minmax on $ficrrd old_min=$old_min min=$min old_max=$old_max max=$max"
			);

			my @ds_list = get_dsnames($ficrrd);

			# first arg for rrdtune is rrd file
			my @param = ($ficrrd);

			# build list for tune
			foreach my $ds (@ds_list) {
				push @param, '-i', "$ds:$min", '-a', "$ds:$max";
			}

			debug(" tune @param");
			RRDs::tune(@param);
			my $err = RRDs::error();
			warning("can not change minmax on $ficrrd : $err") if $err;
		}
		else {
			debug("mod_minmax : no change on $ficrrd");
		}
	}
	debug_sub_end();
	return;
}
######################################################
# set the minimum and maximum value acceptable on a module
sub module_mod_minmax($$$) {
	debug_sub_begin();
	my $real_dir = shift @_;    # module name or directory name
	my $min      = shift @_;
	my $max      = shift @_;

	my $save_dir = goto_dir($real_dir);
	if ( !$save_dir ) {
		return;
	}

	my @liste_rrd = get_rrd_liste();
	mod_minmax( $min, $max, \@liste_rrd );

	chdir $save_dir;
	debug_sub_end();
	return;
}
######################################################
# get the heartbeat from an rrd file
sub get_heartbeat($) {
	debug_sub_begin();
	my $ficrrd = shift @_;

	my $hash = RRDs::info($ficrrd);
	my $err  = RRDs::error();
	warning("can not get info on database $ficrrd : $err") if $err;

	foreach my $key ( keys %{$hash} ) {

		#search first minimal_heartbeat (suppose all are same value)
		next if ( $key !~ m/minimal_heartbeat/ );
		return $hash->{$key};
	}
	debug_sub_end();
	return;
}

######################################################
# a generic heartbeat change
sub change_heartbeat($$) {
	debug_sub_begin();
	my $minfo         = shift @_;    # module info
	my $new_heartbeat = shift @_;    # new value

	my @rrd_list = get_rrd_names($minfo);

	foreach my $database (@rrd_list) {
		my @ds_list = get_dsnames($database);

		# first arg for rrdtune is rrd file
		my @param = ($database);

		# build list for tune
		foreach my $ds (@ds_list) {
			push @param, '-h', "$ds:$new_heartbeat";
		}
		debug(" tune @param\n");
		RRDs::tune(@param);
		my $err = RRDs::error();
		warning("can not change heartbeat in $database : $err") if $err;

	}
	debug_sub_end();
	return;
}
######################################################
sub change_ds($$$) {
	debug_sub_begin();
	my $minfo  = shift @_;    # module info
	my $old_ds = shift @_;    # old ds name
	my $new_ds = shift @_;    # new ds name

	my @rrd_list = get_rrd_names($minfo);

  RRD: foreach my $database (@rrd_list) {

		# check for old ds name
		my @ds_list = get_dsnames($database);

		# first arg for rrdtune is rrd file
		my @param = ($database);
	  DS: foreach my $ds (@ds_list) {
			if ( $ds eq $old_ds ) {

				push @param, '-r', "$old_ds:$new_ds";
				debug(" tune @param\n");
				RRDs::tune(@param);
				my $err = RRDs::error();
				warning("can not change ds in $database : $err") if $err;
				last DS;
			}
		}
	}
	debug_sub_end();
	return;
}
######################################################
# to be used by gprint_title and gprint_param (see below)
# will display params in columns
sub title_format($) {
	my $name = shift @_;

	my $size = SPACE() x 30;
	return substr $size, 0, ( 0 - length $name );
}
######################################################
# title for data columns
sub gprint_title($$$) {
	my $module     = shift @_;
	my $name       = shift @_;
	my $rrd_detals = shift @_;

	push @{$rrd_detals}, '-v', $module;

	$name .= title_format($name);
	push @{$rrd_detals},
	  "COMMENT:$name      Cur      Min     Max     $text{'graph_average'}\\n";
	return;
}
######################################################
sub gprint_param($$$$;$) {
	my $rrd_detals = shift @_;
	my $param      = shift @_;
	my $name       = shift @_
	  || $param;    # if not defined, name is same as internal param
	my $config = shift @_;
	my $opt = shift @_ || EMPTY();    # for area, stack

	my ( $colour, $line ) = get_line_config($config);
	my $name_fmt = $name . title_format($name);
	if ($opt) {

		# area stack
		push @{$rrd_detals}, "$opt:$param#" . $colour . ":$name_fmt";
	}
	else {

		# line
		push @{$rrd_detals}, $line . ":$param#" . $colour . ":$name_fmt";
	}
	if ( exists $ENV{'WEBMINSTAT_NO_LEGEND'} ) {
		push @{$rrd_detals}, '--no-legend';
	}
	else {
		gprint( $param, $rrd_detals );
	}

	return;
}
######################################################
# add a line of statistics about a parameter :
# current, min, max, average value
sub gprint($$) {
	my $name       = shift @_;
	my $rrd_detals = shift @_;

	my $format = '%6.2lf%s'
	  ; # cf rrdgraph_graph %s is used to avoid big numbers and column are not right

	push @{$rrd_detals}, "GPRINT:$name:LAST:$format";          # current
	push @{$rrd_detals}, "GPRINT:$name:MIN:$format";           # min
	push @{$rrd_detals}, "GPRINT:$name:MAX:$format";           # max
	push @{$rrd_detals}, "GPRINT:$name:AVERAGE:$format\\n";    # average

	return;
}
######################################################
# build a def in rrdtool language
sub add_def($$$$$) {
	my $rrd_detals = shift @_;
	my $vol_def    = shift @_;    # name to be build
	my $rrd_file   = shift @_;    # path to rrd file without extension
	my $id         = shift @_;    # ident in rrd file
	my $cf         = shift @_;    # consolidation function

	# just in case, remove extension
	$rrd_file =~ s/[.]rrd$//;
	push @{$rrd_detals}, "DEF:$vol_def=$rrd_file.rrd:$id:$cf";

	return;
}
######################################################
# will empty data in all rrd database in a module
sub empty_module($) {
	debug_sub_begin();
	my $module = shift @_;

	my $save_dir = goto_dir($module);
	if ( !$save_dir ) {
		return;
	}
	my @liste_rrd = get_rrd_liste();

  RRD: foreach my $ficrrd (@liste_rrd) {
		my $fh_in = piped_rrdtool( 'dump', $ficrrd );
		if ( !$fh_in ) {
			next RRD;
		}
		my $fh_out;
		my $ficxml = $ficrrd . '.xml';
		## no critic (RequireBriefOpen)
		if ( open $fh_out, '>', $ficxml ) {

		  LINE: while (<$fh_in>) {
				if (m/<lastupdate> (\d+)/) {

					# lastupdate
					# go back one day before
					my $lastupdate = $1 - 24 * 3600;
					$_ = "<lastupdate> $lastupdate </lastupdate>\n";
				}
				elsif (m/<last_ds> \d+/) {

					# clear last_ds
					$_ = "<last_ds> UNKN </last_ds>\n";
				}
				elsif (m/<row><v>/) {

					# clear data
					s/\d[.]\d*e[+-]\d*/NaN/g;
				}
				print {$fh_out} $_;
			}
			my_close( $fh_in,  "dump of $ficrrd" );
			my_close( $fh_out, $ficxml );

			rrdtool_restore( $ficxml, $ficrrd );
		}
		else {
			warning("can not open xml file : $ERRNO");
			my_close( $fh_in, "dump of $ficrrd" );
			next RRD;
		}
	}
	chdir $save_dir;
	debug_sub_end();
	return;
}
######################################################
# common parameters to create a rrd database
sub get_create_common() {
	return ' --step 60 ';
}
######################################################
# create an rrd database
# return true if ok
sub create_rrd($$$) {
	my $base = shift @_;    # rrd name
	my $ds   = shift @_;    # ex : 'DS:total:GAUGE:300:0:U'
	my $cf   = shift @_;    # ex AVERAGE

	my $iret;
	if ( !-r $base ) {
		my $cmd = $base;
		$cmd .= get_create_common();
		$cmd .= SPACE() . $ds;
		$cmd .= add_rra($cf);
		debug("rrdtool create $cmd");

		## no critic (ProhibitEmptyQuotes)
		my @cmd = split ' ', $cmd;
		RRDs::create(@cmd);
		my $error = RRDs::error();
		if ($error) {
			warning("ERROR while creating rrd $base : $error");
			$iret = 0;
		}
		else {
			debug("database $base created");
			$iret = 1;
		}
	}
	else {
		debug("database $base already exists");
		$iret = 0;
	}
	return $iret;
}
######################################################

# return a list of parameters from an rrd file
# RRDs::info return a hash, so we loose order
sub get_rrd_params($) {
	my $rrd = shift @_;    # rrd file

	my @params;
	my $fh = piped_rrdtool( 'info', $rrd );
	if ($fh) {
		while (<$fh>) {
			if (m/^ds\[(\w+)\][.]type/) {
				my $name = $1;
				push @params, $name;
			}
		}
		my_close( $fh, "info $rrd" );
	}
	return @params;
}
###############################################################################
# change xml format from "xport" to "dump"
sub xport_filter($$) {
	my $fh_xport   = shift @_;
	my $new_period = shift @_;

	my @new_data;
	while (<$fh_xport>) {

		# line of data looks like :
		# <row><t>1182297600</t><v>NaN</v><v>NaN</v><v>NaN</v><v>NaN</v></row>
		if (m{^\s+<row><t>(\d+)</t>(.*)</row>}) {
			my $date = $1;    # 1182297600
			my $data = $2;    # <v>NaN</v><v>NaN</v><v>NaN</v><v>NaN</v>

			# date filter with a modulo
			if ( !( $date % $new_period ) ) {

# xml format (fom rrdtool dump, and for restore) is :
# <!-- 2007-02-10 09:25:28 CET / 1171095928 --> <row><v> 1.0000000000e+00 </v></row>
				my $line = '<!-- '
				  . scalar localtime($date)
				  . " / $date --> <row>$data</row>\n";
				push @new_data, $line;
			}    # date filter
		}    # data test
	}    # while
	return @new_data;
}
###############################################################################
# add a yearly rra for a rrd database
sub add_rra_year($$$) {
	my $rrd   = shift @_;    # rrd file
	my $rra_y = shift @_;    # index
	my $info  = shift @_;    # hash result of rrdtool info

	# get parameters for year rra
	my @rra_params = get_rra_param();
	my $rows       = $rra_params[$rra_y]->{'nr'};
	my $new_pdp    = $rra_params[$rra_y]->{'period'};    # minutes
	my $new_period = $new_pdp * 60;                      # en sec
	my $old_pdp =
	  $info->{'rra[3].pdp_per_row'};    # period for month rra in minutes

	my $facteur_pdp = int( $new_pdp / $old_pdp );

	# get consolidation from other rra
	my $cf = $info->{'rra[0].cf'};      # AVERAGE MAX ....

	# use xport to get data to insert into year rra
	###############################################
	# get list of parameters
	my @params = get_rrd_params($rrd);
	if ( !@params ) {
		warning("can not get rrdtool info : $ERRNO");
		return;
	}

	# get start and time
	my $end   = RRDs::last($rrd);                   # should be now, but ...
	my $start = 'e-' . $rows * $new_period . 's';
	my $max_rows =
	  $rows * $new_period / ( 60 * $old_pdp );    # to be done or only 400 rows

   # build xport command line to extract and smooth data (cf  man rrdgraph_rpn )
	my $xport_cmd = " -e $end -s $start --maxrows $max_rows ";
	foreach my $name (@params) {
		my $nameorig = $name . '_orig';
		$xport_cmd .=
"DEF:$nameorig=$rrd:$name:$cf CDEF:$name=$nameorig,$facteur_pdp,TREND XPORT:$name:\'$name\' ";
	}

#print "start=", scalar localtime($start), " ($start); end=", scalar localtime($end), " ($end)\n";
#print "xport_cmd=$xport_cmd\n";
# rrdtool xport -e 1190299613 -s 1190299494 DEF:x=base.rrd:toto:AVERAGE CDEF:smoothed=x,2,TREND XPORT:smoothed:'toto2'

	# execute xport and populate new_data
	my @new_data;
	my $fh_xport = piped_rrdtool( 'xport', $xport_cmd );
	if ($fh_xport) {

		@new_data = xport_filter( $fh_xport, $new_period );
		my_close( $fh_xport, "rrdtool xport $xport_cmd" );
	}
	else {
		warning("can not start xport: $ERRNO");
		return;
	}

	# build xml file
	################
	my $fh_in = piped_rrdtool( 'dump', $rrd );
	if ( !$fh_in ) {
		return;
	}
	my $fh_out;
	my $ficxml = $rrd . '.xml';
	## no critic (RequireBriefOpen)
	if ( open $fh_out, '>', $ficxml ) {
		my @rra;    # store
		my $nb_rra = 0;    # count rra

		while (<$fh_in>) {
			if (m/<rra>/) {
				$nb_rra++;    # first rra is 1 ...
			}

			if ( $nb_rra == $rra_y ) {

				# be carefull rra_y begins at 0,
				# so we are really at last rra (month)

				# write all month data
				print {$fh_out} $_;

				# test and store to create new year rra
				## no critic (ProhibitCascadingIfElse)
				if (m/<pdp_per_row> \d+/) {
					push @rra, "<pdp_per_row> $new_pdp </pdp_per_row>";
				}
				elsif (m/<row><v>/) {
					next;
				}
				elsif (m{</database>}) {
					next;
				}
				elsif (m{</rra>}) {

					# end of rra, now copy the stored year rra
					print {$fh_out} @rra;
					print {$fh_out} @new_data;
					print {$fh_out} "</database>\n";
					print {$fh_out} $_;                # </rra>

					$nb_rra++;
				}
				else {
					push @rra, $_;
				}
				## use critic
			}
			else {

				# just copy
				print {$fh_out} $_;
			}
		}
		my_close( $fh_in,  "dump of $rrd" );
		my_close( $fh_out, $ficxml );

		# restore xml to rrd and clean
		rrdtool_restore( $ficxml, $rrd );

		info("add yearly rra to $rrd");
	}
	else {
		warning("can not write xml $ficxml : $ERRNO");
		my_close( $fh_in, "dump of $rrd" );
	}
	return;
}
## use critic
###############################################################################
# add a yearly rra for all rrd databases
sub mod_add_year(@) {
	my @liste_rrd = @_;
	debug_sub_begin();

	my $rra_y = get_rra_index('year');
	foreach my $rrd (@liste_rrd) {

		# test on rra number
		my $info = RRDs::info($rrd);
		if ( exists $info->{ 'rra[' . $rra_y . '].rows' } ) {
			debug("yearly rra already exists in $rrd");
			return;
		}
		add_rra_year( $rrd, $rra_y, $info );
	}
	debug_sub_end();
	return;
}
###############################################################################
# check configuration files for problems : warn and try to fix
sub check_config($;$) {
	my $dir =
	  shift @_;    # webminstats configuration directory (/etc/webmin/sysstats)
	my $filter =
	  shift @_;    # an optional filter, to allow test only only one module

	debug_sub_begin();
	my $save_dir = getcwd();

	my $module_dir = $dir . '/modules';
	if ( !chdir $module_dir ) {
		warning("can not chdir to $module_dir : $ERRNO");
		return;
	}

	# need to read to access rrdtools
	read_file( '../config', \%config );

	# read modules dir
	## no critic (ProhibitNoisyQuotes)
	my @modules = readdir_content('.');
	## use critic

	# rra parameters
	my @rra_params = get_rra_param();

	# treat each module
  MODULE: foreach my $module (@modules) {
		next MODULE if ( $module =~ /^[.]{1,2}$/ );    # skip . and ..

		if ($filter) {
			next MODULE unless ( $module =~ m/$filter/ );
		}

		if ( !-d $module ) {
			warning("bad $module (not a directory) in $module_dir");
		}
		else {
			chdir $module;
			debug("check_config module $module");

			my @liste_rrd = get_rrd_liste();
			my %modconfig;
			read_module_config( \%modconfig );

			grow_rra_nr( 'month', $rra_params[ get_rra_index('month') ]->{'nr'},
				@liste_rrd );
			grow_rra_nr( 'week', $rra_params[ get_rra_index('week') ]->{'nr'},
				@liste_rrd );

			#change_rra_pdp('month', 60, @liste_rrd);

			#mod_cf(@liste_rrd);

			mod_add_year(@liste_rrd);

			my $change = 0;

			# controls : check if config file are valid
			#$change += clean_config( \%modconfig );
			$change += check_name( $module, \%modconfig );
			$change += mod_change_display( \%modconfig );
			$change += check_unused( \%modconfig, @liste_rrd );
			$change += check_duplicate_rrd( \%modconfig );
			$change += check_duplicate_name( \%modconfig );
			$change += check_db_uid( \%modconfig, @liste_rrd );
			$change += clean_display( \%modconfig );

			if ($change) {

				# rewrite module config only if necessary
				write_file_hash( 'config', \%modconfig );
			}

			chparent();    # return to /etc/webmin/sysstats/modules
		}
	}    # foreach

	chparent();    # return to /etc/webmin/sysstats

	# cleaning of old lib
	my $oldlib = 'gen-lib.pl';
	unlink $oldlib if ( ( -f $oldlib ) or ( -l $oldlib ) );

	# clean old bash script
	clean_sh();

	# specific module treatment are to be called in update sub

	# return to first dir
	chdir $save_dir;
	debug_sub_end();
	return;
}
###############################################################################
# return last line of database in the format
# epoch data1 data2 ...
sub get_lastupdate($) {

	#	my $database = shift @_;
	#
	#	my $lastupdate = RRDs::last($database);
	#	debug("lastupdate = $lastupdate");
	#	my ( $start, $step, $names, $data ) =
	#	  RRDs::fetch( $database, 'AVERAGE', '-s', $lastupdate - 30 );
	#
	#	#print "Start:       ", scalar localtime($start), " ($start)\n";
	#	#print "Step size:   $step seconds\n";
	#	#print "DS names:    ", join (", ", @$names)."\n";
	#	#print "Data points: ", $#$data + 1, "\n";
	#	#print "Data:\n";
	#
	#	my @values;    # result
	#	               # may recover several data : keep the last (most recent)
	#  LINE: foreach my $line ( @{$data} ) {
	#		my $ok  = 0;
	#		my @tab = ();
	#		push @tab, $start;    # epoch
	#		foreach my $val ( @{$line} ) {
	#			if ( defined $val ) {
	#
	#				# ok
	#				push @tab, $val;
	#				$ok = 1;
	#			}
	#			else {
	#				# add dummy value
	#				push @tab, qw{};
	#			}
	#		}
	#		debug("tab = @tab");
	#		if ($ok) {
	#
	#			# keep it
	#			@values = @tab;
	#		}
	#		$start += $step;
	#	}
	#
	#	# to return something defined
	#	if ( !@values ) {
	#		push @values, ( $start - $step, q{} );
	#	}
	#
	#	debug("values = @values");
	#	return @values;
	#}

   # from https://lists.oetiker.ch/pipermail/rrd-users/2011-February/017573.html
   # Accepts: file.rrd, [--daemon hostname] (same args as rrdtool lastupdate)
   # # Returns: timestamp, data, [data, ...]  (same return values, too)
	my ( $file, @args ) = @_;

	my $ts = RRDs::last(@_);
	if ( RRDs::error() ) {
		warning( 'lastupdate() last error: ' . RRDs::error() );
		return;
	}

	# get step
	my $rh_info = RRDs::info($file);
	if ( RRDs::error() ) {
		warning( 'lastupdate() info error: ' . RRDs::error() );
		return;
	}
	my $step1 = $rh_info->{'step'};

   # Assume step is 300. If yours is different, fetch it with RRDs::info().
   # The start value is earlier than last's timestamp because rrdtool stores
   # deltas. I'm curious if we can set both -s and -e to $ts - 300 or if there's
   # a possibility that we might be off by one. This is the safer solution.
   #
	my ( $start, $step, $names, $data ) = RRDs::fetch(
		$file,
		'AVERAGE',
		'-s' => $ts - $step1,
		'-e' => $ts,
		@args
	);
	if ( RRDs::error() ) {
		warning( 'lastupdate() fetch error: ' . RRDs::error() );
		return;
	}

	# Find the last row with defined datapoints.
	for my $i ( reverse 0 .. $#{$data} ) {
		## no critic (ProhibitBooleanGrep)
		if ( grep { defined } @{ $data->[$i] } ) {
			return ( $start + $step * $i, @{ $data->[$i] } );
		}
	}

	warning("No data found in $file");
	return;
}
############################################################
# to be called after each RRDs::update
sub rrd_control_update($) {
	my $rrdbase = shift @_;

	# control RRDs::update call
	my $err = RRDs::error();
	if ($err) {
		warning("ERROR while updating $rrdbase database : $err");
		return;
	}

	# time control
	my $last_data = RRDs::last($rrdbase);
	$err = RRDs::error();
	if ($err) {
		warning("ERROR can not get last data time of $rrdbase: $err");
		return;
	}
	else {
		my $cur_time = time;
		my $diff     = abs $cur_time - $last_data;
		if ( $diff > 30 ) {
			warning(
"ERROR while updating $rrdbase database : last database data is "
				  . localtime $cur_time );
			return;
		}
	}
	return;
}
###############################################################################
1;
