
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a lib dedicated to module install/upgrade
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);
use File::Basename;
use File::Copy;

my $SLASH = SLASH();
my $DOT   = DOT();
my $DASH  = DASH();

## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory %in $module_name $root_directory);

use vars qw( $module_dir $cron_dir );
## use critic

my $general_script = 'sysstats.pl';

# all files necessary for aquisition
my @Aqui_libs = (
	$general_script,              'sysstats-gen-lib.pl',
	'sysstats-aquisition-lib.pl', 'sysstats-constants-lib.pl',
	'sysstats-alarm-lib.pl',      'sysstats-action-lib.pl',
	'sysstats-rrd-lib.pl',
);

my $flag_symlinks = 1;    # default type of install

###############################################################################
# return aquisition script path
sub aquisition_script() {
	return $module_config_directory . $SLASH . $general_script;
}
###############################################################################
# 		section : module installation and upgrade
###############################################################################
# to be able to use "old" copy install instead symlinks
# for example from batch_install
sub is_symlink_install() {
	if ( exists $ENV{'INSTALL_COPY'} ) {
		return 0;
	}
	else {
		return $flag_symlinks;
	}
}
###############################################################################
# low-level copy sub
# now use symbolic links
sub copy_scripts($$) {
	debug_sub_begin();
	my $src_fic = shift @_;    # source file to copy
	my $dst_fic = shift @_;    # destination file

	# todo try avoid unlink
	my_unlink($dst_fic);

	if ( is_symlink_install() ) {

		# symbolic link
		chmod oct 755, $src_fic;
		symlink $src_fic, $dst_fic;
	}
	else {

		# old copy system
		copy( $src_fic, $dst_fic );
	}

	debug_sub_end();
	return;
}

###############################################################################
# copy aquisition script and libs (gen-lib)
sub create_script_sysstats() {

	debug_sub_begin();

	my $sysstat_dir = sysstat_dir();

	foreach my $lib (@Aqui_libs) {
		copy_scripts( $sysstat_dir . $lib,
			$module_config_directory . $SLASH . $lib );
	}

	debug_sub_end();
	return;
}
###############################################################################
sub make_db_id($) {
	my $r_config = shift @_;

	my %id;
	foreach my $key ( keys %{$r_config} ) {
		my $rrd = is_vol( $key, $r_config->{$key} );
		if ($rrd) {
			my $id = get_vol_name( $r_config->{$key} );
			$id{$id} = $rrd;
		}
	}
	return %id;
}
###############################################################################
sub mod_upgrade_copy($$) {
	my $old_info = shift @_;
	my $minfo    = shift @_;

	# cleaning of .old .new files to avoid keep old files
	my @old_list = get_file_names( $minfo->{'dbtables'}, '*.old' );
	foreach my $fic (@old_list) {
		my_unlink($fic);
	}
	my @new_list = get_file_names( $minfo->{'dbtables'}, '*.new' );
	foreach my $fic (@new_list) {
		my_unlink($fic);
	}

	# copy old config and database to current directory with old extension
	my_copy( $old_info->{'config_file'}, $minfo->{'config_file'} . '.old' );
	my @rrd_list = get_rrd_names($old_info);
	foreach my $fic (@rrd_list) {
		my $base = basename($fic);
		my_copy( $fic, $minfo->{'config_dir'} . $base . '.old' );
	}

	# copy stamp files (todo share stamp_ext)
	my @stamp_list = get_file_names( $old_info->{'dbtables'}, '*.stamp' );
	foreach my $fic (@stamp_list) {
		my_copy( $fic, $minfo->{'config_dir'} );
		debug("copy stamp $fic to $minfo->{'config_dir'}");
	}

	# rename all new files with new extension
	my_rename( $minfo->{'config_file'}, $minfo->{'config_file'} . '.new' );
	my @new_rrd = get_rrd_names($minfo);
	foreach my $fic (@new_rrd) {
		my_rename( $fic, $fic . '.new' );
	}

	return;
}
###############################################################################
sub mod_upgrade_mergeold($$$$$) {
	my $minfo      = shift @_;
	my $old_config = shift @_;
	my $new_config = shift @_;
	my $my_config  = shift @_;
	my $new_id     = shift @_;

	# get keep configuration
	# keep = 1 for manual config : generic/custom
	# keep = 0 for automatic config : disk
	my $keep = $new_config->{'keep'};

	my $volnum = 1;
	############################
	# loop on old config
	############################
	foreach my $old_key ( tri_config( keys %{$old_config} ) ) {
		my $old_rrd = is_vol( $old_key, $old_config->{$old_key} );
		if ($old_rrd) {

			# is a volume parameter
			my $old_id = get_vol_name( $old_config->{$old_key} );

			if ( ( exists $new_id->{$old_id} ) or $keep ) {

				# we have a common volume (parameter)

				# rename old database to rrd to keep data
				my_rename(
					$minfo->{'config_dir'} . $old_rrd . '.rrd.old',
					$minfo->{'config_dir'} . $volnum . '.rrd'
				);

				# remove new empty database
				if ( exists $new_id->{$old_id} ) {
					my $new_rrd =
					  $minfo->{'config_dir'} . $new_id->{$old_id} . '.rrd.new';
					my_unlink($new_rrd);
				}

				# dirty hack for inodes in disk module
				my $old_inode_rrd =
				  $minfo->{'config_dir'} . 'i' . $old_rrd . '.rrd.old';
				if ( -f $old_inode_rrd ) {
					my_rename( $old_inode_rrd,
						$minfo->{'config_dir'} . 'i' . $volnum . '.rrd' );

					my $new_inode_rrd =
					    $minfo->{'config_dir'} . 'i'
					  . $new_id->{$old_id}
					  . '.rrd.new';
					my_unlink($new_inode_rrd);
				}

				# build new key
				my $prefix = get_vol_prefix($old_key);
				my $key    = $prefix . $volnum;

				# build new config
				my @config = unpack_config( $old_config->{$old_key} );
				shift @config;    # remove first elem : database id
				unshift @config, $volnum;    # set database id

				info(
"keep old volume $key ($old_id), database $volnum (was $old_key, $old_rrd.rrd)"
				);
				$my_config->{$key} = pack_config(@config);
				$volnum++;
			}
			else {

				# we have a volume parameter in old config and not in new one
				info(
					"skip old parameter $old_key ($old_id), database $old_rrd");

				# so move old rrd
				my_rename(
					$minfo->{'config_dir'} . $old_rrd . '.rrd.old',
					$minfo->{'config_dir'} . $old_rrd . '.rrd.sav',
				);
			}
		}
		else {

			# not a volume

			if ( exists $new_config->{$old_key} ) {

				my %param_new = (
					keep    => 1,
					version => 1,
				);
				my %param_keep = (
					runstop => 1,
					db_uid  => 1,
				);

				## no critic (ProhibitCascadingIfElse)
				# parameter exists in the 2 config : keep it
				if ( $old_config->{$old_key} eq $new_config->{$old_key} ) {

					# no problem : same value
					$my_config->{$old_key} = $old_config->{$old_key};
					info( "keep unchanged parameter $old_key : "
						  . $my_config->{$old_key} );
				}
				elsif ( $old_key eq 'runstop' ) {

					# keep old config
					$my_config->{$old_key} = $old_config->{$old_key};
					info( "keep old parameter $old_key : "
						  . $my_config->{$old_key} );

				}
				elsif ( exists $param_new{$old_key} ) {

					# keep last
					$my_config->{$old_key} = $new_config->{$old_key};
					info( "keep new parameter $old_key : "
						  . $my_config->{$old_key} );
				}
				elsif ( exists $param_keep{$old_key} ) {
					if ($keep) {

						# config is done in old config
						$my_config->{$old_key} = $old_config->{$old_key};
						info( "keep old parameter $old_key : "
							  . $my_config->{$old_key} );
					}
					else {

						# config test is done in new
						$my_config->{$old_key} = $new_config->{$old_key};
						info( "keep new parameter $old_key : "
							  . $my_config->{$old_key} );
					}
				}
				else {

					# value changed : which one keep ?
					# old may have been changed by the user
					# new value must be set by module "upgrade" sub
					$my_config->{$old_key} = $old_config->{$old_key};
					info(   "keep old (changed) parameter $old_key : "
						  . $my_config->{$old_key}
						  . ' (new was '
						  . $new_config->{$old_key}
						  . ')' );
				}
			}
			elsif ( $old_key =~ m/^gdisplay/ ) {
				info("keep parameter $old_key");
				$my_config->{$old_key} = $old_config->{$old_key};
			}
			else {

				# is an old parameter, not in new config  :
				# keep it (custom, period, logarithme)
				# cleaning should be done by other modules
				info("keep old parameter $old_key");
				$my_config->{$old_key} = $old_config->{$old_key};
			}
		}
	}
	return $volnum;
}
###############################################################################
sub mod_upgrade_mergenew($$$$$$) {
	my $minfo      = shift @_;
	my $old_config = shift @_;
	my $new_config = shift @_;
	my $my_config  = shift @_;
	my $old_id     = shift @_;
	my $volnum     = shift @_;

	############################
	# loop on new install config
	############################
	foreach my $new_key ( tri_config( keys %{$new_config} ) ) {
		my $new_rrd = is_vol( $new_key, $new_config->{$new_key} );
		if ($new_rrd) {

			# is a volume parameter
			my $new_id = get_vol_name( $new_config->{$new_key} );

			if ( !exists $old_id->{$new_id} ) {

				# this is a new volume

				#rename database
				my_rename(
					$minfo->{'config_dir'} . $new_rrd . '.rrd.new',
					$minfo->{'config_dir'} . $volnum . '.rrd'
				);

				# dirty hack for inodes in disk module
				my $new_inode_rrd =
				  $minfo->{'config_dir'} . 'i' . $new_rrd . '.rrd.new';
				if ( -f $new_inode_rrd ) {
					my_rename( $new_inode_rrd,
						$minfo->{'config_dir'} . 'i' . $volnum . '.rrd' );
				}

				# build new key
				my $prefix = get_vol_prefix($new_key);
				my $key    = $prefix . $volnum;

				# build new config
				my @config = unpack_config( $new_config->{$new_key} );
				shift @config;    # remove first elem : database id
				unshift @config, $volnum;    # set database id

				info(
"new volume $key ($new_id), database $volnum (orig $new_key, $new_rrd.rrd)"
				);
				$my_config->{$key} = pack_config(@config);
				$volnum++;
			}
		}
		else {

			# not a volume
			if ( !exists $old_config->{$new_key} ) {

				# new parameter : keep it
				$my_config->{$new_key} = $new_config->{$new_key};
				info("keep new parameter $new_key");
			}
		}
	}
	return;
}
###############################################################################
# used to upgrade a module from a version to another
# we have to keep data
sub mod_upgrade($$$$) {
	my ( $minfo, $new_config, $defaults, $old_info ) =
	  ( $_[0], $_[1], $_[2], $_[3] );

	debug_sub_begin();

	my $old_versin  = $old_info->{'module_ver'};
	my $new_version = $minfo->{'module_ver'};

	#	info(   'upgrade of '
	#		  . $minfo->{'module_name'}
	#		  . " from $old_versin to $new_version" );

	# get old config
	my %old_config;
	read_module_config( \%old_config, $old_info->{'config_file'} );

	# call preupgrade module function if exists
	if ( defined &main::preupgrade ) {
		preupgrade( $old_info, \%old_config, $defaults );
	}

	# copy and rename
	mod_upgrade_copy( $old_info, $minfo );

	# populate parameters id
	my %old_id = make_db_id( \%old_config );
	my %new_id = make_db_id($new_config);
	my %my_config;    # config to be set

	my $volnum =
	  mod_upgrade_mergeold( $minfo, \%old_config, $new_config, \%my_config,
		\%new_id );
	mod_upgrade_mergenew( $minfo, \%old_config, $new_config, \%my_config,
		\%old_id, $volnum );

	# do something with remaining databases (static parameters)
	# 1 : rename all remaining old rrd bases
	my @old = get_file_names( $minfo->{'config_dir'}, '*.rrd.old' );
	foreach my $base (@old) {
		my $orig = $base;
		$orig =~ s/[.]old$//;
		if ( -f $orig ) {

			# problem !
			warning("do not know what to do with old $base");
		}
		else {
			my_rename( $base, $orig );
			info("keep old database $orig");
		}
	}

	# 2 : remove all new common rrd files
	# 3 : and preserve all new rrd files
	my @new = get_file_names( $minfo->{'config_dir'}, '*.rrd.new' );
	foreach my $base (@new) {
		my $orig = $base;
		$orig =~ s/[.]new$//;
		if ( -f $orig ) {
			my_unlink($base);
		}
		else {
			my_rename( $base, $orig );
			info("keep new database $base");
		}
	}

	write_config( $minfo, \%my_config );

	# we have to read again merged file
	read_module_config( \%my_config, $old_info->{'config_file'} );

	# be careful if sample_rate change
	check_heartbeat( $minfo, \%my_config );

	# call postupgrade module function
	if ( defined &main::postupgrade ) {
		info('call postupgrade');
		postupgrade( $minfo, \%my_config, $defaults );
	}

	# remove old module version
	remove_module( $old_info, 1 );

	debug_sub_end();
	return;
}
##########################################################
# install module's script
# called from all modules
sub install_script($$) {
	my ( $minfo, $script ) = ( $_[0], $_[1] );

	debug_sub_begin();

	my $module_name = $minfo->{'idx'};

	# clean older script if exists
	# see also pre_install_change_24
	my $old_script = $minfo->{'config_dir'} . 'run.pl';
	if ( ( -e $old_script ) or ( -l $old_script ) ) {
		debug("remove old script $old_script");
		my_unlink $old_script;
	}

	# directories are relative
	# and ends with a trailing / (cf get_sysstats_module_info)
	my $source_file = $minfo->{'mod_dir'} . $script;

	# if a specific os script exist, copy this one, otherthise the standard one
	my $source_file_os = $source_file . $DOT . $gconfig{'os_type'};
	my $source_copy = ( -e $source_file_os ) ? $source_file_os : $source_file;

	my $sysstat_dir = sysstat_dir();
	my $dest_file   = $minfo->{'config_dir'} . get_module_script($module_name);
	copy_scripts( $sysstat_dir . $source_copy, $dest_file );

	debug_sub_end();
	return;
}

###############################################################################
# copy help files from modules to central help
sub install_help($) {
	my $minfo = $_[0];
	debug_sub_begin();

	# use symbolic links
	my @helpfiles = glob sysstat_dir() . $minfo->{'mod_dir'} . '/help/*.html';
	my $nb        = 0;
	foreach my $help (@helpfiles) {
		my $help_base = basename($help);

		my $target = 'help/' . $help_base;

		my_unlink($target);
		if ( symlink $help, $target ) {

			#info("install $help on $target");
			$nb++;
		}
		else {
			warning("problem in install help $help on $target");
		}
	}
	info("install $nb help files for $minfo->{'module_name'}");

	debug_sub_end();
	return;
}
###############################################################################
# check if a file is in a tar.gz module archive
sub check_module_file ($$) {
	my $module_file = $_[0];
	my $file_check  = $_[1];

	debug_sub_begin();

	# check if $file_check exist
	my $cmd = "gunzip -c $module_file | tar -tf - | grep  $file_check";
	debug($cmd);
	## no critic (ProhibitBacktickOperators)
	my $out = `$cmd`;
	## use critic

	my $rep;
	if ( ( index $out, $file_check ) >= 1 ) {
		$rep = 1;
	}
	else {
		$rep = 0;
	}
	debug_sub_end();
	return $rep;
}

###############################################################################
# check if a module is already installed
# the tar.gz archive contains name with version
# we extract the relative path to mandatory file : module.info
# and check if it exists on the disk
#sub check_if_already_installed($) {
#	my $module_file = $_[0];
#
#	debug_sub_begin();
#
#	my $cmd = "gunzip -c $module_file | tar -tf - | grep module.info";
#	debug($cmd);
#	## no critic (ProhibitBacktickOperators)
#	my $out = `$cmd`;
#	## use critic
#	chomp $out;
#
#	my $code_ret;
#
#	if ( -e $module_dir . $SLASH . $out ) {
#
#		# exist
#		$code_ret = 1;
#	}
#	else {
#
#		# do not exist
#		$code_ret = 0;
#	}
#	debug_sub_end();
#	return $code_ret;
#}

###############################################################################
# check if a tar.gz archive seems complete
# then untar it
sub check_module_file_install($) {
	my $module_file = $_[0];

	my $full_module_file = $module_dir . $SLASH . $module_file;

	debug_sub_begin();

	# check if module.info exist
	if ( !check_module_file( $full_module_file, 'module.info' ) ) {
		display_install_state( $text{'index_emodule_file'} . ' module.info' );
		clean_install($module_file);
		return;
	}

	# check for library
	if ( !check_module_file( $full_module_file, 'lib.pl' ) ) {
		display_install_state( $text{'index_emodule_file'} . ' lib.pl' );
		clean_install($module_file);
		return;
	}

	# check if directory already exist
	#if ( check_if_already_installed($full_module_file) ) {
	#	print '<tt>' . $text{'index_emodule_installed'} . "</tt>\n";
	#	clean_install($module_file);
	#	return;
	#}

	# extract files from tar.gz
	if ( !chdir $module_dir ) {
		dying("can not change to $module_dir directory, $ERRNO");
	}

	my $cmd = "(gunzip -c $module_file | tar -xvf -)";
	debug("cmd=$cmd");
	## no critic (ProhibitBacktickOperators)
	my $out = `$cmd 2>&1`;
	## use critic
	debug("out = $out");

	# we should get real module directory name here
	# but the output is different on linux and on bsd
	if ( $out =~ m/^x / ) {

		# bsd : x users-0.14/
		$out =~ s/^x //;
	}
	my ( $mname, undef ) = split $SLASH, $out;

	# delete tar.gz
	my_unlink $module_file;
	chparent();

	debug_sub_end();
	return $mname;
}

############################################################################
sub clean_install($) {
	my $mname = shift @_;    # module name : cpu
	debug_sub_begin();

	# code directory
	my $code_module_dir = $module_dir . $SLASH . $mname;
	if ( -d $code_module_dir ) {
		rmtree($code_module_dir);
	}

	# data directory
	my $etc_module_dir =
	  $module_config_directory . $SLASH . $module_dir . $SLASH . $mname;
	if ( -d $etc_module_dir ) {
		rmtree($etc_module_dir);
	}

	debug_sub_end();
	return;
}
###############################################################################
# Check to see if a module was installed with its databases files (rrd)
sub check_rrd_files($) {
	debug_sub_begin();

	my $dir = shift @_;

	# read file names
	my @fic = readdir_content($dir);

	my $rep = 0;    # not found
	                # search for database
	foreach my $fic (@fic) {
		if ( $fic =~ m/rrd$/ ) {
			$rep = 1;
			last;
		}
	}

	debug_sub_end();
	return $rep;
}
###############################################################################
# update from 0.9.0 to 0.9.1
sub pre_install_change_09() {
	my $old_cron_file = 'webminstats.cron';

	if ( is_new_cron() ) {

		# change cron job name so it can stopped
		my $old_cron = $cron_dir . $old_cron_file;
		if ( -f $old_cron ) {
			my $cron_file = cron_file();
			my_rename( $old_cron, $cron_dir . $cron_file );
			debug("rename $old_cron in ${cron_dir}$cron_file");
		}
	}
	else {

		# delete old entry from crontab
		my $fic = $module_config_directory . '/systats.sh';
		delete_cron_task($fic);
		debug('stop old cron job');
	}

	return;
}
###############################################################################
# upgrade from 2.2 to 2.3 :
sub pre_install_change_23() {

	# upgrade from 2.2 to 2.3 :
	# suppress version from module names
	{

		# modules on libexec/share
		my $main_dir =
		  $root_directory . $SLASH . $module_name . $SLASH . $module_dir;
		my @main_modules = readdir_content($main_dir);
		foreach my $mod (@main_modules) {
			next if ( is_tarfile($mod) );
			if ( $mod =~ m/(\w+)-(.*)/ ) {
				my $new_mod = $1;
				rename $main_dir . $SLASH . $mod, $main_dir . $SLASH . $new_mod;
				debug("on $root_directory rename module $mod into  $new_mod");
			}
		}

		# help
		my $help_dir =
		  $root_directory . $SLASH . $module_name . $SLASH . 'help/';

		#info("search for badlinks on $help_dir");
		my @help_files = readdir_content($help_dir);
		foreach my $file (@help_files) {
			my $dest = $help_dir . $file;
			if ( ( -l $dest ) and ( !-f readlink $dest ) ) {
				info("found broken link on $dest");

				# what is the module name ?
				# file should be like config-amavis.fr.html or amavis.de.html
				if ( $file =~ m/(.*?)[.]/ ) {
					my $mod_name = $1;
					$mod_name =~ s/config-//;

					#info("maj help on $mod_name");
					my $src = $main_dir . $SLASH . $mod_name . '/help/' . $file;
					if ( -f $src ) {
						my_unlink $dest;
						symlink $src, $dest
						  or warning("can not link $src to $dest");
					}
					else {
						warning("can not find help file $src");
					}
				}
			}
		}

		# etc
		my $etc_module_dir = $module_config_directory . $SLASH . $module_dir;
		my @etc_modules    = readdir_content($etc_module_dir);
		foreach my $mod (@etc_modules) {
			if ( $mod =~ m/(\w+)-(.*)/ ) {
				my $new_mod = $1;
				rename $etc_module_dir . $SLASH . $mod,
				  $etc_module_dir . $SLASH . $new_mod;
				debug("on etc rename module $mod into $new_mod");

				# change links in etc dir
				my $src =
				  $main_dir . $SLASH . $new_mod . $SLASH . $new_mod . '.pl',
				  my $dest =
				    $etc_module_dir
				  . $SLASH
				  . $new_mod
				  . $SLASH
				  . $new_mod . '.pl';
				my_unlink $dest;
				symlink $src, $dest;
			}
			else {
				debug("on etc $mod already without version");
			}
		}
	}
	return;
}
###############################################################################
# upgrade from 2.3 to 2.4
sub pre_install_change_24() {

	# change cron name according module name
	{
		my $old_cron_file = 'webminstats';

		if ( is_new_cron() ) {

			# change cron job name so it can stopped
			my $old_cron = $cron_dir . $old_cron_file;
			if ( -f $old_cron ) {
				my $cron_file = cron_file();
				my_rename( $old_cron, $cron_dir . $cron_file );
				debug("rename $old_cron in ${cron_dir}$cron_file");
			}
		}
		else {

			# delete old entry from crontab
			my $fic = $module_config_directory . '/systats.sh';
			delete_cron_task($fic);
			debug('stop old cron job');
		}
	}

	# change name of aquisition script in /etc
	{
		my $etc_module_dir = $module_config_directory . $SLASH . $module_dir;
		my @etc_modules    = readdir_content($etc_module_dir);
		foreach my $mod (@etc_modules) {
			my $path       = $etc_module_dir . $SLASH . $mod . $SLASH;
			my $old_script = $path . 'run.pl';
			my $new_script = $path . get_module_script($mod);

			# be careful : -e does not match broken links !
			if ( ( -e $old_script ) or ( -l $old_script ) ) {
				rename $old_script, $new_script;
				debug("on etc rename script $old_script into $new_script");
			}
			else {
				debug("on etc no script $old_script found");
			}
		}
	}
	return;
}
###############################################################################
# code to call before any installation
sub pre_install_change($) {
	my $flag_cron = shift @_;

	debug_sub_begin();

	# update from 0.9.0 to 0.9.1
	pre_install_change_09();

	# upgrade from 2.2 to 2.3 :
	pre_install_change_23();

	# upgrade from 2.3 to 2.4
	pre_install_change_24();

	# some display
	print "<hr>\n";
	if ($flag_cron) {
		print html_strong('cron') . " stop<br>\n";
		stop_sysstats();
	}

	print '<table ' . w100() . '>';
	begin_row();
	title_row( $text{'index_installing_modules'} );
	end_row();

	debug_sub_end();
	return;
}
###############################################################################
sub clear_redefine() {

	# to avoid warnings on redefined subs
	# cf perl cookbook 10.14
	## no critic (ProhibitAmpersandSigils)
	undef &change_module_config;
	undef &check_for_runstop;
	undef &create_base;
	undef &graph_data;
	undef &init_param;
	undef &install_module;
	undef &list_params;
	undef &mod_config;
	undef &mod_config_save;
	undef &param2device;
	undef &preupgrade;
	undef &postupgrade;
	undef &read_static_param;
	undef &tab_vol;
	undef &test_module_config;
	undef &toggle_runstop;
	undef &toggle_display;
	undef &upgrade;
	undef &write_config;
	## use critic

	return;
}
###############################################################################
# read the version in config file
sub read_config_version($) {
	my $configfile = shift @_;

	my $version;
	if ( !-f $configfile ) {
		warning("can not find $configfile");

	}
	else {
		if ( open my $fh, '<', $configfile ) {
			while (<$fh>) {
				chomp;
				if (m/^version=(.*)/) {
					$version = $1;
					last;
				}
			}
			close $fh or warning("can not close $configfile : $ERRNO");
		}
		else {
			warning("can not open $configfile : $ERRNO");
		}
	}
	return $version;
}
###############################################################################
# build a list of modules in etc
sub get_installed_modules() {

	# build a list of modules in etc
	my $etc_module  = $module_config_directory . $SLASH . $module_dir;
	my @etc_modules = readdir_content($etc_module);

	my %etc_modules;
	foreach my $mod (@etc_modules) {

		my $mod_version = read_config_version(
			$etc_module . $SLASH . $mod . $SLASH . 'config' );

		if ( exists $etc_modules{$mod} ) {

			# there is already another version
			print html_strong('warning')
			  . " : duplicate $module_config_directory module $mod version $etc_modules{$mod} and $mod_version<br>\n";
		}
		else {
			$etc_modules{$mod} = $mod_version;
			debug("find module on etc $mod : version $mod_version");
		}
	}
	return %etc_modules;
}
###############################################################################
# return true is the file is a tar.gz
sub is_tarfile($) {
	my $m = shift @_;
	return ( ( index $m, '.tar.gz' ) != -1 );
}
###############################################################################
# check for duplicates
sub check_duplicate_modules($) {
	my $r_modules = shift @_;

	my %modules;
	foreach my $m ( @{$r_modules} ) {

		# just keep installed modules
		next if ( is_tarfile($m) );
		my %local_minfo = get_sysstats_module_info($m);

		# check duplicates
		my $mod         = $local_minfo{'module_name'};
		my $mod_version = $local_minfo{'module_ver'};
		if ( exists $modules{$mod} ) {
			print html_strong('warning')
			  . " : duplicate module $mod versions $modules{$mod} and $mod_version<br>\n";
		}
		else {
			$modules{$mod} = $mod_version;
			debug("find module $mod : version $mod_version");
		}
	}
	return %modules;
}
###############################################################################
# display install messages
my $buffer = EMPTY();

sub display_install_state($;$) {
	my $msg = shift @_;
	my $opt = shift @_;

	print '<tt> ' . $msg . '</tt>';
	$buffer .= " $msg ";
	if ($opt) {
		print html_strong(' -> ');
		$buffer .= ' -> ';
	}

	return;
}
###############################################################################
# begin to display a line about install
sub display_install_module($) {
	my $m = shift @_;

	begin_row();
	print '<td>' . html_strong( $text{'index_installing_file'} );
	display_install_state( $m, 1 );

	$buffer = $text{'index_installing_file'};

	return;
}
###############################################################################
# end the install message line
sub display_end_module() {

	print '</td>';
	end_row();

	info($buffer);
	$buffer = EMPTY();
	return;
}
###############################################################################
# check if this module is supported under that OS.
# Delete module directory if not.
sub check_module_os_support($$) {
	my $mname = shift @_;
	my $minfo = shift @_;

	debug_sub_begin();
	if ( ( index $minfo->{'os_support'}, $gconfig{'os_type'} ) == -1 ) {
		display_install_state( $text{'index_eos'} );
		clean_install($mname);
		warning( $mname . $text{'check_os'} . " ($gconfig{'os_type'})" );
		return 0;
	}
	else {
		return 1;
	}
}
###############################################################################
# get default config if exists
sub get_default($$) {
	my $mname = shift @_;
	my $minfo = shift @_;

	debug_sub_begin();
	my $defaults_file = $minfo->{'defaults_file'};
	my %defaults;

	if ( -r $defaults_file ) {

		if ( !read_file( $defaults_file, \%defaults ) ) {
			print text( 'ecannot_readdefaults', $defaults_file );
			clean_install($mname);
			warning("WARNING : can not read $defaults_file");
		}
		else {
			debug('read defaults ok');
		}
	}
	else {
		debug("$defaults_file not found");
	}
	return %defaults;
}
###############################################################################
# create internal.rrd
sub create_internal_rrd($$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;

	my $base      = $minfo->{'dbtables'} . 'internal.rrd';
	my $heartbeat = compute_heartbeat( $my_config->{'sample_rate'} );
	my $ds        = "DS:total:GAUGE:$heartbeat:0:U";
	my $cf        = $my_config->{'cf'};
	create_rrd( $base, $ds, $cf );

	return;
}
###############################################################################
# call from install_module subs
sub finalize_install($$$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $fscript   = shift @_;    # script name

	write_config( $minfo, $my_config, 1 );
	install_script( $minfo, $fscript );
	create_internal_rrd( $minfo, $my_config );

	return;
}
###############################################################################
# call module install code
sub try_install_module($$$) {
	my $mname    = shift @_;
	my $minfo    = shift @_;
	my $defaults = shift @_;

	debug_sub_begin();
	my $mod_lib = $minfo->{'mod_lib'};

	# because each module define the same function names
	debug("call require $mod_lib");
	clear_redefine();

	# load functions from lib
	unset_warnings();
	eval { require $mod_lib };
	set_warnings();
	if ($EVAL_ERROR) {
		warning("problem on require $mod_lib : $EVAL_ERROR");
		return 0;
	}
	else {
		debug("require $mod_lib ok");

		if ( !install_module( $minfo, $defaults ) ) {
			display_install_state( $text{'index_emodule_install'} );
			clean_install($mname);
			warning("WARNING : can not install module $mname");
			return 0;
		}
		else {
			debug("install module $mname ok");
			return 1;
		}
	}
}
###############################################################################
# search for an older full version of the module
#sub check_upgrade1($$$$) {
#	my $minfo        = shift @_;
#	my $defaults     = shift @_;
#	my $full_modules = shift @_;
#	my $etc_modules  = shift @_;
#
#	debug_sub_begin();
#
#	# module name without version :
#	my $new_module         = $minfo->{'module_name'};
#	my $new_module_version = $minfo->{'module_ver'};
#	my $update             = 0;
#
#	# look for an older version
#	foreach my $m2 ( keys %{$full_modules} ) {
#
#		my $other_mod         = $m2;
#		my $other_mod_version = $full_modules->{$other_mod};
#
#		if (    ( $new_module eq $other_mod )
#			and ( $new_module_version ne $other_mod_version ) )
#		{
#			my %my_config;
#			if ( !read_file( $minfo->{'config_file'}, \%my_config ) ) {
#				text_error( 'ecannot_readconfig', $minfo->{'config_file'} );
#			}
#
#			# we build a pseudo hash
#			# todo : remove version here
#			my %local_minfo;
#			$local_minfo{'config_dir'} =
#			    $module_config_directory
#			  . $SLASH
#			  . $module_dir
#			  . $SLASH
#			  . $other_mod
#			  . $DASH
#			  . $other_mod_version;
#			$local_minfo{'dbtables'}    = $local_minfo{'config_dir'};
#			$local_minfo{'module_ver'}  = $other_mod_version;
#			$local_minfo{'module_name'} = $other_mod;
#			$local_minfo{'config_file'} =
#			  $local_minfo{'config_dir'} . '/config';
#			$local_minfo{'mod_dir'} =
#			  $module_dir . $SLASH . $other_mod . $DASH . $other_mod_version;
#			debug("upgrade $other_mod from old module");
#
#			# update from local_minfo to minfo
#			mod_upgrade( $minfo, \%my_config, $defaults, \%local_minfo );
#			$update = 1;
#			delete $etc_modules->{$other_mod};
#			delete $full_modules->{$other_mod};
#			display_install_state( 'upgrade', 1 );
#		}
#		else {
#			debug("test upgrade on $other_mod $other_mod_version");
#		}
#	}
#	debug_sub_end();
#	return $update;
#}
###############################################################################
# we have to check in /etc/webmin (webminstat upgrade)
sub check_upgrade2($$$$) {
	my $minfo        = shift @_;
	my $defaults     = shift @_;
	my $full_modules = shift @_;
	my $etc_modules  = shift @_;

	debug_sub_begin();

	# module name without version :
	my $new_module         = $minfo->{'module_name'};
	my $new_module_version = $minfo->{'module_ver'};
	my $update             = 0;

	delete $ENV{'WEBMINSTATS_UPDATE'} if ( exists $ENV{'WEBMINSTATS_UPDATE'} );
	foreach my $other_mod ( keys %{$etc_modules} ) {
		my $other_mod_version = $etc_modules->{$other_mod};
		if ( $new_module eq $other_mod ) {
			if ( $new_module_version ne $other_mod_version ) {

				# find another version of same module
				info(
"upgrade $other_mod from /etc ($other_mod_version -> $new_module_version)"
				);
				$ENV{'WEBMINSTATS_UPDATE'} = 1;
				my %my_config;
				if ( !read_module_config( \%my_config, $minfo->{'config_file'} )
				  )
				{
					text_error( 'ecannot_readconfig', $minfo->{'config_file'} );
				}

				# we build a pseudo hash
				my $config_dir =
				    $module_config_directory
				  . $SLASH
				  . $module_dir
				  . $SLASH
				  . $other_mod . '.old';
				my %local_minfo = (
					'idx'         => $other_mod,
					'module_ver'  => $other_mod_version,
					'module_name' => $other_mod,
					'config_dir'  => $config_dir,
					'dbtables'    => $config_dir,
					'config_file' => $config_dir . '/config',
				);

				mod_upgrade( $minfo, \%my_config, $defaults, \%local_minfo );
				$update = 1;
				delete $etc_modules->{$other_mod};
				display_install_state( 'upgrade from etc', 1 );
			}
			else {

				# should not happen
				debug(
"(check_upgrade2) find same version in etc for $other_mod $other_mod_version"
				);
			}
		}
	}    # foreach
	debug_sub_end();
	return $update;
}
###############################################################################
# test if version are the same or not
# if equal : we do not install the module
# if not : save (move) old config directory
# nb : this directory will be removed at end of mod_upgrade
sub save_old_config($$) {
	my $mname = shift @_;
	my $minfo = shift @_;

	my $iret;    # return code

	my $install_version = $minfo->{'module_ver'};
	my $etc_version = read_config_version( $minfo->{'config_dir'} . 'config' );
	if ( $install_version eq $etc_version ) {
		debug("already installed $mname ($install_version)");

		# because webmin remove links (module update)
		install_help($minfo);

		display_install_state( $text{'index_emodule_installed'} );
		display_end_module();
		$iret = 0;
	}
	else {

		# save old module
		# we need to remove trailing /
		my $old_module = $minfo->{'config_dir'};
		$old_module =~ s{/$}{};
		$old_module .= '.old';
		if ( !move( $minfo->{'config_dir'}, $old_module ) ) {
			warning(
				"can not move $minfo->{'config_dir'} into $old_module : $ERRNO"
			);
		}
		else {
			info("move $minfo->{'config_dir'} into $old_module");
		}
		$iret = 1;
	}
	return $iret;
}
###############################################################################
# Check to see if any modules to install, called from index.cgi, batch_install.pl
# If there are any files ending with tar.gz in modules directory, install files
# todo : split this code : too big sub
sub check_new_modules() {
	debug_sub_begin();

	my $flag_cron = is_webmin_sysstats_running();

	# get all files in install directory
	my @modules = readdir_content($module_dir);

	# already installed modules
	my %full_modules;

	# modules in /etc
	my %etc_modules;

	#set_debug();
	my $nb_modules_install = 0;
  MOD: foreach my $m (@modules) {
		next MOD unless ( is_tarfile($m) );

		debug("detect $m");

		if ( $nb_modules_install == 0 ) {

			# Has basic directory structor been created yet?
			## no critic (ProhibitNoisyQuotes)
			my $full_module_dir = $module_config_directory . '/' . $module_dir;
			## use critic
			if ( !-x $full_module_dir ) {
				mkdir $full_module_dir, oct 755;
			}

			# there is some modules
			pre_install_change($flag_cron);

			# name on etc can be changed by pre_install
			%full_modules = check_duplicate_modules( \@modules );
			%etc_modules  = get_installed_modules();
		}
		$nb_modules_install++;

		display_install_module($m);

		# untar file and checks
		my $mname = check_module_file_install($m) || EMPTY();
		if ( !$mname ) {
			display_end_module();
			next MOD;
		}
		debug("real new module name : $mname");
		display_install_state( $text{'index_files_installed'}, 1 );

		my %minfo = get_sysstats_module_info($mname);
		display_install_state( build_full_name( \%minfo ), 1 );

		# check os support
		if ( !check_module_os_support( $mname, \%minfo ) ) {
			display_end_module();
			next MOD;
		}

		if ( -d $minfo{'config_dir'} ) {

			# module already exists : save old config
			next MOD unless save_old_config( $mname, \%minfo );
		}
		else {
			info("install new module $mname");
		}

		# create config directory
		if ( !mkdir $minfo{'config_dir'}, oct 755 ) {
			warning("can not create $minfo{'config_dir'} : $ERRNO");
		}
		else {
			debug("create $minfo{'config_dir'}");
		}

		# call module install code
		my %defaults = get_default( $mname, \%minfo );
		if ( try_install_module( $mname, \%minfo, \%defaults ) ) {

			# seems ok
			#print '<tt>' . $text{'word_completed'} . "</tt>\n";
		}
		else {
			display_install_state( $text{'word_aborted'} );
			display_end_module();
			next MOD;
		}

		install_help( \%minfo );

		# check for an upgrade ?
		# module name without version :
		my $new_module         = $minfo{'module_name'};
		my $new_module_version = $minfo{'module_ver'};

		# we have to check in /etc/webmin (webminstat upgrade)
		my $update =
		  check_upgrade2( \%minfo, \%defaults, \%full_modules, \%etc_modules );
		my $action = ($update) ? 'update' : 'install';

		# register module
		$etc_modules{$new_module}  = $new_module_version;
		$full_modules{$new_module} = $new_module_version;

		# log and display
		display_install_state( $text{'word_completed'} );
		my $full_module_name = build_full_name( \%minfo );
		my_webmin_log( $action, 'module', $full_module_name );

		# close row from display_install_module
		print '</td>';
		end_row();
	}    # MOD

	#unset_debug();

	# post installation
	if ($nb_modules_install) {
		post_install();
	}

	# check for bad links
	fix_broken_links();

	debug_sub_end();
	return;
}
###################################################################################################
# post installation
sub post_install() {

	print "</table><br>\n";

	# copy the script before modifs
	create_script_sysstats();

	print html_strong('database modifications') . ' ...';

	# post-install changes
	check_config($module_config_directory);

	print "ok<br>\n";

	# start webminstats
	start_sysstats();
	print html_strong('cron') . " restart<br>\n";
	return;
}
##########################################################
# return true if find a bad link
sub test_bad_link($) {
	my $file = shift @_;

	if ( -l $file ) {
		my $target = readlink $file;
		if ( -f $target ) {
			return 0;
		}
		else {
			return 1;
		}
	}
	return 0;
}
##########################################################
# in case of webmin install from tar.gz with name+version directory,
# symbolic links will be broken :
# - cron
# - script and lib for aquisition
sub fix_broken_links() {

	# test cron
	my $cronfile = $cron_dir . cron_file();
	if ( test_bad_link($cronfile) ) {
		stop_sysstats();
		start_sysstats();
		info("fix bad link for $cronfile");
	}

	foreach my $file (@Aqui_libs) {
		if ( test_bad_link( $module_config_directory . $file ) ) {
			create_script_sysstats();
			info("fix bad link for $file");
			last;
		}
	}

	return;
}
##########################################################
1;
