
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
#
# this lib is dedicated to cgi/html global code
#
###############################################################################

use strict;
use warnings;

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in %access %tconfig );

use subs qw( &init_config &get_module_acl );
use subs qw( &foreign_call &lock_file &unlock_file &read_file );

# from sysstat-lib
use vars qw( $defline );
use subs qw( &debug_sub_begin &debug_sub_end );
## use critic

my $EMPTY = EMPTY();

###############################################################################
# common html general sub
###############################################################################
sub html_comment($) {
	my $txt = shift @_;

	print "<!-- $txt -->\n";
	return;
}
###############################################################################
# recode li sub as from CGI module
sub li($) {

	return html_tag( 'li', $_[0] );
}
##########################################################
sub begin_table($;$) {
	my $summary = shift @_;
	my $option  = shift @_;

	begin_row();
	print '<td><table summary="' . $summary . '" ' . w100();
	print " $option" if ($option);
	print ">\n";
	return;
}
##########################################################
sub end_table () {
	print '</table></td>';
	end_row();
	return;
}

##########################################################
sub begin_row(;$) {
	my $option = shift @_ || $EMPTY;
	print "<tr $option>\n";
	return;
}
##########################################################
sub end_row() {
	print "</tr>\n";
	return;
}
##########################################################
sub titre_table($) {
	my $titre = shift @_;
	begin_row();
	title_row( $titre, 'align=center' );
	end_row();
	return;
}
##########################################################
sub title_row($;$) {
	my $titre  = shift @_ || $EMPTY;
	my $option = shift @_ || $EMPTY;

	print td( html_strong($titre), $option );
	return;
}
##########################################################
sub td($;$) {
	my $text   = shift @_ || $EMPTY;
	my $option = shift @_ || $EMPTY;

	return html_tag( 'td', $text, $option );
}
##########################################################
# a generic html tag maker
sub html_tag($;$$) {
	my $tag    = shift @_;
	my $text   = shift @_ || $EMPTY;
	my $option = shift @_ || $EMPTY;

	return "<$tag $option>$text</$tag>\n";
}
##########################################################
sub separator() {
	begin_row();
	print td( '<hr>', w100() );
	end_row();
	return;
}
##########################################################
# to redirect a page to another
# to be used when nothing was printed on page
sub myredirect($) {
	my $url = shift @_;    # should not begin with a /

	#if ( $url !~ m/sysstats/ ) {
	#	$url = '/sysstats/' . $url;
	#}

	wrapper( 'redirect', $url );
	return;
}
##########################################################
# to redirect from a page to another
# to be used after an "header" command
# use javascript
sub jredirect($) {
	my $url = shift @_;    # should not begin with a /

	# get current url
	#	my $uri = $ENV{'REQUEST_URI'};
	#
	#	# remove sysstats from path
	#	# because it is added by $url
	#	if ( $uri =~ m{(.*)/sysstats/} ) {
	#		$uri = $1 . $url;
	#	}
	#	else {
	#		$uri = $url;
	#	}
	print "<script LANGUAGE=JavaScript>\n";
	print 'window.location.replace("' . $url . '")';
	print "</script>\n";
	return;
}
##########################################################
# by default in %
sub width($;$) {

	my $size     = shift @_;
	my $flag_pix = shift @_;    # to set in pixels

	my $val = ' width="' . $size;
	if ( !$flag_pix ) {
		$val .= PCENT();
	}
	$val .= DQUOTE();
	return $val;
}
##########################################################
# 15%
sub w15() {
	return width(15);
}
##########################################################
# 35%
sub w35() {
	return width(35);
}
##########################################################
# 100%
sub w100() {
	return width(100);
}
##########################################################
sub begin_html_config(;$) {
	my $full = shift @_;

	if ($full) {
		print '<table summary="top" border ' . w100() . ">\n";

		begin_row();
		print "<td>\n";
	}
	print "<form action=\"save_config.cgi\">\n";
	print wrapper( 'ui_hidden', 'idx', $in{'idx'} );
	print wrapper( 'ui_hidden', 'opt', 'general' );
	print '<table summary="changes" ' . w100() . ">\n";
	return;
}
##########################################################
sub end_html_config(;@) {
	my @columns = @_;

	print "</table>\n";
	print '<input type=submit value="' . $text{'word_save'} . '">';
	print '<input type=submit name="init" value="init">' if (@columns);
	print "</form>\n";

	begin_param(@columns);
	return;
}
##########################################################
# display parameters title
sub begin_param(@) {
	my @columns = @_;

	# test for cpu module : called with 1 parameter instead a real array
	# to have an init button
	if ( (@columns) and ( $columns[0] =~ m/^\D/ ) ) {
		print '</td>';
		end_row();

		begin_row($cb);
		print '<td>';
		print '<table summary="columns" ' . w100() . ">\n";
		titre_table( $text{'config_parameters'} );

		begin_table( 'configuration', 'border' );
		begin_row();

		# add two common columns
		push @columns, 'word_colour', 'word_subtitle_controls';

		foreach my $col (@columns) {
			my $arg = $tb;

			# depends upon icon number !
			$arg .= width( 22 * 10, 1 ) if ( $col eq 'word_subtitle_controls' );
			title_row( $text{$col}, $arg );
		}
	}
	return;
}
###############################################################################
# get new display names
# set by change_display sub (below)
sub test_display_change($) {
	my $myconfig = shift @_;

	my $new_display_id   = $in{'displayname_def'};
	my $new_display_name = $in{'gdisplay_def'};

	if ($new_display_name) {
		$myconfig->{ 'gdisplay' . $new_display_id } = $new_display_name;
	}
	return;
}
#############################################################################
# to be used with test_display_change sub
sub change_display($;$) {
	my $my_config = shift @_;
	my $no_sep    = shift @_;
	debug_sub_begin();

	titre_table( $text{'display_names'} );

	begin_table('change_display');

	#option_display_name($my_config);
	my %list = get_display_list($my_config);

	begin_row();
	print td( $text{'display_old'} );
	my @options;
	foreach my $opt ( keys %list ) {
		my @opts = ( $opt, "$list{$opt} ($opt)" );
		push @options, \@opts;
	}
	print td( wrapper( 'ui_select', 'displayname_def', 0, \@options ), w35() );
	print td( $text{'display_new'} );
	print td( wrapper( 'ui_textbox', 'gdisplay_def', q{}, 10 ) );

	# clean button
	print td('<input type=submit name="clean" value="clean">');

	end_row();
	end_table();
	separator() unless ($no_sep);

	debug_sub_end();
	return;
}
#############################################################################
# display list to change the sample rate
# work with test_sample_rate sub
sub change_sample_rate($) {
	debug_sub_begin();
	my $my_config = shift @_;

	my $sample_rate = $my_config->{'sample_rate'};

	begin_row();
	title_row( $text{'sample_rate'}, w15() );

	# sample rate from 1 (every minute) to 10 (every 10 minutes)
	my @options;
	foreach my $opt ( 1 .. 10 ) {
		push @options, $opt;
	}
	print td( wrapper( 'ui_select', 'sample_rate', $sample_rate, \@options ),
		w35() );
	end_row();

	debug_sub_end();
	return;
}
##########################################################
# default (graph_type)
##########################################################
# test/apply changes set by change_default
sub test_default_change($) {
	debug_sub_begin();
	my $my_config = shift @_;

	if ( $my_config->{'graph_type'} != $in{'graph_type'} ) {
		$my_config->{'graph_type'} = $in{'graph_type'};
		clear_cache( $my_config->{'name'} );
	}

	debug_sub_end();
	return;
}
##########################################################
# display configuration : type of graph
# to be used with test_default_change
sub change_default($;$$) {
	my $graph_type = shift @_;
	my $text1      = shift @_ || $text{'word_line'};
	my $text2      = shift @_ || $text{'word_bar'};
	debug_sub_begin();

	titre_table( $text{'default'} );
	begin_table('change_default');

	begin_row();
	print td( '<input type=radio name="graph_type" value=1 '
		  . checked($graph_type)
		  . "> $text1" );

	print td( '<input type=radio name="graph_type" value=0 '
		  . checked( !$graph_type )
		  . "> $text2" );
	end_row();

	end_table();
	debug_sub_end();
	return;
}
##########################################################
# overload global parameter in local module
##########################################################
# test one parameter (called by test_local_params)
# if the given key is different from global value
# save it in the module config
sub test_local_param($$) {
	my $my_config = shift @_;
	my $key       = shift @_;

	my $value = $in{$key};

	# an empty value reset the parameters to the global value
	if ( !$value ) {
		delete $my_config->{$key};
		return;
	}

	# test the difference with global parameter
	return if ( $value == $config{$key} );

	# test if a number
	if ( is_unsigned_real($value) ) {
		$my_config->{$key} = $value;
	}
	else {
		warning("value $value of $key is not numeric");
	}
	return;
}
##########################################################
# test module specific configuration
sub test_local_params($) {
	my $my_config = shift @_;

	test_local_param( $my_config, 'max_mod_time' );
	test_local_param( $my_config, 'alarm_time' );
	test_local_param( $my_config, 'max_log_size' );
	return;
}
##########################################################
# display a value (global or local) for parameters
# and allow to change
sub display_local_param($$) {
	my $my_config = shift @_;
	my $key       = shift @_;
	debug_sub_begin();

	my $value;
	my $type;
	if ( exists $my_config->{$key} ) {

		# local
		$type  = 'local';
		$value = $my_config->{$key};
	}
	else {

		# global
		$type  = 'global';
		$value = $config{$key};
	}

	begin_row();
	print td($key);
	print td("($type)");
	print td( wrapper( 'ui_textbox', $key, $value, 10 ) );
	end_row();

	debug_sub_end();
	return;
}
##########################################################
# display module (local) parameters for
# max_mod_time alarm_time max_log_size
# this allow to have customized parameters by module
# (different from general config)
sub change_local_params($;$) {
	my $my_config = shift @_;
	my $no_sep    = shift @_;
	debug_sub_begin();

	titre_table( $text{'exec_param'} );
	begin_table('change_local_params');

	display_local_param( $my_config, 'max_mod_time' );
	display_local_param( $my_config, 'alarm_time' );
	display_local_param( $my_config, 'max_log_size' );

	end_table();
	separator() unless ($no_sep);
	debug_sub_end();
	return;
}
##########################################################
# 	change database parameters
##########################################################
# display current values and allow change
# see also test_minmax
sub change_minmax($) {
	my $minfo = shift @_;
	debug_sub_begin();

	# get current from first rrd database
	my ( $min, $max ) = get_minmax( ( get_rrd_names($minfo) )[0] );

	begin_row();
	title_row( $text{'minimum'}, w15() );
	print td( wrapper( 'ui_textbox', 'minimum', $min, 5 ), w35() );
	end_row();

	begin_row();
	title_row( $text{'maximum'}, w15() );
	print td( wrapper( 'ui_textbox', 'maximum', $max, 5 ), w35() );
	end_row();

	debug_sub_end();
	return;
}
##########################################################
# display consolidation function
# see also test_cf
sub change_cf($) {
	debug_sub_begin();
	my $my_config = shift @_;

	my $cf = $my_config->{'cf'};

	my @cf_list = ( 'AVERAGE', 'MAX', 'MIN', 'LAST' );

	begin_row();
	title_row( $text{'cf'}, w15() );
	print td( wrapper( 'ui_select', 'cf', $cf, \@cf_list ), w35() );
	end_row();

	debug_sub_end();
	return;
}
##########################################################
# display all database parameters to be changed
# to be used with test_database_change
sub change_database($$;$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $no_sep    = shift @_;

	titre_table('database');
	begin_table('change_database');

	change_sample_rate($my_config);
	change_cf($my_config);
	change_minmax($minfo);

	end_table();
	separator() unless ($no_sep);
	debug_sub_end();
	return;
}
#############################################################################
# scale (logarithme)
#############################################################################
# test/apply changes set by change_scale sub
sub test_scale_change($$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;

	if ( defined $in{'logarithme'} ) {
		if (
			(
				    ( defined $my_config->{'logarithme'} )
				and ( $my_config->{'logarithme'} != $in{'logarithme'} )
			)
			or ( !defined $my_config->{'logarithme'} )
		  )
		{
			$my_config->{'logarithme'} = $in{'logarithme'};
			clear_cache( $my_config->{'name'} );
		}
	}

	debug_sub_end();
	return;
}
##########################################################
# change scale between normal and log
# to be used with test_scale_change
sub change_scale($$;$) {
	debug_sub_begin();
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $no_sep    = shift @_;

	titre_table( $text{'scale'} );
	begin_table('change_scale');

	begin_row;
	title_row( 'logarithmes ?', w15() );

	my $logar =
	  (       ( exists $my_config->{'logarithme'} )
		  and ( $my_config->{'logarithme'} ) );

	print td( '<input type=radio name="logarithme" value=1 '
		  . checked($logar)
		  . "> $text{'word_yes'}" );
	print td( '<input type=radio name="logarithme" value=0 '
		  . checked( !$logar )
		  . "> $text{'word_no'}" );

	end_row();

	end_table();
	separator() unless ($no_sep);

	debug_sub_end();
	return;
}
##################################################################
sub html_space(;$) {
	my $nb = shift @_ || 1;

	my $txt;
	foreach my $i ( 1 .. $nb ) {
		$txt .= "<img src=\"images/space.gif\" border=0 alt=\"space\">\n";
	}
	return $txt;
}
##################################################################
sub html_space2(;$) {
	my $nb = shift @_ || 1;

	my $txt;
	foreach my $i ( 1 .. $nb ) {
		$txt .= "<img src=\"images/space2.gif\" border=0 alt=\"space\">\n";
	}
	return $txt;
}
##########################################################
sub emptycell(;$) {
	my $nb = shift @_ || 1;

	foreach my $i ( 1 .. $nb ) {
		print td('&nbsp;');
	}
	return;
}
##########################################################
# parameter changes
##########################################################
# runstop parameter
sub display_runstop($$$) {
	my $vol_num = shift @_;
	my $runstop = shift @_;
	my $vol     = shift @_;

	my $txt;
	if ($runstop) {

		# stopped
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=runstop&$vol=$vol_num",
			'<img src="images/record16.gif" border=0 alt=record>'
		);
	}
	else {

		# run
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=runstop&$vol=$vol_num",
			'<img src="images/pause16.gif" border=0 alt=pause>'
		);
	}
	$txt .= html_space();
	return $txt;
}
##########################################################
# display parameter
sub display_display($$$) {
	my $vol_num = shift @_;
	my $display = shift @_;
	my $vol     = shift @_;

	my $txt;
	if ( is_param_display($display) ) {

		# the display is now a number (not an icon) to allow different display
		# for the same module
		$txt .=
		  html_link( "edit_module.cgi?idx=$in{'idx'}&opt=display&$vol=$vol_num",
			$display );
	}
	else {

		# we do not show the value -1 but an icon here
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=display&$vol=$vol_num",
			'<img src="images/showdata16.gif" border=0 alt=showdata>'
		);
	}
	$txt .= html_space();
	return $txt;
}
##########################################################
# move up in list
sub display_up($$) {
	my $vol_num = shift @_;
	my $vol     = shift @_;

	my $txt;
	if ( $vol_num > 1 ) {

		# top : quick move to first place
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=top&$vol=$vol_num",
			'<img src="images/top16.gif" border=0 alt=top>'
		);

		# up : move one place up
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=up&$vol=$vol_num",
			'<img src="images/up16.gif" border=0 alt=up>'
		);
	}
	else {
		$txt .= html_space2(2);
	}
	$txt .= html_space();
	return $txt;
}
##########################################################
# move down in list
# display icons to move down the volume
sub display_down($$$) {
	my $vol_num   = shift @_;
	my $my_config = shift @_;
	my $vol       = shift @_;

	my $txt;

	# warning : be careful between volume key and database id
	# in case of delete, we can have : vol27=28,...
	my $max_vol = get_max_vol($my_config);
	if ( $vol_num < $max_vol ) {

		# down : move one place down
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=down&$vol=$vol_num",
			'<img src="images/down16.gif" border=0 alt=down>'
		);

		# bottom : quick move to last place
		$txt .= html_link(
			"edit_module.cgi?idx=$in{'idx'}&opt=bottom&$vol=$vol_num",
			'<img src="images/bottom16.gif" border=0 alt=bottom>'
		);
	}
	else {
		$txt .= html_space2(2);
	}
	$txt .= html_space();
	return $txt;
}
##########################################################
# to delete a parameter
sub display_trash($$) {
	my $vol_num = shift @_;
	my $vol     = shift @_;

	return html_link(
		"edit_module.cgi?idx=$in{'idx'}&opt=rm&$vol=$vol_num",
		'<img src="images/trash16.gif" border=0 alt=trash>'
	);
}
##########################################################
# an icon to be able to dump the rrd database
sub display_dump_database($$$) {
	my $vol_num   = shift @_;
	my $my_config = shift @_;
	my $prefx     = shift @_;

	# get database name
	my $id = is_vol( $prefx . $vol_num, $my_config->{ $prefx . $vol_num } );

	return html_link(
		"show_database.cgi?idx=$in{'idx'}&vol=$id",
		'<img src="images/debug16.gif" border=0 alt="debug">'
	);
}
##########################################################
# display all parameter changes
sub display_parameter_line($$$$$) {
	my $vol_num   = shift @_;
	my $runstop   = shift @_;
	my $display   = shift @_;
	my $my_config = shift @_;
	my $prefx     = shift @_ || 'vol';
	my $vol       = $prefx . '_num';

	my $txt;
	debug_sub_begin();
	if ( $access{'configure'} ) {
		$txt =
		    display_up( $vol_num, $vol )
		  . display_down( $vol_num, $my_config, $vol )
		  . display_runstop( $vol_num, $runstop, $vol )
		  . display_display( $vol_num, $display, $vol )
		  . display_trash( $vol_num, $vol )
		  . display_dump_database( $vol_num, $my_config, $prefx );
	}
	debug_sub_end();
	return $txt;
}
##########################################################
# generic sub to change a parameter line
# be careful : config line must have a "standard" format :
# $vol_id, $vol_name, $vol_colour, $runstop, $display
sub mod_config_opt($$$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;
	my $pre       = shift @_;

	debug_sub_begin();
	my $vol = $pre . '_num';

	#my $vol = shift @_ || 'vol_num';

	## no critic (ProhibitCascadingIfElse)
	if ( $in{'opt'} ) {
		if ( !exists $in{$vol} ) {
			warning( 'problem on ' . $in{'opt'} . ' with vol = ' . $vol );
			return;
		}
		if ( $in{'opt'} eq 'up' ) {

			# cf display_up
			exchange(
				\$my_config->{ $pre . $in{$vol} },
				\$my_config->{ $pre . ( $in{$vol} - 1 ) }
			);
		}
		elsif ( $in{'opt'} eq 'top' ) {

			# cf display_up
			my $vol_num = $in{$vol};
			my $line    = $my_config->{ $pre . $vol_num };
			my $num     = $vol_num;
			while ( defined $my_config->{ $pre . ( $num - 1 ) } ) {
				$my_config->{ $pre . $num } =
				  $my_config->{ $pre . ( $num - 1 ) };
				$num--;
			}
			$my_config->{ $pre . $num } = $line;
		}
		elsif ( $in{'opt'} eq 'down' ) {

			# cf display_down
			exchange(
				\$my_config->{ $pre . $in{$vol} },
				\$my_config->{ $pre . ( $in{$vol} + 1 ) }
			);
		}
		elsif ( $in{'opt'} eq 'bottom' ) {

			# cf display_down
			my $vol_num = $in{$vol};
			my $line    = $my_config->{ $pre . $vol_num };

			my $num = $vol_num;
			while ( defined $my_config->{ $pre . ( $num + 1 ) } ) {
				$my_config->{ $pre . $num } =
				  $my_config->{ $pre . ( $num + 1 ) };
				$num++;
			}
			$my_config->{ $pre . $num } = $line;
		}
		elsif ( $in{'opt'} eq 'runstop' ) {

			# cf display_runstop
			my $line = $my_config->{ $pre . $in{$vol} };
			$my_config->{ $pre . $in{$vol} } = toggle_runstop($line);
		}
		elsif ( $in{'opt'} eq 'display' ) {

			# cf display_display
			my $line = $my_config->{ $pre . $in{$vol} };
			$my_config->{ $pre . $in{$vol} } = toggle_display($line);
		}
		elsif ( $in{'opt'} eq 'rm' ) {

			# cf display_trash
			my $vol_num = $in{$vol};

			remove_vol( $my_config, $minfo->{'dbtables'}, $pre . $vol_num );

			# compute new db_uid
			my @rrd_list = get_rrd_names($minfo);
			check_db_uid( $my_config, @rrd_list );
		}

		write_config( $minfo, $my_config );
	}
	## use critic
	debug_sub_end();
	return;
}
###############################################################################
# colour and line
###############################################################################
sub display_colour($) {
	my $vol_colour = shift @_;

	# color
	my $txt =
	    $text{ 'colour_' . $vol_colour }
	  ? $text{ 'colour_' . $vol_colour }
	  : "<pre>#$vol_colour </pre>";
	print td( html_color( "#$vol_colour", $txt ) );
	return;
}
###############################################################################
# option_colourselect(name,current_value, default_value, [help])
# if help is provided, displays help link
sub option_colourselect($$$) {
	my $name      = $_[0];       # color name
	my $v         = lc $_[1];    # current color
	my $v_default = $_[2];       # default color

	debug_sub_begin();

	html_comment('begin option_colourselect');

	# display selected color
	print td( "#$v", width(3) . " bgcolor=\"#$v\"" ) if ($v);

	$v = 'default' if ( $v eq $v_default );

	print '<td>';
	print "<strong>\n";

	print '<select name="' . $name . '1_def">';

	# print default value
	if ( defined $v_default ) {
		printf "<option value='default' %s>$text{'word_default'}</option>\n",
		  ( $v eq 'default' ) ? 'selected' : $EMPTY;
	}
	else {
		printf "<option value='nocolour' %s>$text{'word_select'}</option>\n",
		  ( $v eq 'nocolour' ) || ( $v eq $EMPTY ) ? 'selected' : $EMPTY;
	}

	# print list of proposed colours
	my @colors = generate_colors();
	my $trouve = 0;
	foreach my $color (@colors) {
		my $text_color =
		  ( $text{ 'colour_' . $color } )
		  ? $text{ 'colour_' . $color }
		  : $color;
		if ( $v eq $color ) {
			$trouve = 1;
			print "<option value=\"$color\" selected>$text_color</option>\n";
		}
		else {
			print "<option value=\"$color\" >$text_color</option>\n";
		}
	}
	print '</select></strong>';

	# and an input for a free color in rgb : ff0000 == red
	if ( ($trouve) or ( $v eq 'default' ) ) {
		print ' #' . wrapper( 'ui_textbox', $name . '2_def', q{}, 8 );
	}
	else {
		print ' #' . wrapper( 'ui_textbox', $name . '2_def', $v, 8 );
	}

	print "</td>\n";
	html_comment('end option_colourselect');
	debug_sub_end();
	return;
}
###############################################################################
# get color from the 2 entries
sub get_colourselect($$) {
	my $name     = shift @_;
	my $defaults = shift @_;

	my $color;
	if ( $in{ $name . '2_def' } ) {

		# numerical entry
		$color = $in{ $name . '2_def' };
	}
	elsif ( $in{ $name . '1_def' } eq 'default' ) {

		# default color
		$color = $defaults;
	}
	else {

		# pre-defined entry
		$color = $in{ $name . '1_def' };
	}

	#debug("get_colourselect name=$name color=$color");
	return $color;
}
###############################################################################
sub option_lineselect($$) {
	my $name = shift @_;
	my $line = shift @_ || $defline;    # current value

	my @line_list = ( 'LINE1', 'LINE2', 'LINE3' );
	print td( wrapper( 'ui_select', $name . '_linedef', $line, \@line_list ) );

	return;
}
###############################################################################
sub get_lineselect($) {
	my $name = shift @_;
	return $in{ $name . '_linedef' };
}
##########################################################
# when editing parameters
##########################################################
# edit a parameter display
sub radio_display($$$) {
	my $display   = shift @_;    # current display
	my $my_config = shift @_;
	my $name      = shift @_;

	debug_sub_begin();

	# display ?
	title_row( $text{'word_display_data'} );
	my %list = get_display_list($my_config);
	my @options;
	my $i = -1;                  # stop
	while ( $i <= $config{'max_display'} ) {

		my $value = ( exists $list{$i} ) ? $list{$i} : $i;
		my @opts = ( $i, $value );
		push @options, \@opts;
		$i++;
	}
	print td( wrapper( 'ui_select', $name, $display, \@options ) );

	debug_sub_end();
	return;
}
##########################################################
# edit a parameter runstop
sub radio_runstop($$) {
	my $runstop   = shift @_;    # current runstop
	my $my_config = shift @_;

	debug_sub_begin();

	# runstop ?
	title_row( $text{'word_pause_datacapture'} );
	print td( '<input type=radio name="runstop_def" value=1 '
		  . checked($runstop)
		  . "> $text{'word_yes'}" );
	print td( '<input type=radio name="runstop_def" value=0 '
		  . checked( !$runstop )
		  . "> $text{'word_no'}" );

	debug_sub_end();
	return;
}
##########################################################
#sub radio_display_runstop($$$) {
#	my $display   = shift @_;    # current display
#	my $runstop   = shift @_;    # current runstop
#	my $my_config = shift @_;
#
#	debug_sub_begin();
#
#	# display ?
#	begin_row();
#	radio_display( $display, $my_config, 'display_def' );
#	end_row();
#
#	# runstop ?
#	begin_row();
#	radio_runstop( $runstop, $my_config );
#	end_row();
#
#	debug_sub_end();
#	return;
#}
##########################################################
sub modify_display_runstop($$$) {
	my $display   = shift @_;
	my $runstop   = shift @_;
	my $my_config = shift @_;

	# display ?
	begin_row();
	radio_display( $display, $my_config, 'display_def' );
	end_row();

	# runstop ?
	begin_row();
	radio_runstop( $runstop, $my_config );
	end_row();
	return;
}
##########################################################
sub modify_colour($$$$$) {
	my $vol_config = shift @_;
	my $display    = shift @_;
	my $runstop    = shift @_;
	my $my_config  = shift @_;
	my $prefx      = shift @_ || 'vol';

	my ( $vol_colour, $vol_line ) = get_line_config($vol_config);

	begin_row();
	title_row( $text{'word_colour'} );
	option_colourselect( $prefx . '_colour', $vol_colour, $EMPTY );
	option_lineselect( $prefx . '_line', $vol_line );
	end_row();

	modify_display_runstop( $display, $runstop, $my_config );
	return;
}
##########################################################
# general web sub
##########################################################
# used by debug modules or log.cgi
sub display_html($) {
	my $ligne = shift @_;

	if ( $ligne =~ m/warning/i ) {
		print html_color( 'red', $ligne ) . '<br>' . "\n";
	}
	else {
		print "$ligne<br>\n";
	}
	return;
}
##########################################################
# to display a button, which call navigation.cgi
sub aff_navigation_button($$$$$$) {
	my $mod_list     = shift @_;    # module list
	my $ind_wanted   = shift @_;    # module indice to point to
	my $ind_current  = shift @_;    # current module indice
	my $text         = shift @_;    # text to display in button
	my $display_type = shift @_;    # form action to be called
	my $options      = shift @_;

	print '<td>' . wrapper( 'ui_form_start', 'navigation.cgi' );

	my $mod         = $mod_list->[$ind_wanted];
	my $module_name = $mod->{'module_name'};
	if ( $ind_wanted == $ind_current ) {

		print wrapper( 'ui_form_end',
			[ [ 'undef', "$text:$module_name", undef, 1 ] ] );
	}
	else {

		print wrapper( 'ui_hidden', 'cgi', $display_type );
		print wrapper( 'ui_hidden', 'idx', $module_name );
		print wrapper( 'ui_hidden', 'opt', $options );

		print wrapper( 'ui_form_end', [ [ 'undef', "$text:$module_name" ] ] );
	}
	print "</td>\n";

	return;
}
##########################################################
# used in display and config for quick navigation from module to another
sub navigation($$$$) {
	my $current_module = shift @_;          # cpu, mem
	my $display_type   = shift @_;          # display, navigation ...
	my $options        = shift @_ || q{};
	my $filter         = shift @_;

	my @mod_list = get_mod_list($filter);

	# search previous and next indices
	my $ind_current = 0;
	my $ind_previous;
	my $ind_next;
  RECH: foreach my $mod (@mod_list) {
		my $module_name = $mod->{'module_name'};

		if ( $module_name eq $current_module ) {
			$ind_previous = ( $ind_current > 0 ) ? $ind_current - 1 : 0;
			$ind_next =
			  ( $ind_current < $#mod_list ) ? $ind_current + 1 : $#mod_list;
			last RECH;
		}
		$ind_current++;
	}

	print '<table summary="navigation">';
	begin_row();
	print td( $text{'display_navigation'} );

	# first button
	aff_navigation_button( \@mod_list, 0, $ind_current, '|<', $display_type,
		$options );

	# previous button
	aff_navigation_button( \@mod_list, $ind_previous, $ind_current, '<',
		$display_type, $options );

	# select form to access all others module
	print '<td><form action="navigation.cgi">';
	print wrapper( 'ui_hidden', 'cgi', $display_type );
	print wrapper( 'ui_hidden', 'opt', $options );
	print '<input type=submit value="' . $text{'goto'} . '">';

	my @options;
	foreach my $mod (@mod_list) {
		my $module_name = $mod->{'module_name'};
		push @options, $module_name;
	}
	print wrapper( 'ui_select', 'idx', $current_module, \@options );
	print "</form></td>\n";

	# button for next module
	aff_navigation_button( \@mod_list, $ind_next, $ind_current, '>',
		$display_type, $options );

	# last button
	aff_navigation_button( \@mod_list, $#mod_list, $ind_current, '>|',
		$display_type, $options );

	end_row();
	print "</table>\n";

	return;
}
##########################################################
# send all "custom" parameters for graph
sub add_graph_arg() {

	foreach my $var ( 'idx', 'opt', 'gtype', 'display' ) {
		print wrapper( 'ui_hidden', $var, $in{$var} )
		  if ( exists $in{$var} );
	}
	return;
}
##########################################################
# from to select a period
# used by display_custom and display_all
sub select_period_form($;$$) {
	my $from     = shift @_;
	my $minfo    = shift @_;
	my $flag_end = shift @_;

	print '<form action="' . basename($from) . "\">\n";
	add_graph_arg();

	# force to clear cache
	print wrapper( 'ui_hidden', 'nocache', 1 );

	print '<table summary="config" border=0>';    # configuration table

	# first row : time
	begin_row();
	print td( $text{'period'}, w15() );

	# number
	my @options = ( 1 .. 31 );
	print td( wrapper( 'ui_select', 'custom', $in{'custom'}, \@options ),
		w15() );

	# period type : hour day week month year
	@options = ();
	foreach my $period ( list_periods() ) {
		my @opts = ( $period, $text{$period} );
		push @options, \@opts;
	}
	print td( wrapper( 'ui_select', 'periode', $in{'periode'}, \@options ) );
	end_row();

	if ($minfo) {

		if ($flag_end) {

			# end of graph (time) : will be displayed as yyyymmddhhmmss
			begin_row();
			my $end =
			  $minfo->{'custom_end'}; # end is negative in the past, 0 means now
			my $date_end = strftime( '%Y%m%d%H%M', localtime( time + $end ) );
			print td('date end');
			print td( wrapper( 'ui_checkbox', 'custom_now', 1, 'now' ) );

			print td('(yyyymmddhhmm)');
			print td( wrapper( 'ui_textbox', 'custom_datend', $date_end, 12 ) );
			end_row();
		}

		# second row : min /max on y axis
		begin_row();
		print td('y scale');
		print td( wrapper( 'ui_checkbox', 'custom_ydef', 1, 'default' ) );

		# min
		my $ymin = $minfo->{'custom_ymin'} || 0;
		print td('y min');
		print td( wrapper( 'ui_textbox', 'custom_ymin', $ymin, 10 ) );

		# max
		my $ymax = $minfo->{'custom_ymax'} || $EMPTY;
		print td('y max');
		print td( wrapper( 'ui_textbox', 'custom_ymax', $ymax, 10 ) );
		end_row();
	}

	# third row : buttons
	# display
	begin_row();
	print td("<input type=submit value=\"$text{'display'}\">");

	# save
	print td("<input type=submit name=\"save\" value=\"$text{'word_save'}\">");

	# reset
	print td(
		"<input type=submit name =\"reset\" value=\"$text{'word_reset'}\">");
	end_row();

	print "</table>\n";    # config
	print '</form>';
	return;
}
##########################################################
# display a guess checkbox
sub guess() {

	print td( wrapper( 'ui_checkbox', 'guess', 1, 'guess' ) );
	return;
}
###############################################################################
# used by my_footer
sub add_icon($$$$$$) {
	my $r_links  = shift @_;
	my $r_titles = shift @_;
	my $r_icons  = shift @_;
	my $link     = shift @_;
	my $title    = shift @_;
	my $icon     = shift @_;

	push @{$r_links},  $link;
	push @{$r_titles}, $title;
	push @{$r_icons},  $icon;

	return;
}
###############################################################################
# display common links for all screen
# one general for all screens
# one dedicated for a module
sub my_footer(;$) {
	my $idx = shift @_;

	# commons
	my ( @links, @titles, @icons );

	# for a module
	my ( @links_m, @titles_m, @icons_m );

	# commons icons
	# main
	add_icon( \@links, \@titles, \@icons, 'index.cgi',
		"webminstats $text{'index'}",
		'images/gohome.gif' );

	# panorama
	add_icon( \@links, \@titles, \@icons, 'display_all.cgi',
		$text{'all_modules'}, 'images/all.gif' );

	# log
	add_icon( \@links, \@titles, \@icons, 'log.cgi', $text{'log'},
		'images/log.gif' );

	# debug common
	add_icon( \@links, \@titles, \@icons, 'debug.cgi', $text{'debug'},
		'images/debug.gif' );

	# release notes
	add_icon( \@links, \@titles, \@icons, 'about.cgi', $text{'about'},
		'images/about.gif' );

	# icons for modules
	if ($idx) {

		# display
		add_icon( \@links_m, \@titles_m, \@icons_m, "display.cgi?idx=$idx",
			$text{'display'}, 'images/display.gif' );

		# snapshot
		add_icon( \@links_m, \@titles_m, \@icons_m, "snapshot.cgi?idx=$idx",
			$text{'module_snapshot'},
			'images/snapshot.gif' );

		# debug module
		add_icon( \@links_m, \@titles_m, \@icons_m, "debug_module.cgi?idx=$idx",
			$text{'module_debug'}, 'images/debug.gif' );
	}

	if ( $access{'configure'} ) {

		# config all
		add_icon( \@links, \@titles, \@icons, 'config.cgi',
			$text{'title_configure_modules'},
			'images/settings.gif' );

		# check all config
		add_icon( \@links, \@titles, \@icons, 'check_config.cgi',
			$text{'check_config'}, 'images/check_config.gif' );

		if ($idx) {

			# config module
			add_icon( \@links_m, \@titles_m, \@icons_m,
				"edit_module.cgi?idx=$idx", $text{'module_configure'},
				'images/config.gif' );

			# action
			add_icon( \@links_m, \@titles_m, \@icons_m,
				"config_action.cgi?idx=$idx", $text{'module_action_config'},
				'images/run.gif' );

			# check config
			add_icon( \@links_m, \@titles_m, \@icons_m,
				"check_config.cgi?idx=$idx", $text{'module_check_config'},
				'images/check_config.gif' );

			# clone if not already a clone
			add_icon( \@links_m, \@titles_m, \@icons_m,
				"clone_mod.cgi?idx=$idx", $text{'word_clone'},
				'images/clonedata.gif' )
			  if ( !is_clone($idx) );
		}
		else {

			# clone
			add_icon( \@links, \@titles, \@icons, 'clone_mod.cgi',
				$text{'word_clone'}, 'images/clonedata.gif' );
		}
	}

	print ui_hr();
	print ui_subheading( $text{'menu_global'} );
	my_icons_table( \@links, \@titles, \@icons, $config{'icon_by_row'} );
	if ($idx) {
		print ui_hr();
		print ui_subheading( $text{'menu_module'} );
		my_icons_table( \@links_m, \@titles_m, \@icons_m,
			$config{'icon_by_row'} );
	}

	# link to return to main module page
	my ( undef, $filename, undef ) = caller;
	wrapper( 'ui_print_footer', 'index.cgi', $text{'index'} )
	  unless ( $filename =~ m/index[.]cgi$/ );
	return;
}
###############################################################################
#       alarm  	ok
# stop	orange	black
# run 	red	green
sub alarm_color($$) {
	my $status   = shift @_;
	my $is_alarm = shift @_;

	my $color;
	if ( $status eq 'stopped' ) {

		#			orange		black
		$color = ($is_alarm) ? COLOR_WARN() : COLOR_OFF();
	}
	else {

		#			red		green
		$color = ($is_alarm) ? COLOR_ALARM() : COLOR_OK();
	}
	return $color;
}
###############################################################################
sub display_module_version($) {
	my $version = shift @_;

	if ( is_beta_version($version) ) {
		return html_color( COLOR_BETA(), $version );
	}
	elsif ( is_stable_version($version) ) {
		return html_color( COLOR_OK(), $version );
	}
	else {

		# not stables releases
		return "$version";
	}
}
###############################################################################
# is used in config.cgi, edit_module.cgi to display information
# on actions about a module
sub display_module_action($$) {
	my $mod       = shift @_;
	my $my_config = shift @_;

	my $module     = $mod->{'idx'};
	my $config_dir = $mod->{'config_dir'};
	my ( $color, $msg );
	my $is_action = is_action($my_config);
	if ($is_action) {
		my $is_cur_action = is_cur_action($config_dir);
		if ($is_cur_action) {
			$color = COLOR_ALARM();
			$msg   = $is_cur_action;
		}
		else {
			$color = COLOR_OK();
			$msg   = $is_action;
		}
	}
	else {
		$color = COLOR_OFF();
		$msg   = $text{'word_no'};
	}
	return html_link( 'config_action.cgi?idx=' . $module,
		html_color( $color, $msg ) );
}
###############################################################################
# prepare a message with info on a module :
# version
# state (run/stop)
# alarms
sub display_module_info($$) {
	my $minfo     = shift @_;
	my $my_config = shift @_;

	my $module      = $minfo->{'module_name'};
	my $ver         = $minfo->{'module_ver'};
	my $status      = runstop_status( $minfo, $my_config );
	my $is_alarm    = test_perm_alarm( $minfo->{'config_dir'} );
	my $is_clone    = is_clone($module);
	my $module_name = $module;
	if ($is_clone) {
		$module_name .= " ($text{'clone_of'} $is_clone)";
	}

	my $color = alarm_color( $status, $is_alarm );

	my $msg  = "module $module_name " . display_module_version($ver) . ' : ';
	my $msg2 = "$status ( ";
	if ($is_alarm) {
		$msg2 .= 'alarm : '
		  . html_link( "clear_alarm.cgi?idx=$module",
			get_alarm_msg( $minfo->{'config_dir'} ) );
	}
	else {
		$msg2 .= $text{'word_ok'};
	}
	$msg2 .= ' )';

	return $msg . html_color( $color, $msg2 );
}
###############################################################################
# to be used by display_* for an automatic refresh (every 60 seconds)
sub meta_refresh() {
	return '<meta http-equiv=Refresh content=60>';
}
###############################################################################
sub get_meta() {

	# add html pragma to avoid cache in browsers
	# http://www.i18nguy.com/markup/metatags.html
	my $meta = '<meta http-equiv="Cache-Control" content="no-cache" />';
	$meta .= '<meta http-equiv="Pragma" content="no-cache" />';
	$meta .= '<meta http-equiv="Expires" content="0" />';

	# refresh or not refresh ?
	if ( !exists $in{'norefresh'} ) {
		$meta .= meta_refresh();
	}
	return $meta;
}
###############################################################################
# print an html title
sub print_title($) {
	my $text = shift @_;

	print html_tag( 'h1', $text );
	return;
}
###############################################################################
sub print_button($$$$$) {
	my $pre = shift @_;    # parameter prefix : vol
	my $idx = shift @_;    # module name : cpu/amavis
	my $opt = shift @_;    # action name : new_vol
	my $vol = shift @_;    # parameter (vol) index
	my $msg = shift @_;    # button text

	begin_row();
	print '<td align=left>';
	print wrapper( 'ui_hidden', 'idx', $idx );
	print wrapper( 'ui_hidden', 'opt', $opt );
	print wrapper( 'ui_hidden', $pre,  $vol );
	print "<input type=submit value=\"$msg\">";
	print '</td>';
	end_row();

	return;
}
###############################################################################
sub print_input($$$$$$;$) {
	my $title        = shift @_;    # text title before input
	my $name         = shift @_;    # input name
	my $value        = shift @_;    # value to display
	my $flag_chooser = shift @_;    # if true, display file chooser
	my $flag_guess   = shift @_;    # if true, display guess button
	my $flag_tr      = shift @_;    # if true, print html row delimiters
	my $type         = shift @_
	  || 'text';    # text by default, but password is needed by mysql

	begin_row() if ($flag_tr);
	title_row($title);
	print
"<td><input type=$type name=\"$name\" size=32 maxlength=128 value=\"$value\">";
	if ($flag_chooser) {
		print wrapper( 'file_chooser_button', $name, 0 );
	}
	print '</td>';
	if ($flag_guess) {

		# rem : guess contains td
		guess();
	}
	end_row() if ($flag_tr);

	return;
}
###############################################################################
sub is_invalid_entry($) {
	my $value = shift @_;

	# bad entry from forms are
	# 1 : empty text fields
	# 2 : from select, the first field ('select')
	return ( ( !defined $value ) || ( $value eq $EMPTY ) || ( $value eq '0' ) );
}
###############################################################################
sub is_invalid_colour() {
	return (  ( $in{'vol_colour1_def'} eq 'nocolour' )
		  and ( $in{'vol_colour2_def'} eq $EMPTY ) );
}
######################################################################################
# get cgi parameters and build cgi line
sub get_opts($$) {
	my $exclude = shift @_;    # parameter to be excluded
	my $opts    = shift @_;    # array of cgi parameters

	# others options
	my $opt = q{};

	foreach my $o ( @{$opts} ) {
		next if ( $o eq $exclude );
		$opt .= "&$o=$in{$o}" if ( exists $in{$o} );
	}
	return $opt;
}
###############################################################################
# build an html hyperlink
sub html_link($$;$) {
	my $target = shift @_;
	my $text   = shift @_;
	my $title  = shift @_;

	my $titre = ($title) ? ' title="' . $title . q{"} : q{};

	return "<a href=\"$target\" $titre>$text</a>";
}
###############################################################################
# dipslay text with color (old style, no css)
sub html_color($$) {
	my $color = shift @_;
	my $text  = shift @_;

	return html_tag( 'font', $text, "color='$color'" );
}
###############################################################################
# another display without css
sub html_strong($) {
	my $text = shift @_;

	return html_tag( 'strong', $text );
}
###############################################################################
# called to display menus, depending upon webminstats config (icons)
# - can be icons
# - can be links
sub my_icons_table($$$) {

	if ( ( exists $config{'icons'} ) and ( !$config{'icons'} ) ) {

		# show text links : force using my own code instead icons_table
		# because some theme (authentic-theme) does not use tconfig{'noicons'}
		my $sav = $tconfig{'noicons'};
		$tconfig{'noicons'} = 1;
		graph_table(@_);
		$tconfig{'noicons'} = $sav;
	}
	else {

		# show icons
		wrapper( 'icons_table', @_ );
	}
	return;
}
###############################################################################
# a clone of generate_icon (webmin) to have display_all work with authentic-theme
# generate_graph(image, title, link, [href], [width], [height], [before-title], [after-title])
# Prints HTML for an icon image. The parameters are :
# image - URL for the image, like images/foo.gif
# title - Text to appear under the icon
# link - Optional destination for the icon's link
# href - Other HTML attributes to be added to the <a href> for the link
# width - Optional width of the icon
# height - Optional height of the icon
# before-title - HTML to appear before the title link, but which is not actually in the link
# after-title - HTML to appear after the title link, but which is not actually in the link

sub generate_graph($$$;$$$$$) {

	#     0       1       2      3       4      5         6       7
	my ( $image, $title, $link, $hrefs, $width, $height, $before, $after ) = @_;

	load_theme_library();
	if ( defined &theme_generate_graph ) {
		theme_generate_graph(@_);
		return;
	}

	# icons default size
	my $def_size = 48;

	my $href = clean_var($hrefs);

	$width  = $def_size if !defined $width;
	$height = $def_size if !defined $height;

	my $before_title = clean_var($before);
	my $after_title  = clean_var($after);
	print '<div>';

	if ( $tconfig{'noicons'} ) {

		# to have some space around text links
		print "<table border><tr><td width=$width height=$height>\n",
		  "</td></tr></table>\n";
		if ($link) {
			print
			  "$before_title<a href=\"$link\" $href>$title</a>$after_title\n";
		}
		else {
			print "$before_title$title$after_title\n";
		}
	}
	elsif ($link) {

		#print "<table border><tr><td width=$def_size height=$def_size>\n",
		print "<table border><tr><td>\n",
		  "<a href=\"$link\" $href><img src=\"$image\" alt=\"\" border=0 ",
		  "width=$width height=$height></a></td></tr></table>\n";
		print "$before_title<a href=\"$link\" $href>$title</a>$after_title\n";
	}
	else {
		#print "<table border><tr><td width=$def_size height=$def_size>\n",
		print "<table border><tr><td>\n",
		  "<img src=\"$image\" alt=\"\" border=0 width=$width height=$height>",
		  "</td></tr></table>\n";
		print "$before_title$title$after_title\n";
	}
	print '</div>';
	return;
}
###############################################################################
# a clone of icons_table (webmin) to have display_all work with authentic-theme
# graph_table(&links, &titles, &icons, [columns], [href], [width], [height], &befores, &afters)
# Renders a column table of icons. The useful parameters are :
# links - An array ref of link destination URLs for the icons.
# titles - An array ref of titles to appear under the icons.
# icons - An array ref of URLs for icon images.
# columns - Number of columns to layout the icons with. Defaults to 4.

## no critic (ProhibitManyArgs)
sub graph_table($$$;$$$$$$) {

	#          0       1      2          3        4
	my (
		$links, $titles, $icons, $columns, $href,

		#  5        6        7       8
		$width, $height, $before, $after
	) = @_;

	load_theme_library();
	if ( defined &theme_graph_table ) {
		theme_graph_table(@_);
		return;
	}
	my $need_tr;
	my $cols = $columns ? $columns : 4;
	my $per = int( 100.0 / $cols );
	print "<table class='graph_table' width=100% cellpadding=5>\n";
	## no critic (ProhibitCStyleForLoops)
	my $i = 0;
	for ( $i = 0 ; $i < @{ $_[0] } ; $i++ ) {
		if ( $i % $cols == 0 ) { print "<tr>\n"; }
		print "<td width=$per% align=center valign=top>\n";
		generate_graph(
			$icons->[$i],     # 0
			$titles->[$i],    # 1
			$links->[$i],     # 2
			ref($href) ? $href->[$i] : $href,    # 3
			$width,                              # 4
			$height,                             # 5
			$before->[$i],                       # 6
			$after->[$i],                        # 7
		);
		print "</td>\n";
		if ( $i % $cols == $cols - 1 ) { print "</tr>\n"; }
	}
	while ( $i++ % $cols ) {
		print "<td width=$per%></td>\n";
		$need_tr++;
	}
	print "</tr>\n" if ($need_tr);
	print "</table>\n";
	return;
}
## use critic
###############################################################################
1;
